import { getStore } from '@boldpenguin/sdk-redux';
import { createStore } from '@stencil/store';
class ReduxStoreService {
  constructor() {
    this.mapDispatchToProps = (component, props) => {
      Object.keys(props).forEach((actionName) => {
        const action = props[actionName];
        Object.defineProperty(component, actionName, {
          get: () => (...args) => this.store.dispatch(action(...args)),
          configurable: true,
          enumerable: true,
        });
      });
    };
    this.mapStateToProps = (component, mapState) => {
      // TODO: Don't listen for each component
      const _mapStateToProps = (_component, _mapState) => {
        const mergeProps = mapState(this.store.getState());
        Object.keys(mergeProps).forEach((newPropName) => {
          const newPropValue = mergeProps[newPropName];
          component[newPropName] = newPropValue;
          // TODO: can we define new props and still have change detection work?
        });
      };
      const unsubscribe = this.store.subscribe(() => _mapStateToProps(component, mapState));
      _mapStateToProps(component, mapState);
      return unsubscribe;
    };
    const store = getStore();
    const { state } = createStore(store.getState());
    this.state = state;
    this.store = store;
    this.store.subscribe(() => {
      const updated = this.store.getState();
      Object.keys(updated).forEach((key) => {
        this.state[key] = updated[key];
      });
    });
  }
}
export default new ReduxStoreService();
