'use strict';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

/**
 * Adapted from React: https://github.com/facebook/react/blob/master/packages/shared/formatProdErrorMessage.js
 *
 * Do not require this module directly! Use normal throw error calls. These messages will be replaced with error codes
 * during build.
 * @param {number} code
 */
function formatProdErrorMessage(code) {
  return "Minified Redux error #" + code + "; visit https://redux.js.org/Errors?code=" + code + " for the full message or " + 'use the non-minified dev environment for full errors. ';
}

// Inlined version of the `symbol-observable` polyfill
var $$observable = (function () {
  return typeof Symbol === 'function' && Symbol.observable || '@@observable';
})();

/**
 * These are private action types reserved by Redux.
 * For any unknown actions, you must return the current state.
 * If the current state is undefined, you must return the initial state.
 * Do not reference these action types directly in your code.
 */
var randomString = function randomString() {
  return Math.random().toString(36).substring(7).split('').join('.');
};

var ActionTypes = {
  INIT: "@@redux/INIT" + randomString(),
  REPLACE: "@@redux/REPLACE" + randomString(),
  PROBE_UNKNOWN_ACTION: function PROBE_UNKNOWN_ACTION() {
    return "@@redux/PROBE_UNKNOWN_ACTION" + randomString();
  }
};

/**
 * @param {any} obj The object to inspect.
 * @returns {boolean} True if the argument appears to be a plain object.
 */
function isPlainObject(obj) {
  if (typeof obj !== 'object' || obj === null) return false;
  var proto = obj;

  while (Object.getPrototypeOf(proto) !== null) {
    proto = Object.getPrototypeOf(proto);
  }

  return Object.getPrototypeOf(obj) === proto;
}

/**
 * @deprecated
 *
 * **We recommend using the `configureStore` method
 * of the `@reduxjs/toolkit` package**, which replaces `createStore`.
 *
 * Redux Toolkit is our recommended approach for writing Redux logic today,
 * including store setup, reducers, data fetching, and more.
 *
 * **For more details, please read this Redux docs page:**
 * **https://redux.js.org/introduction/why-rtk-is-redux-today**
 *
 * `configureStore` from Redux Toolkit is an improved version of `createStore` that
 * simplifies setup and helps avoid common bugs.
 *
 * You should not be using the `redux` core package by itself today, except for learning purposes.
 * The `createStore` method from the core `redux` package will not be removed, but we encourage
 * all users to migrate to using Redux Toolkit for all Redux code.
 *
 * If you want to use `createStore` without this visual deprecation warning, use
 * the `legacy_createStore` import instead:
 *
 * `import { legacy_createStore as createStore} from 'redux'`
 *
 */

function createStore(reducer, preloadedState, enhancer) {
  var _ref2;

  if (typeof preloadedState === 'function' && typeof enhancer === 'function' || typeof enhancer === 'function' && typeof arguments[3] === 'function') {
    throw new Error(formatProdErrorMessage(0) );
  }

  if (typeof preloadedState === 'function' && typeof enhancer === 'undefined') {
    enhancer = preloadedState;
    preloadedState = undefined;
  }

  if (typeof enhancer !== 'undefined') {
    if (typeof enhancer !== 'function') {
      throw new Error(formatProdErrorMessage(1) );
    }

    return enhancer(createStore)(reducer, preloadedState);
  }

  if (typeof reducer !== 'function') {
    throw new Error(formatProdErrorMessage(2) );
  }

  var currentReducer = reducer;
  var currentState = preloadedState;
  var currentListeners = [];
  var nextListeners = currentListeners;
  var isDispatching = false;
  /**
   * This makes a shallow copy of currentListeners so we can use
   * nextListeners as a temporary list while dispatching.
   *
   * This prevents any bugs around consumers calling
   * subscribe/unsubscribe in the middle of a dispatch.
   */

  function ensureCanMutateNextListeners() {
    if (nextListeners === currentListeners) {
      nextListeners = currentListeners.slice();
    }
  }
  /**
   * Reads the state tree managed by the store.
   *
   * @returns {any} The current state tree of your application.
   */


  function getState() {
    if (isDispatching) {
      throw new Error(formatProdErrorMessage(3) );
    }

    return currentState;
  }
  /**
   * Adds a change listener. It will be called any time an action is dispatched,
   * and some part of the state tree may potentially have changed. You may then
   * call `getState()` to read the current state tree inside the callback.
   *
   * You may call `dispatch()` from a change listener, with the following
   * caveats:
   *
   * 1. The subscriptions are snapshotted just before every `dispatch()` call.
   * If you subscribe or unsubscribe while the listeners are being invoked, this
   * will not have any effect on the `dispatch()` that is currently in progress.
   * However, the next `dispatch()` call, whether nested or not, will use a more
   * recent snapshot of the subscription list.
   *
   * 2. The listener should not expect to see all state changes, as the state
   * might have been updated multiple times during a nested `dispatch()` before
   * the listener is called. It is, however, guaranteed that all subscribers
   * registered before the `dispatch()` started will be called with the latest
   * state by the time it exits.
   *
   * @param {Function} listener A callback to be invoked on every dispatch.
   * @returns {Function} A function to remove this change listener.
   */


  function subscribe(listener) {
    if (typeof listener !== 'function') {
      throw new Error(formatProdErrorMessage(4) );
    }

    if (isDispatching) {
      throw new Error(formatProdErrorMessage(5) );
    }

    var isSubscribed = true;
    ensureCanMutateNextListeners();
    nextListeners.push(listener);
    return function unsubscribe() {
      if (!isSubscribed) {
        return;
      }

      if (isDispatching) {
        throw new Error(formatProdErrorMessage(6) );
      }

      isSubscribed = false;
      ensureCanMutateNextListeners();
      var index = nextListeners.indexOf(listener);
      nextListeners.splice(index, 1);
      currentListeners = null;
    };
  }
  /**
   * Dispatches an action. It is the only way to trigger a state change.
   *
   * The `reducer` function, used to create the store, will be called with the
   * current state tree and the given `action`. Its return value will
   * be considered the **next** state of the tree, and the change listeners
   * will be notified.
   *
   * The base implementation only supports plain object actions. If you want to
   * dispatch a Promise, an Observable, a thunk, or something else, you need to
   * wrap your store creating function into the corresponding middleware. For
   * example, see the documentation for the `redux-thunk` package. Even the
   * middleware will eventually dispatch plain object actions using this method.
   *
   * @param {Object} action A plain object representing “what changed”. It is
   * a good idea to keep actions serializable so you can record and replay user
   * sessions, or use the time travelling `redux-devtools`. An action must have
   * a `type` property which may not be `undefined`. It is a good idea to use
   * string constants for action types.
   *
   * @returns {Object} For convenience, the same action object you dispatched.
   *
   * Note that, if you use a custom middleware, it may wrap `dispatch()` to
   * return something else (for example, a Promise you can await).
   */


  function dispatch(action) {
    if (!isPlainObject(action)) {
      throw new Error(formatProdErrorMessage(7) );
    }

    if (typeof action.type === 'undefined') {
      throw new Error(formatProdErrorMessage(8) );
    }

    if (isDispatching) {
      throw new Error(formatProdErrorMessage(9) );
    }

    try {
      isDispatching = true;
      currentState = currentReducer(currentState, action);
    } finally {
      isDispatching = false;
    }

    var listeners = currentListeners = nextListeners;

    for (var i = 0; i < listeners.length; i++) {
      var listener = listeners[i];
      listener();
    }

    return action;
  }
  /**
   * Replaces the reducer currently used by the store to calculate the state.
   *
   * You might need this if your app implements code splitting and you want to
   * load some of the reducers dynamically. You might also need this if you
   * implement a hot reloading mechanism for Redux.
   *
   * @param {Function} nextReducer The reducer for the store to use instead.
   * @returns {void}
   */


  function replaceReducer(nextReducer) {
    if (typeof nextReducer !== 'function') {
      throw new Error(formatProdErrorMessage(10) );
    }

    currentReducer = nextReducer; // This action has a similiar effect to ActionTypes.INIT.
    // Any reducers that existed in both the new and old rootReducer
    // will receive the previous state. This effectively populates
    // the new state tree with any relevant data from the old one.

    dispatch({
      type: ActionTypes.REPLACE
    });
  }
  /**
   * Interoperability point for observable/reactive libraries.
   * @returns {observable} A minimal observable of state changes.
   * For more information, see the observable proposal:
   * https://github.com/tc39/proposal-observable
   */


  function observable() {
    var _ref;

    var outerSubscribe = subscribe;
    return _ref = {
      /**
       * The minimal observable subscription method.
       * @param {Object} observer Any object that can be used as an observer.
       * The observer object should have a `next` method.
       * @returns {subscription} An object with an `unsubscribe` method that can
       * be used to unsubscribe the observable from the store, and prevent further
       * emission of values from the observable.
       */
      subscribe: function subscribe(observer) {
        if (typeof observer !== 'object' || observer === null) {
          throw new Error(formatProdErrorMessage(11) );
        }

        function observeState() {
          if (observer.next) {
            observer.next(getState());
          }
        }

        observeState();
        var unsubscribe = outerSubscribe(observeState);
        return {
          unsubscribe: unsubscribe
        };
      }
    }, _ref[$$observable] = function () {
      return this;
    }, _ref;
  } // When a store is created, an "INIT" action is dispatched so that every
  // reducer returns their initial state. This effectively populates
  // the initial state tree.


  dispatch({
    type: ActionTypes.INIT
  });
  return _ref2 = {
    dispatch: dispatch,
    subscribe: subscribe,
    getState: getState,
    replaceReducer: replaceReducer
  }, _ref2[$$observable] = observable, _ref2;
}

function assertReducerShape(reducers) {
  Object.keys(reducers).forEach(function (key) {
    var reducer = reducers[key];
    var initialState = reducer(undefined, {
      type: ActionTypes.INIT
    });

    if (typeof initialState === 'undefined') {
      throw new Error(formatProdErrorMessage(12) );
    }

    if (typeof reducer(undefined, {
      type: ActionTypes.PROBE_UNKNOWN_ACTION()
    }) === 'undefined') {
      throw new Error(formatProdErrorMessage(13) );
    }
  });
}
/**
 * Turns an object whose values are different reducer functions, into a single
 * reducer function. It will call every child reducer, and gather their results
 * into a single state object, whose keys correspond to the keys of the passed
 * reducer functions.
 *
 * @param {Object} reducers An object whose values correspond to different
 * reducer functions that need to be combined into one. One handy way to obtain
 * it is to use ES6 `import * as reducers` syntax. The reducers may never return
 * undefined for any action. Instead, they should return their initial state
 * if the state passed to them was undefined, and the current state for any
 * unrecognized action.
 *
 * @returns {Function} A reducer function that invokes every reducer inside the
 * passed object, and builds a state object with the same shape.
 */


function combineReducers(reducers) {
  var reducerKeys = Object.keys(reducers);
  var finalReducers = {};

  for (var i = 0; i < reducerKeys.length; i++) {
    var key = reducerKeys[i];

    if (typeof reducers[key] === 'function') {
      finalReducers[key] = reducers[key];
    }
  }

  var finalReducerKeys = Object.keys(finalReducers); // This is used to make sure we don't warn about the same

  var shapeAssertionError;

  try {
    assertReducerShape(finalReducers);
  } catch (e) {
    shapeAssertionError = e;
  }

  return function combination(state, action) {
    if (state === void 0) {
      state = {};
    }

    if (shapeAssertionError) {
      throw shapeAssertionError;
    }

    var hasChanged = false;
    var nextState = {};

    for (var _i = 0; _i < finalReducerKeys.length; _i++) {
      var _key = finalReducerKeys[_i];
      var reducer = finalReducers[_key];
      var previousStateForKey = state[_key];
      var nextStateForKey = reducer(previousStateForKey, action);

      if (typeof nextStateForKey === 'undefined') {
        throw new Error(formatProdErrorMessage(14) );
      }

      nextState[_key] = nextStateForKey;
      hasChanged = hasChanged || nextStateForKey !== previousStateForKey;
    }

    hasChanged = hasChanged || finalReducerKeys.length !== Object.keys(state).length;
    return hasChanged ? nextState : state;
  };
}

/**
 * Composes single-argument functions from right to left. The rightmost
 * function can take multiple arguments as it provides the signature for
 * the resulting composite function.
 *
 * @param {...Function} funcs The functions to compose.
 * @returns {Function} A function obtained by composing the argument functions
 * from right to left. For example, compose(f, g, h) is identical to doing
 * (...args) => f(g(h(...args))).
 */
function compose() {
  for (var _len = arguments.length, funcs = new Array(_len), _key = 0; _key < _len; _key++) {
    funcs[_key] = arguments[_key];
  }

  if (funcs.length === 0) {
    return function (arg) {
      return arg;
    };
  }

  if (funcs.length === 1) {
    return funcs[0];
  }

  return funcs.reduce(function (a, b) {
    return function () {
      return a(b.apply(void 0, arguments));
    };
  });
}

/**
 * Creates a store enhancer that applies middleware to the dispatch method
 * of the Redux store. This is handy for a variety of tasks, such as expressing
 * asynchronous actions in a concise manner, or logging every action payload.
 *
 * See `redux-thunk` package as an example of the Redux middleware.
 *
 * Because middleware is potentially asynchronous, this should be the first
 * store enhancer in the composition chain.
 *
 * Note that each middleware will be given the `dispatch` and `getState` functions
 * as named arguments.
 *
 * @param {...Function} middlewares The middleware chain to be applied.
 * @returns {Function} A store enhancer applying the middleware.
 */

function applyMiddleware() {
  for (var _len = arguments.length, middlewares = new Array(_len), _key = 0; _key < _len; _key++) {
    middlewares[_key] = arguments[_key];
  }

  return function (createStore) {
    return function () {
      var store = createStore.apply(void 0, arguments);

      var _dispatch = function dispatch() {
        throw new Error(formatProdErrorMessage(15) );
      };

      var middlewareAPI = {
        getState: store.getState,
        dispatch: function dispatch() {
          return _dispatch.apply(void 0, arguments);
        }
      };
      var chain = middlewares.map(function (middleware) {
        return middleware(middlewareAPI);
      });
      _dispatch = compose.apply(void 0, chain)(store.dispatch);
      return _objectSpread2(_objectSpread2({}, store), {}, {
        dispatch: _dispatch
      });
    };
  };
}

function Y(e){for(var t=arguments.length,r=Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];throw Error("[Immer] minified error nr: "+e+(r.length?" "+r.map(function(i){return "'"+i+"'"}).join(","):"")+". Find the full error at: https://bit.ly/3cXEKWf")}function qe(e){return !!e&&!!e[z]}function Ae(e){var t;return !!e&&(function(r){if(!r||typeof r!="object")return !1;var n=Object.getPrototypeOf(r);if(n===null)return !0;var o=Object.hasOwnProperty.call(n,"constructor")&&n.constructor;return o===Object||typeof o=="function"&&Function.toString.call(o)===Ug}(e)||Array.isArray(e)||!!e[as]||!!(!((t=e.constructor)===null||t===void 0)&&t[as])||_o(e)||$o(e))}function Ve(e,t,r){r===void 0&&(r=!1),ft(e)===0?(r?Object.keys:gt)(e).forEach(function(n){r&&typeof n=="symbol"||t(n,e[n],e);}):e.forEach(function(n,o){return t(o,n,e)});}function ft(e){var t=e[z];return t?t.i>3?t.i-4:t.i:Array.isArray(e)?1:_o(e)?2:$o(e)?3:0}function vt(e,t){return ft(e)===2?e.has(t):Object.prototype.hasOwnProperty.call(e,t)}function xg(e,t){return ft(e)===2?e.get(t):e[t]}function Ki(e,t,r){var n=ft(e);n===2?e.set(t,r):n===3?e.add(r):e[t]=r;}function Yi(e,t){return e===t?e!==0||1/e==1/t:e!=e&&t!=t}function _o(e){return Rg&&e instanceof Map}function $o(e){return Fg&&e instanceof Set}function xe(e){return e.o||e.t}function wo(e){if(Array.isArray(e))return Array.prototype.slice.call(e);var t=is(e);delete t[z];for(var r=gt(t),n=0;n<r.length;n++){var o=r[n],a=t[o];a.writable===!1&&(a.writable=!0,a.configurable=!0),(a.get||a.set)&&(t[o]={configurable:!0,writable:!0,enumerable:a.enumerable,value:e[o]});}return Object.create(Object.getPrototypeOf(e),t)}function Ao(e,t){return t===void 0&&(t=!1),Oo(e)||qe(e)||!Ae(e)||(ft(e)>1&&(e.set=e.add=e.clear=e.delete=kg),Object.freeze(e),t&&Ve(e,function(r,n){return Ao(n,!0)},!0)),e}function kg(){Y(2);}function Oo(e){return e==null||typeof e!="object"||Object.isFrozen(e)}function he(e){var t=xo[e];return t||Y(18,e),t}function Dg(e,t){xo[e]||(xo[e]=t);}function So(){return ht}function jo(e,t){t&&(he("Patches"),e.u=[],e.s=[],e.v=t);}function Hr(e){Co(e),e.p.forEach(Mg),e.p=null;}function Co(e){e===ht&&(ht=e.l);}function Ji(e){return ht={p:[],l:ht,h:e,m:!0,_:0}}function Mg(e){var t=e[z];t.i===0||t.i===1?t.j():t.O=!0;}function Po(e,t){t._=t.p.length;var r=t.p[0],n=e!==void 0&&e!==r;return t.h.g||he("ES5").S(t,e,n),n?(r[z].P&&(Hr(t),Y(4)),Ae(e)&&(e=Zr(t,e),t.l||Kr(t,e)),t.u&&he("Patches").M(r[z].t,e,t.u,t.s)):e=Zr(t,r,[]),Hr(t),t.u&&t.v(t.u,t.s),e!==os?e:void 0}function Zr(e,t,r){if(Oo(t))return t;var n=t[z];if(!n)return Ve(t,function(s,c){return Xi(e,n,t,s,c,r)},!0),t;if(n.A!==e)return t;if(!n.P)return Kr(e,n.t,!0),n.t;if(!n.I){n.I=!0,n.A._--;var o=n.i===4||n.i===5?n.o=wo(n.k):n.o,a=o,i=!1;n.i===3&&(a=new Set(o),o.clear(),i=!0),Ve(a,function(s,c){return Xi(e,n,o,s,c,r,i)}),Kr(e,o,!1),r&&e.u&&he("Patches").N(n,r,e.u,e.s);}return n.o}function Xi(e,t,r,n,o,a,i){if(qe(o)){var s=Zr(e,o,a&&t&&t.i!==3&&!vt(t.R,n)?a.concat(n):void 0);if(Ki(r,n,s),!qe(s))return;e.m=!1;}else i&&r.add(o);if(Ae(o)&&!Oo(o)){if(!e.h.D&&e._<1)return;Zr(e,o),t&&t.A.l||Kr(e,o);}}function Kr(e,t,r){r===void 0&&(r=!1),!e.l&&e.h.D&&e.m&&Ao(t,r);}function Io(e,t){var r=e[z];return (r?xe(r):e)[t]}function es(e,t){if(t in e)for(var r=Object.getPrototypeOf(e);r;){var n=Object.getOwnPropertyDescriptor(r,t);if(n)return n;r=Object.getPrototypeOf(r);}}function ke(e){e.P||(e.P=!0,e.l&&ke(e.l));}function Eo(e){e.o||(e.o=wo(e.t));}function To(e,t,r){var n=_o(t)?he("MapSet").F(t,r):$o(t)?he("MapSet").T(t,r):e.g?function(o,a){var i=Array.isArray(o),s={i:i?1:0,A:a?a.A:So(),P:!1,I:!1,R:{},l:a,t:o,k:null,o:null,j:null,C:!1},c=s,u=rr;i&&(c=[s],u=nr);var l=Proxy.revocable(c,u),d=l.revoke,p=l.proxy;return s.k=p,s.j=d,p}(t,r):he("ES5").J(t,r);return (r?r.A:So()).p.push(n),n}function Ng(e){return qe(e)||Y(22,e),function t(r){if(!Ae(r))return r;var n,o=r[z],a=ft(r);if(o){if(!o.P&&(o.i<4||!he("ES5").K(o)))return o.t;o.I=!0,n=ts(r,a),o.I=!1;}else n=ts(r,a);return Ve(n,function(i,s){o&&xg(o.t,i)===s||Ki(n,i,t(s));}),a===3?new Set(n):n}(e)}function ts(e,t){switch(t){case 2:return new Map(e);case 3:return Array.from(e)}return wo(e)}function Lg(){function e(i,s){var c=a[i];return c?c.enumerable=s:a[i]=c={configurable:!0,enumerable:s,get:function(){var u=this[z];return rr.get(u,i)},set:function(u){var l=this[z];rr.set(l,i,u);}},c}function t(i){for(var s=i.length-1;s>=0;s--){var c=i[s][z];if(!c.P)switch(c.i){case 5:n(c)&&ke(c);break;case 4:r(c)&&ke(c);}}}function r(i){for(var s=i.t,c=i.k,u=gt(c),l=u.length-1;l>=0;l--){var d=u[l];if(d!==z){var p=s[d];if(p===void 0&&!vt(s,d))return !0;var h=c[d],v=h&&h[z];if(v?v.t!==p:!Yi(h,p))return !0}}var g=!!s[z];return u.length!==gt(s).length+(g?0:1)}function n(i){var s=i.k;if(s.length!==i.t.length)return !0;var c=Object.getOwnPropertyDescriptor(s,s.length-1);if(c&&!c.get)return !0;for(var u=0;u<s.length;u++)if(!s.hasOwnProperty(u))return !0;return !1}var a={};Dg("ES5",{J:function(i,s){var c=Array.isArray(i),u=function(d,p){if(d){for(var h=Array(p.length),v=0;v<p.length;v++)Object.defineProperty(h,""+v,e(v,!0));return h}var g=is(p);delete g[z];for(var y=gt(g),m=0;m<y.length;m++){var $=y[m];g[$]=e($,d||!!g[$].enumerable);}return Object.create(Object.getPrototypeOf(p),g)}(c,i),l={i:c?5:4,A:s?s.A:So(),P:!1,I:!1,R:{},l:s,t:i,k:u,o:null,O:!1,C:!1};return Object.defineProperty(u,z,{value:l,writable:!0}),u},S:function(i,s,c){c?qe(s)&&s[z].A===i&&t(i.p):(i.u&&function u(l){if(l&&typeof l=="object"){var d=l[z];if(d){var p=d.t,h=d.k,v=d.R,g=d.i;if(g===4)Ve(h,function(A){A!==z&&(p[A]!==void 0||vt(p,A)?v[A]||u(h[A]):(v[A]=!0,ke(d)));}),Ve(p,function(A){h[A]!==void 0||vt(h,A)||(v[A]=!1,ke(d));});else if(g===5){if(n(d)&&(ke(d),v.length=!0),h.length<p.length)for(var y=h.length;y<p.length;y++)v[y]=!1;else for(var m=p.length;m<h.length;m++)v[m]=!0;for(var $=Math.min(h.length,p.length),S=0;S<$;S++)h.hasOwnProperty(S)||(v[S]=!0),v[S]===void 0&&u(h[S]);}}}}(i.p[0]),t(i.p));},K:function(i){return i.i===4?r(i):n(i)}});}var rs,ht,qo=typeof Symbol!="undefined"&&typeof Symbol("x")=="symbol",Rg=typeof Map!="undefined",Fg=typeof Set!="undefined",ns=typeof Proxy!="undefined"&&Proxy.revocable!==void 0&&typeof Reflect!="undefined",os=qo?Symbol.for("immer-nothing"):((rs={})["immer-nothing"]=!0,rs),as=qo?Symbol.for("immer-draftable"):"__$immer_draftable",z=qo?Symbol.for("immer-state"):"__$immer_state",Ug=""+Object.prototype.constructor,gt=typeof Reflect!="undefined"&&Reflect.ownKeys?Reflect.ownKeys:Object.getOwnPropertySymbols!==void 0?function(e){return Object.getOwnPropertyNames(e).concat(Object.getOwnPropertySymbols(e))}:Object.getOwnPropertyNames,is=Object.getOwnPropertyDescriptors||function(e){var t={};return gt(e).forEach(function(r){t[r]=Object.getOwnPropertyDescriptor(e,r);}),t},xo={},rr={get:function(e,t){if(t===z)return e;var r=xe(e);if(!vt(r,t))return function(o,a,i){var s,c=es(a,i);return c?"value"in c?c.value:(s=c.get)===null||s===void 0?void 0:s.call(o.k):void 0}(e,r,t);var n=r[t];return e.I||!Ae(n)?n:n===Io(e.t,t)?(Eo(e),e.o[t]=To(e.A.h,n,e)):n},has:function(e,t){return t in xe(e)},ownKeys:function(e){return Reflect.ownKeys(xe(e))},set:function(e,t,r){var n=es(xe(e),t);if(n!=null&&n.set)return n.set.call(e.k,r),!0;if(!e.P){var o=Io(xe(e),t),a=o==null?void 0:o[z];if(a&&a.t===r)return e.o[t]=r,e.R[t]=!1,!0;if(Yi(r,o)&&(r!==void 0||vt(e.t,t)))return !0;Eo(e),ke(e);}return e.o[t]===r&&(r!==void 0||t in e.o)||Number.isNaN(r)&&Number.isNaN(e.o[t])||(e.o[t]=r,e.R[t]=!0),!0},deleteProperty:function(e,t){return Io(e.t,t)!==void 0||t in e.t?(e.R[t]=!1,Eo(e),ke(e)):delete e.R[t],e.o&&delete e.o[t],!0},getOwnPropertyDescriptor:function(e,t){var r=xe(e),n=Reflect.getOwnPropertyDescriptor(r,t);return n&&{writable:!0,configurable:e.i!==1||t!=="length",enumerable:n.enumerable,value:r[t]}},defineProperty:function(){Y(11);},getPrototypeOf:function(e){return Object.getPrototypeOf(e.t)},setPrototypeOf:function(){Y(12);}},nr={};Ve(rr,function(e,t){nr[e]=function(){return arguments[0]=arguments[0][0],t.apply(this,arguments)};}),nr.deleteProperty=function(e,t){return nr.set.call(this,e,t,void 0)},nr.set=function(e,t,r){return rr.set.call(this,e[0],t,r,e[0])};var Vg=function(){function e(r){var n=this;this.g=ns,this.D=!0,this.produce=function(o,a,i){if(typeof o=="function"&&typeof a!="function"){var s=a;a=o;var c=n;return function(g){var y=this;g===void 0&&(g=s);for(var m=arguments.length,$=Array(m>1?m-1:0),S=1;S<m;S++)$[S-1]=arguments[S];return c.produce(g,function(A){var q;return (q=a).call.apply(q,[y,A].concat($))})}}var u;if(typeof a!="function"&&Y(6),i!==void 0&&typeof i!="function"&&Y(7),Ae(o)){var l=Ji(n),d=To(n,o,void 0),p=!0;try{u=a(d),p=!1;}finally{p?Hr(l):Co(l);}return typeof Promise!="undefined"&&u instanceof Promise?u.then(function(g){return jo(l,i),Po(g,l)},function(g){throw Hr(l),g}):(jo(l,i),Po(u,l))}if(!o||typeof o!="object"){if((u=a(o))===void 0&&(u=o),u===os&&(u=void 0),n.D&&Ao(u,!0),i){var h=[],v=[];he("Patches").M(o,u,h,v),i(h,v);}return u}Y(21,o);},this.produceWithPatches=function(o,a){if(typeof o=="function")return function(u){for(var l=arguments.length,d=Array(l>1?l-1:0),p=1;p<l;p++)d[p-1]=arguments[p];return n.produceWithPatches(u,function(h){return o.apply(void 0,[h].concat(d))})};var i,s,c=n.produce(o,a,function(u,l){i=u,s=l;});return typeof Promise!="undefined"&&c instanceof Promise?c.then(function(u){return [u,i,s]}):[c,i,s]},typeof(r==null?void 0:r.useProxies)=="boolean"&&this.setUseProxies(r.useProxies),typeof(r==null?void 0:r.autoFreeze)=="boolean"&&this.setAutoFreeze(r.autoFreeze);}var t=e.prototype;return t.createDraft=function(r){Ae(r)||Y(8),qe(r)&&(r=Ng(r));var n=Ji(this),o=To(this,r,void 0);return o[z].C=!0,Co(n),o},t.finishDraft=function(r,n){var o=r&&r[z];var a=o.A;return jo(a,n),Po(void 0,a)},t.setAutoFreeze=function(r){this.D=r;},t.setUseProxies=function(r){r&&!ns&&Y(20),this.g=r;},t.applyPatches=function(r,n){var o;for(o=n.length-1;o>=0;o--){var a=n[o];if(a.path.length===0&&a.op==="replace"){r=a.value;break}}o>-1&&(n=n.slice(o+1));var i=he("Patches").$;return qe(r)?i(r,n):this.produce(r,function(s){return i(s,n)})},e}(),ue=new Vg,ss=ue.produce;ue.produceWithPatches.bind(ue),ue.setAutoFreeze.bind(ue),ue.setUseProxies.bind(ue),ue.applyPatches.bind(ue),ue.createDraft.bind(ue),ue.finishDraft.bind(ue);var Yr="NOT_FOUND";function Bg(e){var t;return {get:function(n){return t&&e(t.key,n)?t.value:Yr},put:function(n,o){t={key:n,value:o};},getEntries:function(){return t?[t]:[]},clear:function(){t=void 0;}}}function Gg(e,t){var r=[];function n(s){var c=r.findIndex(function(l){return t(s,l.key)});if(c>-1){var u=r[c];return c>0&&(r.splice(c,1),r.unshift(u)),u.value}return Yr}function o(s,c){n(s)===Yr&&(r.unshift({key:s,value:c}),r.length>e&&r.pop());}function a(){return r}function i(){r=[];}return {get:n,put:o,getEntries:a,clear:i}}var zg=function(t,r){return t===r};function Wg(e){return function(r,n){if(r===null||n===null||r.length!==n.length)return !1;for(var o=r.length,a=0;a<o;a++)if(!e(r[a],n[a]))return !1;return !0}}function Hg(e,t){var r=typeof t=="object"?t:{equalityCheck:t},n=r.equalityCheck,o=n===void 0?zg:n,a=r.maxSize,i=a===void 0?1:a,s=r.resultEqualityCheck,c=Wg(o),u=i===1?Bg(c):Gg(i,c);function l(){var d=u.get(arguments);if(d===Yr){if(d=e.apply(null,arguments),s){var p=u.getEntries(),h=p.find(function(v){return s(v.value,d)});h&&(d=h.value);}u.put(arguments,d);}return d}return l.clearCache=function(){return u.clear()},l}function Zg(e){var t=Array.isArray(e[0])?e[0]:e;if(!t.every(function(n){return typeof n=="function"})){var r=t.map(function(n){return typeof n=="function"?"function "+(n.name||"unnamed")+"()":typeof n}).join(", ");throw new Error("createSelector expects all input-selectors to be functions, but received the following types: ["+r+"]")}return t}function Kg(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];var o=function(){for(var i=arguments.length,s=new Array(i),c=0;c<i;c++)s[c]=arguments[c];var u=0,l,d={memoizeOptions:void 0},p=s.pop();if(typeof p=="object"&&(d=p,p=s.pop()),typeof p!="function")throw new Error("createSelector expects an output function after the inputs, but received: ["+typeof p+"]");var h=d,v=h.memoizeOptions,g=v===void 0?r:v,y=Array.isArray(g)?g:[g],m=Zg(s),$=e.apply(void 0,[function(){return u++,p.apply(null,arguments)}].concat(y)),S=e(function(){for(var q=[],ne=m.length,se=0;se<ne;se++)q.push(m[se].apply(null,arguments));return l=$.apply(null,q),l});return Object.assign(S,{resultFunc:p,memoizedResultFunc:$,dependencies:m,lastResult:function(){return l},recomputations:function(){return u},resetRecomputations:function(){return u=0}}),S};return o}var f=Kg(Hg);function cs(e){var t=function(n){var o=n.dispatch,a=n.getState;return function(i){return function(s){return typeof s=="function"?s(o,a,e):i(s)}}};return t}var us=cs();us.withExtraArgument=cs;var ls=us,Yg=function(){var e=function(t,r){return e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(n,o){n.__proto__=o;}||function(n,o){for(var a in o)Object.prototype.hasOwnProperty.call(o,a)&&(n[a]=o[a]);},e(t,r)};return function(t,r){if(typeof r!="function"&&r!==null)throw new TypeError("Class extends value "+String(r)+" is not a constructor or null");e(t,r);function n(){this.constructor=t;}t.prototype=r===null?Object.create(r):(n.prototype=r.prototype,new n);}}(),Jg=function(e,t){var r={label:0,sent:function(){if(a[0]&1)throw a[1];return a[1]},trys:[],ops:[]},n,o,a,i;return i={next:s(0),throw:s(1),return:s(2)},typeof Symbol=="function"&&(i[Symbol.iterator]=function(){return this}),i;function s(u){return function(l){return c([u,l])}}function c(u){if(n)throw new TypeError("Generator is already executing.");for(;r;)try{if(n=1,o&&(a=u[0]&2?o.return:u[0]?o.throw||((a=o.return)&&a.call(o),0):o.next)&&!(a=a.call(o,u[1])).done)return a;switch(o=0,a&&(u=[u[0]&2,a.value]),u[0]){case 0:case 1:a=u;break;case 4:return r.label++,{value:u[1],done:!1};case 5:r.label++,o=u[1],u=[0];continue;case 7:u=r.ops.pop(),r.trys.pop();continue;default:if(a=r.trys,!(a=a.length>0&&a[a.length-1])&&(u[0]===6||u[0]===2)){r=0;continue}if(u[0]===3&&(!a||u[1]>a[0]&&u[1]<a[3])){r.label=u[1];break}if(u[0]===6&&r.label<a[1]){r.label=a[1],a=u;break}if(a&&r.label<a[2]){r.label=a[2],r.ops.push(u);break}a[2]&&r.ops.pop(),r.trys.pop();continue}u=t.call(e,r);}catch(l){u=[6,l],o=0;}finally{n=a=0;}if(u[0]&5)throw u[1];return {value:u[0]?u[1]:void 0,done:!0}}},Jr=function(e,t){for(var r=0,n=t.length,o=e.length;r<n;r++,o++)e[o]=t[r];return e},Xg=Object.defineProperty,em=Object.defineProperties,tm=Object.getOwnPropertyDescriptors,ds=Object.getOwnPropertySymbols,rm=Object.prototype.hasOwnProperty,nm=Object.prototype.propertyIsEnumerable,ps=function(e,t,r){return t in e?Xg(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r},De=function(e,t){for(var r in t||(t={}))rm.call(t,r)&&ps(e,r,t[r]);if(ds)for(var n=0,o=ds(t);n<o.length;n++){var r=o[n];nm.call(t,r)&&ps(e,r,t[r]);}return e},ko=function(e,t){return em(e,tm(t))},om=function(e,t,r){return new Promise(function(n,o){var a=function(c){try{s(r.next(c));}catch(u){o(u);}},i=function(c){try{s(r.throw(c));}catch(u){o(u);}},s=function(c){return c.done?n(c.value):Promise.resolve(c.value).then(a,i)};s((r=r.apply(e,t)).next());})},am=typeof window!="undefined"&&window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__?window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__:function(){if(arguments.length!==0)return typeof arguments[0]=="object"?compose:compose.apply(null,arguments)};function fs(e){if(typeof e!="object"||e===null)return !1;var t=Object.getPrototypeOf(e);if(t===null)return !0;for(var r=t;Object.getPrototypeOf(r)!==null;)r=Object.getPrototypeOf(r);return t===r}var im=function(e){Yg(t,e);function t(){for(var r=[],n=0;n<arguments.length;n++)r[n]=arguments[n];var o=e.apply(this,r)||this;return Object.setPrototypeOf(o,t.prototype),o}return Object.defineProperty(t,Symbol.species,{get:function(){return t},enumerable:!1,configurable:!0}),t.prototype.concat=function(){for(var r=[],n=0;n<arguments.length;n++)r[n]=arguments[n];return e.prototype.concat.apply(this,r)},t.prototype.prepend=function(){for(var r=[],n=0;n<arguments.length;n++)r[n]=arguments[n];return r.length===1&&Array.isArray(r[0])?new(t.bind.apply(t,Jr([void 0],r[0].concat(this)))):new(t.bind.apply(t,Jr([void 0],r.concat(this))))},t}(Array);function Do(e){return Ae(e)?ss(e,function(){}):e}function No(e){return typeof e=="boolean"}function vm(){return function(t){return hm(t)}}function hm(e){e===void 0&&(e={});var t=e.thunk,r=t===void 0?!0:t,s=new im;if(r&&(No(r)?s.push(ls):s.push(ls.withExtraArgument(r.extraArgument))),"production"!=="production");return s}var Lo="production"==="production";function gm(e){var t=vm(),r=e||{},n=r.reducer,o=n===void 0?void 0:n,a=r.middleware,i=a===void 0?t():a,s=r.devTools,c=s===void 0?!0:s,u=r.preloadedState,l=u===void 0?void 0:u,d=r.enhancers,p=d===void 0?void 0:d,h;if(typeof o=="function")h=o;else if(fs(o))h=combineReducers(o);else throw new Error('"reducer" is a required argument, and must be a function or an object of functions that can be passed to combineReducers');var v=i;if(typeof v=="function"&&(v=v(t),!Lo))throw new Error("when using a middleware builder function, an array of middleware must be returned");var g=applyMiddleware.apply(void 0,v),y=compose;c&&(y=am(De({trace:!Lo},typeof c=="object"&&c)));var m=[g];Array.isArray(p)?m=Jr([g],p):typeof p=="function"&&(m=p(m));var $=y.apply(void 0,m);return createStore(h,l,$)}function K(e,t){function r(){for(var n=[],o=0;o<arguments.length;o++)n[o]=arguments[o];if(t){var a=t.apply(void 0,n);if(!a)throw new Error("prepareAction did not return an object");return De(De({type:e,payload:a.payload},"meta"in a&&{meta:a.meta}),"error"in a&&{error:a.error})}return {type:e,payload:n[0]}}return r.toString=function(){return ""+e},r.type=e,r.match=function(n){return n.type===e},r}function $s(e){var t={},r=[],n,o={addCase:function(a,i){var s=typeof a=="string"?a:a.type;if(s in t)throw new Error("addCase cannot be called with two reducers for the same action type");return t[s]=i,o},addMatcher:function(a,i){return r.push({matcher:a,reducer:i}),o},addDefaultCase:function(a){return n=a,o}};return e(o),[t,r,n]}function mm(e){return typeof e=="function"}function bm(e,t,r,n){r===void 0&&(r=[]),"production"!=="production";var o=typeof t=="function"?$s(t):[t,r,n],a=o[0],i=o[1],s=o[2],c;if(mm(e))c=function(){return Do(e())};else {var u=Do(e);c=function(){return u};}function l(d,p){d===void 0&&(d=c());var h=Jr([a[p.type]],i.filter(function(v){var g=v.matcher;return g(p)}).map(function(v){var g=v.reducer;return g}));return h.filter(function(v){return !!v}).length===0&&(h=[s]),h.reduce(function(v,g){if(g)if(qe(v)){var y=v,m=g(y,p);return m===void 0?v:m}else {if(Ae(v))return ss(v,function($){return g($,p)});var m=g(v,p);if(m===void 0){if(v===null)return v;throw Error("A case reducer on a non-draftable value must not return undefined")}return m}return v},d)}return l.getInitialState=c,l}function ym(e,t){return e+"/"+t}function E(e){var t=e.name;if(!t)throw new Error("`name` is a required option for createSlice");typeof process!="undefined"&&"production"==="development"&&e.initialState===void 0&&console.error("You must provide an `initialState` value that is not `undefined`. You may have misspelled `initialState`");var r=typeof e.initialState=="function"?e.initialState:Do(e.initialState),n=e.reducers||{},o=Object.keys(n),a={},i={},s={};o.forEach(function(l){var d=n[l],p=ym(t,l),h,v;"reducer"in d?(h=d.reducer,v=d.prepare):h=d,a[l]=h,i[p]=h,s[l]=v?K(p,v):K(p);});function c(){var l=typeof e.extraReducers=="function"?$s(e.extraReducers):[e.extraReducers],d=l[0],p=d===void 0?{}:d,h=l[1],v=h===void 0?[]:h,g=l[2],y=g===void 0?void 0:g,m=De(De({},p),i);return bm(r,function($){for(var S in m)$.addCase(S,m[S]);for(var A=0,q=v;A<q.length;A++){var ne=q[A];$.addMatcher(ne.matcher,ne.reducer);}y&&$.addDefaultCase(y);})}var u;return {name:t,reducer:function(l,d){return u||(u=c()),u(l,d)},actions:s,caseReducers:a,getInitialState:function(){return u||(u=c()),u.getInitialState()}}}var _m="ModuleSymbhasOwnPr-0123456789ABCDEFGHNRVfgctiUvz_KqYTJkLxpZXIjQW",$m=function(e){e===void 0&&(e=21);for(var t="",r=e;r--;)t+=_m[Math.random()*64|0];return t},wm=["name","message","stack","code"],Ro=function(){function e(t,r){this.payload=t,this.meta=r;}return e}(),Os=function(){function e(t,r){this.payload=t,this.meta=r;}return e}(),Am=function(e){if(typeof e=="object"&&e!==null){for(var t={},r=0,n=wm;r<n.length;r++){var o=n[r];typeof e[o]=="string"&&(t[o]=e[o]);}return t}return {message:String(e)}},L=function(){function e(t,r,n){var o=K(t+"/fulfilled",function(l,d,p,h){return {payload:l,meta:ko(De({},h||{}),{arg:p,requestId:d,requestStatus:"fulfilled"})}}),a=K(t+"/pending",function(l,d,p){return {payload:void 0,meta:ko(De({},p||{}),{arg:d,requestId:l,requestStatus:"pending"})}}),i=K(t+"/rejected",function(l,d,p,h,v){return {payload:h,error:(n&&n.serializeError||Am)(l||"Rejected"),meta:ko(De({},v||{}),{arg:p,requestId:d,rejectedWithValue:!!h,requestStatus:"rejected",aborted:(l==null?void 0:l.name)==="AbortError",condition:(l==null?void 0:l.name)==="ConditionError"})}}),c=typeof AbortController!="undefined"?AbortController:function(){function l(){this.signal={aborted:!1,addEventListener:function(){},dispatchEvent:function(){return !1},onabort:function(){},removeEventListener:function(){},reason:void 0,throwIfAborted:function(){}};}return l.prototype.abort=function(){},l}();function u(l){return function(d,p,h){var v=n!=null&&n.idGenerator?n.idGenerator(l):$m(),g=new c,y;function m(S){y=S,g.abort();}var $=function(){return om(this,null,function(){var S,A,q,ne,se,ae,pe;return Jg(this,function(ce){switch(ce.label){case 0:return ce.trys.push([0,4,,5]),ne=(S=n==null?void 0:n.condition)==null?void 0:S.call(n,l,{getState:p,extra:h}),Sm(ne)?[4,ne]:[3,2];case 1:ne=ce.sent(),ce.label=2;case 2:if(ne===!1||g.signal.aborted)throw {name:"ConditionError",message:"Aborted due to condition callback returning false."};return se=new Promise(function(re,tr){return g.signal.addEventListener("abort",function(){return tr({name:"AbortError",message:y||"Aborted"})})}),d(a(v,l,(A=n==null?void 0:n.getPendingMeta)==null?void 0:A.call(n,{requestId:v,arg:l},{getState:p,extra:h}))),[4,Promise.race([se,Promise.resolve(r(l,{dispatch:d,getState:p,extra:h,requestId:v,signal:g.signal,abort:m,rejectWithValue:function(re,tr){return new Ro(re,tr)},fulfillWithValue:function(re,tr){return new Os(re,tr)}})).then(function(re){if(re instanceof Ro)throw re;return re instanceof Os?o(re.payload,v,l,re.meta):o(re,v,l)})])];case 3:return q=ce.sent(),[3,5];case 4:return ae=ce.sent(),q=ae instanceof Ro?i(null,v,l,ae.payload,ae.meta):i(ae,v,l),[3,5];case 5:return pe=n&&!n.dispatchConditionRejection&&i.match(q)&&q.meta.condition,pe||d(q),[2,q]}})})}();return Object.assign($,{abort:m,requestId:v,arg:l,unwrap:function(){return $.then(Om)}})}}return Object.assign(u,{pending:a,rejected:i,fulfilled:o,typePrefix:t})}return e.withTypes=function(){return e},e}();function Om(e){if(e.meta&&e.meta.rejectedWithValue)throw e.payload;if(e.error)throw e.error;return e.payload}function Sm(e){return e!==null&&typeof e=="object"&&typeof e.then=="function"}var jm=function(e){return e&&typeof e.match=="function"},Cm=function(e,t){return jm(e)?e.match(t):e(t)};function _(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return function(r){return e.some(function(n){return Cm(n,r)})}}var Fo="listenerMiddleware";K(Fo+"/add"),K(Fo+"/removeAll"),K(Fo+"/remove");typeof queueMicrotask=="function"&&queueMicrotask.bind(typeof window!="undefined"?window:typeof global!="undefined"?global:globalThis),Lg();var Pm=(e=>(e.LIGHT="Light",e.COLOR="Color",e))(Pm||{}),Im=(e=>(e.ApplicationForms="application_forms",e.PartnerPhoneCalls="partner_phone_calls",e.FeatureCheck="features",e.Questions="questions",e.Rules="rules",e.RuleExcludedChoices="rule_excluded_choices",e.SearchableQuestions="searchable_questions",e.SendToExchange="send_to_exchange",e.TakeOver="take_over",e))(Im||{}),mt=(e=>(e.NotConnected="NotConnected",e.Connected="Connected",e.Disconnected="Disconnected",e.Connecting="Connecting",e.Disconnecting="Disconnecting",e))(mt||{}),Em=(e=>(e.Create="create",e.Index="index",e.Show="show",e.Update="update",e))(Em||{}),js=(e=>(e.Development="development",e.Alpha="alpha",e.Beta="beta",e.Production="prod",e))(js||{}),Tm=(e=>(e.Accepted="Accepted",e.Expired="Expired",e.Not_Created="Not created",e.Queued="Queued to send",e.Sent="Sent",e))(Tm||{}),qm=(e=>(e.INVITE="invite",e.RESET="reset",e))(qm||{}),bt=(e=>(e.token="token",e.cableToken="cable_token",e.user="user",e.state="state",e.expiresAt="expiresAt",e.refreshAt="refreshAt",e.impersonated="impersonated",e.terminalTheme="terminalTheme",e))(bt||{}),x=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Configured="Configured",e.Error="Error",e))(x||{}),yt=(e=>(e.Healthy="Healthy",e.Error="Error",e))(yt||{}),xm=(e=>(e.OAuth="OAuth",e.Login="Login",e))(xm||{}),km=(e=>(e.storageKey="BP_SDK_ANALYTICS_VARS",e.additionalQuoteOnline="onlinebuy",e.additionalQuoteOffline="nobuy",e))(km||{}),Dm=(e=>(e.activeIdUpdate="activeIdUpdate",e.activeQuestionSetChange="activeQuestionSetChange",e.addItem="addItem",e.answerStateChanged="answerStateChanged",e.bpAnalyticsTrack="bpAnalyticsTrack",e.closePanel="closePanel",e.deleteItem="deleteItem",e.deletePool="deletePool",e.epFieldChanged="epFieldChanged",e.errorsUpdate="errorsUpdate",e.fakeValueUpdate="fakeValueUpdate",e.filterUpdate="filterUpdate",e.formLoaded="formLoaded",e.nudgeAnswer="nudgeAnswer",e.openPanel="openPanel",e.placesUpdate="placesUpdate",e.questionSetAdded="questionSetAdded",e.questionSetCompletionChanged="questionSetCompletionChanged",e.quoteFailure="quoteFailure",e.quotesChanged="quotesChanged",e.quoteSuccess="quoteSuccess",e.removeActiveAnswer="removeActiveAnswer",e.saveAnswer="saveAnswer",e.saveVehicle="saveVehicle",e.sdkError="sdkError",e.tabChanged="tabChanged",e.updateActiveAnswer="updateActiveAnswer",e.userAuthenticationChanged="userAuthenticationChanged",e.valueUpdate="valueUpdate",e.vinUpdate="vinUpdate",e.ymmcUpdate="ymmcUpdate",e))(Dm||{}),Mm=(e=>(e.vin="vin",e.ymm="ymm",e.trailers="Trailers",e))(Mm||{}),Nm=(e=>(e.trailers="trailers",e))(Nm||{}),Lm=(e=>(e.Configuration="configuration",e.Make="make",e.Model="model",e.Year="year",e))(Lm||{}),Rm=(e=>(e.Address="address",e.Accordion="accordion",e.AccumulatePercentage="accumulate_percentage",e.Vehicle="vehicle",e.Driver="driver",e.Stepper="stepper",e.Step="step",e.Coverage="coverage",e.AutoCollapsedAccordion="auto_collapsed_accordion",e))(Rm||{}),j=(e=>(e.Address="address",e.Anchor="anchor",e.CheckBox="checkbox",e.CoverageType="coverage_type",e.Currency="currency",e.Date="date",e.DeletePool="delete_pool",e.Dialog="dialog",e.DriversLicense="drivers_license",e.DropDown="dropdown",e.Email="email",e.FederalEmployeeIdentification="fein",e.JSON="json",e.Label="label",e.Masked="masked",e.Number="number",e.Object="object",e.Percentage="percentage",e.Phone="phone",e.Radio="radio",e.Select="select",e.SocialSecurityNumber="ssn",e.Step="step",e.Tenant="tenant",e.Text="text",e.Vehicle="vehicle",e.Url="url",e.VIN="vin",e.Year="year",e.ZipCode="zipcode",e))(j||{}),Xr=(e=>(e.Accumulator="accumulator",e.AddressJSON="address_json",e.BusinessClass="business_class",e.City="city",e.CommercialAutoACV="ca_acv",e.CommercialAutoCategory="ca_category",e.CommercialAutoCount="ca_count",e.CommercialAutoDrivetrain="ca_drivetrain",e.CommercialAutoFlow="ca_flow",e.CommercialAutoGCW="ca_gcw",e.CommercialAutoGVW="ca_gvw",e.CommercialAutoJson="ca_json",e.CommercialAutoMake="ca_make",e.CommercialAutoModel="ca_model",e.CommercialAutoMSRP="ca_msrp",e.CommercialAutoSubModel="ca_submodel",e.CommercialAutoTractor="ca_tractor",e.CommercialAutoVIN="ca_vin",e.CommercialAutoYear="ca_year",e.County="county",e.DriversLicenseState="drivers_license_state",e.Indent="indent",e.Increment="increment",e.JornayaLeadID="jornaya_lead_id",e.LinesOfBusiness="lines_of_business",e.LinesOfBusinessAdditional="lines_of_business_additional",e.MotorVehicleReportComplete="mvr_complete",e.MotorVehicleReportState="mvr_state",e.MotorVehicleReportCondition="mvr_report_condition",e.State="state",e.Street1="street_1",e.Street2="street_2",e.ZipCode="zipcode",e))(Xr||{}),Fm=(e=>(e.CoverageLimit="coverage limit",e))(Fm||{}),Qm=(e=>(e.MaximumDate="maxDate",e.MaximumValue="max",e.MinimumDate="minDate",e.MinimumValue="min",e.Pattern="pattern",e.Required="required",e))(Qm||{}),Um=(e=>(e.Acord="acord",e.Acord125="acord_125",e.Acord126="acord_126",e.Acord130="acord_130",e))(Um||{}),Qo=(e=>(e.BP_QUOTABLE="bp_quotable",e.OFFLINE="offline",e.ONLINE="online",e))(Qo||{}),en=(e=>(e.Pending="pending",e.Queued="queued",e.Executing="executing",e.Completed="completed",e.Failed="failed",e.Aborted="aborted",e))(en||{}),Vm=(e=>(e.ClueReport="ClueReport",e.MotorVehicleReport="MotorVehicleReport",e.RequestPrefill="RequestPrefill",e.DriverScore="DriverScore",e.JornayaAudit="JornayaAudit",e.DotNumber="DotNumber",e))(Vm||{}),_t=(e=>(e.Incomplete="incomplete",e.InProgress="in_progress",e.Completed="completed",e.Quoted="quoted",e))(_t||{}),Bm=(e=>(e.not_permitted_to_bind="not_permitted_to_bind",e.sent_to_wholesaler="sent_to_wholesaler",e))(Bm||{}),Gm=(e=>(e.dialog="dialog",e.wholesale_workflow="wholesale_workflow",e))(Gm||{}),Uo=(e=>(e.NON_ADMITTED="non_admitted",e.WHOLESALER="wholesaler",e.INSTANT_ENROLLMENT="instant_enrollment",e.ADDITIONAL_MARKET="additional_market",e.OPT_IN_QUOTE_FLOW="opt_in_quote_flow",e))(Uo||{});var $t=(e=>(e.Answer="answer",e.Application="application_form",e.QuoteRequest="quote_request",e))($t||{}),Vo=(e=>(e.unsent="unsent",e.in_progress="in_progress",e.sent="sent",e.completed="completed",e.failed="failed",e.timed_out="timed_out",e.ineligible="ineligible",e.referral="referral",e.bound="bound",e))(Vo||{}),T=(e=>(e.AddQuestionSet="application_forms_question_set",e.Answer="answer",e.Appetite="appetite",e.ApplicationForm="application_form",e.ApplicationFormPercentCompleted="application_form_percent_completed",e.ApplicationFormProcess="application_form_process",e.Batch="batch",e.CarrierProducts="eligible_carriers_products",e.Message="message",e.Notification="notification",e.QuoteRequest="quote_request",e.RealTimeEligibility="real_time_eligibility",e.RemoveQuestionSet="destroy_application_forms_question_set",e))(T||{}),zm=(e=>(e.AcordForm="acord_form_pdf",e.AgentQuoteStart="agent_quote_start",e.ClientQuoteStart="client_quote_start",e.Storefront="storefront",e))(zm||{}),b=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Error="Error",e))(b||{});const Wm={email:"",expiresAt:0,firstName:"",lastName:"",name:"",options:{},permissions:[],refreshCount:0,tenantLogoUrl:"",tenantName:"",tenantTheme:"",userId:"",terminalTheme:void 0,token:"",user:void 0},Hm={},Zm={bcsChoiceLimit:7,driverLimit:20,enableAutocomplete:!0,enableDetailedRelevanceChips:!1,enableQuestionSetNavigation:!0,enableTrailerSupport:!0,enableUserApplicationGuard:!1,enableWebSockets:!0,quoteStatusFilter:[Vo.completed],renderOfflineQuotes:!1,vehicleLimit:20,renderQuotesAsync:!1,renderQuotesWithAppState:[_t.Completed,_t.Quoted],hideNestedQuestionGroupLabels:!1,showParentQuestionSetsInNavigation:!1},Km={clientId:"",cableToken:"",options:Zm,env:js.Production,debug:!1,googleApiToken:"",uri:"",wsBaseUri:"",theming:!1},Ym={},Jm={appetite:[],buckets:[],cloned:!1,completed_at:new Date(0),created_at:new Date(0),created_by:{id:"",first_name:"",last_name:""},current_bucket_id:null,eligible_carriers_products:[],features:[],finished_quote_requests_count:0,global_origin_tenant_name:null,id:"",inactive:!1,locked:!1,messages:[],owner_id:"",owner:{id:"",first_name:"",last_name:""},owner_is_guest:!1,parent_id:null,percent_completed:0,quote_requests_count:0,real_time_eligibility:{application_form_id:"",carriers:[]},redirect_url:"",reference_id:"",state:_t.Incomplete,taken_over:!1,tenant_id:"",updated_at:new Date(0),user_group_id:"",sent_to_wholesaler_at:null,source:"",source_metadata:null,wholesaler_preferred_quotes:null},Xm={meta:{form:{percent:0,forwardProgressPercent:0}}},eb={list:[],active:"",activeAnswerId:"",idOfNextActiveQuestionSetNotInState:void 0,completedQuestionSetAnswerHasChanged:!1,parentQuestionSets:[]},tb={},rb={},Bo={isAvailable:!1,errorCount:0},nb={configurations:{},makes:{},models:{},selection:{},vin:{},years:{}},Cs={list:[],isLoaded:!1};var ob=Object.defineProperty,Ps=Object.getOwnPropertySymbols,ab=Object.prototype.hasOwnProperty,ib=Object.prototype.propertyIsEnumerable,Is=(e,t,r)=>t in e?ob(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Es=(e,t)=>{for(var r in t||(t={}))ab.call(t,r)&&Is(e,r,t[r]);if(Ps)for(var r of Ps(t))ib.call(t,r)&&Is(e,r,t[r]);return e};Es({},nb),Es({},Cs);const sb={answers:{queue:[],outbound:{}}};var cb=Object.defineProperty,Ts=Object.getOwnPropertySymbols,ub=Object.prototype.hasOwnProperty,lb=Object.prototype.propertyIsEnumerable,qs=(e,t,r)=>t in e?cb(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Be=(e,t)=>{for(var r in t||(t={}))ub.call(t,r)&&qs(e,r,t[r]);if(Ts)for(var r of Ts(t))lb.call(t,r)&&qs(e,r,t[r]);return e};const db={configurations:{},makes:{},models:{},selection:{},vins:{},years:{}},wt={state:b.NotLoaded,map:{}},tn={state:b.NotLoaded},xs={form:b.NotLoaded},pb={answers:Be({},wt),application:Be({},xs),bindQuotes:{},choices:{},carriersProducts:Be({},tn),questionPools:Be({},wt),questionSets:Be({},wt),quotes:Be({},tn),refilters:{},vehicles:Be({},db)};var fb=Object.defineProperty,ks=Object.getOwnPropertySymbols,vb=Object.prototype.hasOwnProperty,hb=Object.prototype.propertyIsEnumerable,Ds=(e,t,r)=>t in e?fb(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Ms=(e,t)=>{for(var r in t||(t={}))vb.call(t,r)&&Ds(e,r,t[r]);if(ks)for(var r of ks(t))hb.call(t,r)&&Ds(e,r,t[r]);return e};const Ns={auth:x.NotLoaded,config:x.NotLoaded,health:yt.Healthy,state:x.NotLoaded,user:x.NotLoaded,socket:mt.NotConnected,socketId:""};Ms({},pb),Ms({},Ns);var gb=(e=>(e.QuestionSetSaved="QuestionSetSaved",e))(gb||{});let M=class extends Error{constructor(t,r){super(t),this.name="ReduxAsyncError",r&&r.parsedBody instanceof Array&&(this.message=`${this.message} ${r.parsedBody.map(o=>o)}`,this.response=r);const n=new.target.prototype;Object.setPrototypeOf?Object.setPrototypeOf(this,n):this.__proto__=n;}};var rn={logger:self.console,WebSocket:self.WebSocket},W={log(...e){this.enabled&&(e.push(Date.now()),rn.logger.log("[ActionCable]",...e));}};const or=()=>new Date().getTime(),nn=e=>(or()-e)/1e3;let Go=class{constructor(t){this.visibilityDidChange=this.visibilityDidChange.bind(this),this.connection=t,this.reconnectAttempts=0;}start(){this.isRunning()||(this.startedAt=or(),delete this.stoppedAt,this.startPolling(),addEventListener("visibilitychange",this.visibilityDidChange),W.log(`ConnectionMonitor started. stale threshold = ${this.constructor.staleThreshold} s`));}stop(){this.isRunning()&&(this.stoppedAt=or(),this.stopPolling(),removeEventListener("visibilitychange",this.visibilityDidChange),W.log("ConnectionMonitor stopped"));}isRunning(){return this.startedAt&&!this.stoppedAt}recordPing(){this.pingedAt=or();}recordConnect(){this.reconnectAttempts=0,this.recordPing(),delete this.disconnectedAt,W.log("ConnectionMonitor recorded connect");}recordDisconnect(){this.disconnectedAt=or(),W.log("ConnectionMonitor recorded disconnect");}startPolling(){this.stopPolling(),this.poll();}stopPolling(){clearTimeout(this.pollTimeout);}poll(){this.pollTimeout=setTimeout(()=>{this.reconnectIfStale(),this.poll();},this.getPollInterval());}getPollInterval(){const{staleThreshold:t,reconnectionBackoffRate:r}=this.constructor,n=Math.pow(1+r,Math.min(this.reconnectAttempts,10)),o=(this.reconnectAttempts===0?1:r)*Math.random();return t*1e3*n*(1+o)}reconnectIfStale(){this.connectionIsStale()&&(W.log(`ConnectionMonitor detected stale connection. reconnectAttempts = ${this.reconnectAttempts}, time stale = ${nn(this.refreshedAt)} s, stale threshold = ${this.constructor.staleThreshold} s`),this.reconnectAttempts++,this.disconnectedRecently()?W.log(`ConnectionMonitor skipping reopening recent disconnect. time disconnected = ${nn(this.disconnectedAt)} s`):(W.log("ConnectionMonitor reopening"),this.connection.reopen()));}get refreshedAt(){return this.pingedAt?this.pingedAt:this.startedAt}connectionIsStale(){return nn(this.refreshedAt)>this.constructor.staleThreshold}disconnectedRecently(){return this.disconnectedAt&&nn(this.disconnectedAt)<this.constructor.staleThreshold}visibilityDidChange(){document.visibilityState==="visible"&&setTimeout(()=>{(this.connectionIsStale()||!this.connection.isOpen())&&(W.log(`ConnectionMonitor reopening stale connection on visibilitychange. visibilityState = ${document.visibilityState}`),this.connection.reopen());},200);}};Go.staleThreshold=6,Go.reconnectionBackoffRate=.15;var mb={message_types:{welcome:"welcome",disconnect:"disconnect",ping:"ping",confirmation:"confirm_subscription",rejection:"reject_subscription"},disconnect_reasons:{unauthorized:"unauthorized",invalid_request:"invalid_request",server_restart:"server_restart",remote:"remote"},default_mount_path:"/cable",protocols:["actioncable-v1-json","actioncable-unsupported"]};const{message_types:ar,protocols:zo}=mb,bb=zo.slice(0,zo.length-1),Ls=[].indexOf;let Wo=class{constructor(t){this.open=this.open.bind(this),this.consumer=t,this.subscriptions=this.consumer.subscriptions,this.monitor=new Go(this),this.disconnected=!0;}send(t){return this.isOpen()?(this.webSocket.send(JSON.stringify(t)),!0):!1}open(){if(this.isActive())return W.log(`Attempted to open WebSocket, but existing socket is ${this.getState()}`),!1;{const t=[...zo,...this.consumer.subprotocols||[]];return W.log(`Opening WebSocket, current state is ${this.getState()}, subprotocols: ${t}`),this.webSocket&&this.uninstallEventHandlers(),this.webSocket=new rn.WebSocket(this.consumer.url,t),this.installEventHandlers(),this.monitor.start(),!0}}close({allowReconnect:t}={allowReconnect:!0}){if(t||this.monitor.stop(),this.isOpen())return this.webSocket.close()}reopen(){if(W.log(`Reopening WebSocket, current state is ${this.getState()}`),this.isActive())try{return this.close()}catch(t){W.log("Failed to reopen WebSocket",t);}finally{W.log(`Reopening WebSocket in ${this.constructor.reopenDelay}ms`),setTimeout(this.open,this.constructor.reopenDelay);}else return this.open()}getProtocol(){if(this.webSocket)return this.webSocket.protocol}isOpen(){return this.isState("open")}isActive(){return this.isState("open","connecting")}triedToReconnect(){return this.monitor.reconnectAttempts>0}isProtocolSupported(){return Ls.call(bb,this.getProtocol())>=0}isState(...t){return Ls.call(t,this.getState())>=0}getState(){if(this.webSocket){for(let t in rn.WebSocket)if(rn.WebSocket[t]===this.webSocket.readyState)return t.toLowerCase()}return null}installEventHandlers(){for(let t in this.events){const r=this.events[t].bind(this);this.webSocket[`on${t}`]=r;}}uninstallEventHandlers(){for(let t in this.events)this.webSocket[`on${t}`]=function(){};}};Wo.reopenDelay=500,Wo.prototype.events={message(e){if(!this.isProtocolSupported())return;const{identifier:t,message:r,reason:n,reconnect:o,type:a}=JSON.parse(e.data);switch(a){case ar.welcome:return this.triedToReconnect()&&(this.reconnectAttempted=!0),this.monitor.recordConnect(),this.subscriptions.reload();case ar.disconnect:return W.log(`Disconnecting. Reason: ${n}`),this.close({allowReconnect:o});case ar.ping:return this.monitor.recordPing();case ar.confirmation:return this.subscriptions.confirmSubscription(t),this.reconnectAttempted?(this.reconnectAttempted=!1,this.subscriptions.notify(t,"connected",{reconnected:!0})):this.subscriptions.notify(t,"connected",{reconnected:!1});case ar.rejection:return this.subscriptions.reject(t);default:return this.subscriptions.notify(t,"received",r)}},open(){if(W.log(`WebSocket onopen event, using '${this.getProtocol()}' subprotocol`),this.disconnected=!1,!this.isProtocolSupported())return W.log("Protocol is unsupported. Stopping monitor and disconnecting."),this.close({allowReconnect:!1})},close(e){if(W.log("WebSocket onclose event"),!this.disconnected)return this.disconnected=!0,this.monitor.recordDisconnect(),this.subscriptions.notifyAll("disconnected",{willAttemptReconnect:this.monitor.isRunning()})},error(){W.log("WebSocket onerror event");}};const yb=function(e,t){if(t!=null)for(let r in t){const n=t[r];e[r]=n;}return e};let _b=class{constructor(t,r={},n){this.consumer=t,this.identifier=JSON.stringify(r),yb(this,n);}perform(t,r={}){return r.action=t,this.send(r)}send(t){return this.consumer.send({command:"message",identifier:this.identifier,data:JSON.stringify(t)})}unsubscribe(){return this.consumer.subscriptions.remove(this)}},$b=class{constructor(t){this.subscriptions=t,this.pendingSubscriptions=[];}guarantee(t){this.pendingSubscriptions.indexOf(t)==-1?(W.log(`SubscriptionGuarantor guaranteeing ${t.identifier}`),this.pendingSubscriptions.push(t)):W.log(`SubscriptionGuarantor already guaranteeing ${t.identifier}`),this.startGuaranteeing();}forget(t){W.log(`SubscriptionGuarantor forgetting ${t.identifier}`),this.pendingSubscriptions=this.pendingSubscriptions.filter(r=>r!==t);}startGuaranteeing(){this.stopGuaranteeing(),this.retrySubscribing();}stopGuaranteeing(){clearTimeout(this.retryTimeout);}retrySubscribing(){this.retryTimeout=setTimeout(()=>{this.subscriptions&&typeof this.subscriptions.subscribe=="function"&&this.pendingSubscriptions.map(t=>{W.log(`SubscriptionGuarantor resubscribing ${t.identifier}`),this.subscriptions.subscribe(t);});},500);}},wb=class{constructor(t){this.consumer=t,this.guarantor=new $b(this),this.subscriptions=[];}create(t,r){const n=t,o=typeof n=="object"?n:{channel:n},a=new _b(this.consumer,o,r);return this.add(a)}add(t){return this.subscriptions.push(t),this.consumer.ensureActiveConnection(),this.notify(t,"initialized"),this.subscribe(t),t}remove(t){return this.forget(t),this.findAll(t.identifier).length||this.sendCommand(t,"unsubscribe"),t}reject(t){return this.findAll(t).map(r=>(this.forget(r),this.notify(r,"rejected"),r))}forget(t){return this.guarantor.forget(t),this.subscriptions=this.subscriptions.filter(r=>r!==t),t}findAll(t){return this.subscriptions.filter(r=>r.identifier===t)}reload(){return this.subscriptions.map(t=>this.subscribe(t))}notifyAll(t,...r){return this.subscriptions.map(n=>this.notify(n,t,...r))}notify(t,r,...n){let o;return typeof t=="string"?o=this.findAll(t):o=[t],o.map(a=>typeof a[r]=="function"?a[r](...n):void 0)}subscribe(t){this.sendCommand(t,"subscribe")&&this.guarantor.guarantee(t);}confirmSubscription(t){W.log(`Subscription confirmed ${t}`),this.findAll(t).map(r=>this.guarantor.forget(r));}sendCommand(t,r){const{identifier:n}=t;return this.consumer.send({command:r,identifier:n})}},Ab=class{constructor(t){this._url=t,this.subscriptions=new wb(this),this.connection=new Wo(this),this.subprotocols=[];}get url(){return Ob(this._url)}send(t){return this.connection.send(t)}connect(){return this.connection.open()}disconnect(){return this.connection.close({allowReconnect:!1})}ensureActiveConnection(){if(!this.connection.isActive())return this.connection.open()}addSubProtocol(t){this.subprotocols=[...this.subprotocols,t];}};function Ob(e){if(typeof e=="function"&&(e=e()),e&&!/^wss?:/i.test(e)){const t=document.createElement("a");return t.href=e,t.href=t.href,t.protocol=t.protocol.replace("http","ws"),t.href}else return e}let Sb=class jg extends Error{constructor(t){super(t),this.name="MissingConfigError",Object.setPrototypeOf(this,jg.prototype);}},jb=class Cg extends Error{constructor(t){super(t),this.name="MissingTokenError",Object.setPrototypeOf(this,Cg.prototype);}},Cb=class Pg extends Error{constructor(t){super(t),this.name="MissingWebsocketUriError",Object.setPrototypeOf(this,Pg.prototype);}};var Pb=Object.defineProperty,Rs=Object.getOwnPropertySymbols,Ib=Object.prototype.hasOwnProperty,Eb=Object.prototype.propertyIsEnumerable,Fs=(e,t,r)=>t in e?Pb(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Qs=(e,t)=>{for(var r in t||(t={}))Ib.call(t,r)&&Fs(e,r,t[r]);if(Rs)for(var r of Rs(t))Eb.call(t,r)&&Fs(e,r,t[r]);return e};const Ho=()=>{};let Tb=class{constructor(t){this.timeout=6e4,this._socketConnected=!1,this.handleConnected=r=>(this._socketConnected||(this._socketConnected=!0),r),this.handleDisconnected=(r,n)=>(this._socketConnected&&(this._socketConnected=!1,setTimeout(()=>{n&&n(),this.socketTimeOut();},this.timeout)),r),this._config=t;}get id(){var t,r;return (r=(t=this._config)==null?void 0:t.id)!=null?r:""}start(){return this.validate(),this.connect()}validate(){if(!this._config)throw new Sb("config is required");if(!this._config.token)throw new jb("config.token is required");if(!this._config.wsBaseUri)throw new Cb("config.wsBaseUri is required")}connect(){let t=this.cable;return t&&t!==this.cable?(this.cable=t,this.cable):(t=new Ab(`${this._config.wsBaseUri}?token=${this._config.token}`),this.cable=t,t.ensureActiveConnection(),this.cable)}getHandlers(t){var r,n,o;const a={received:(r=t==null?void 0:t.received)!=null?r:Ho,connected:(n=t==null?void 0:t.connected)!=null?n:Ho,disconnected:(o=t==null?void 0:t.disconnected)!=null?o:Ho},i=this.handleDisconnected(a.disconnected),s=this.handleConnected(a.connected);return {received:a.received,connected:s,disconnected:i}}subscribe(t,r,n){const o=this.cable;if(!o)return;r?r=Qs({token:this._config.token},r):r={token:this._config.token};const a=Qs({channel:t},r),i=this.getHandlers(n);return o.subscriptions.create(a,i)}disconnect(t=!0){return this.cleanup(this.cable,t)}cleanup(t,r){t&&t.disconnect(),r&&(this.cable=void 0);}socketTimeOut(){this._socketConnected||this.cleanup(this.cable,!1);}};const Zo=K("answers/Push"),qb=K("answersQueue/Check"),on=K("queues/QueueAnswer"),Us=K("auth/NotifyAuthFailed"),Vs=K("quoteResources/Timeout",()=>({payload:{updatedAt:new Date}})),Bs=K("reset/ResetApplication"),ir=K("reset/ResetStore"),Gs=K("selectVehicle/Reset"),an=K("websocket/ConnectionEvent"),Ko=K("websocket/SetActiveSocketId"),ge=K("websocket/ReceiveBatchWebsocket");var xb=(e=>(e.LIGHT="Light",e.COLOR="Color",e))(xb||{}),kb=(e=>(e.ApplicationForms="application_forms",e.PartnerPhoneCalls="partner_phone_calls",e.FeatureCheck="features",e.Questions="questions",e.Rules="rules",e.RuleExcludedChoices="rule_excluded_choices",e.SearchableQuestions="searchable_questions",e.SendToExchange="send_to_exchange",e.TakeOver="take_over",e))(kb||{}),zs=(e=>(e.NotConnected="NotConnected",e.Connected="Connected",e.Disconnected="Disconnected",e.Connecting="Connecting",e.Disconnecting="Disconnecting",e))(zs||{}),Db=(e=>(e.Create="create",e.Index="index",e.Show="show",e.Update="update",e))(Db||{}),sn=(e=>(e.Development="development",e.Alpha="alpha",e.Beta="beta",e.Production="prod",e))(sn||{}),Mb=(e=>(e.Accepted="Accepted",e.Expired="Expired",e.Not_Created="Not created",e.Queued="Queued to send",e.Sent="Sent",e))(Mb||{}),Nb=(e=>(e.INVITE="invite",e.RESET="reset",e))(Nb||{}),Lb=(e=>(e.token="token",e.cableToken="cable_token",e.user="user",e.state="state",e.expiresAt="expiresAt",e.refreshAt="refreshAt",e.impersonated="impersonated",e.terminalTheme="terminalTheme",e))(Lb||{}),sr=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Configured="Configured",e.Error="Error",e))(sr||{}),Ws=(e=>(e.Healthy="Healthy",e.Error="Error",e))(Ws||{}),Rb=(e=>(e.OAuth="OAuth",e.Login="Login",e))(Rb||{}),Fb=(e=>(e.storageKey="BP_SDK_ANALYTICS_VARS",e.additionalQuoteOnline="onlinebuy",e.additionalQuoteOffline="nobuy",e))(Fb||{}),Qb=(e=>(e.activeIdUpdate="activeIdUpdate",e.activeQuestionSetChange="activeQuestionSetChange",e.addItem="addItem",e.answerStateChanged="answerStateChanged",e.bpAnalyticsTrack="bpAnalyticsTrack",e.closePanel="closePanel",e.deleteItem="deleteItem",e.deletePool="deletePool",e.epFieldChanged="epFieldChanged",e.errorsUpdate="errorsUpdate",e.fakeValueUpdate="fakeValueUpdate",e.filterUpdate="filterUpdate",e.formLoaded="formLoaded",e.nudgeAnswer="nudgeAnswer",e.openPanel="openPanel",e.placesUpdate="placesUpdate",e.questionSetAdded="questionSetAdded",e.questionSetCompletionChanged="questionSetCompletionChanged",e.quoteFailure="quoteFailure",e.quotesChanged="quotesChanged",e.quoteSuccess="quoteSuccess",e.removeActiveAnswer="removeActiveAnswer",e.saveAnswer="saveAnswer",e.saveVehicle="saveVehicle",e.sdkError="sdkError",e.tabChanged="tabChanged",e.updateActiveAnswer="updateActiveAnswer",e.userAuthenticationChanged="userAuthenticationChanged",e.valueUpdate="valueUpdate",e.vinUpdate="vinUpdate",e.ymmcUpdate="ymmcUpdate",e))(Qb||{}),Ub=(e=>(e.vin="vin",e.ymm="ymm",e.trailers="Trailers",e))(Ub||{}),Vb=(e=>(e.trailers="trailers",e))(Vb||{}),Bb=(e=>(e.Configuration="configuration",e.Make="make",e.Model="model",e.Year="year",e))(Bb||{}),Gb=(e=>(e.Address="address",e.Accordion="accordion",e.AccumulatePercentage="accumulate_percentage",e.Vehicle="vehicle",e.Driver="driver",e.Stepper="stepper",e.Step="step",e.Coverage="coverage",e.AutoCollapsedAccordion="auto_collapsed_accordion",e))(Gb||{}),O=(e=>(e.Address="address",e.Anchor="anchor",e.CheckBox="checkbox",e.CoverageType="coverage_type",e.Currency="currency",e.Date="date",e.DeletePool="delete_pool",e.Dialog="dialog",e.DriversLicense="drivers_license",e.DropDown="dropdown",e.Email="email",e.FederalEmployeeIdentification="fein",e.JSON="json",e.Label="label",e.Masked="masked",e.Number="number",e.Object="object",e.Percentage="percentage",e.Phone="phone",e.Radio="radio",e.Select="select",e.SocialSecurityNumber="ssn",e.Step="step",e.Tenant="tenant",e.Text="text",e.Vehicle="vehicle",e.Url="url",e.VIN="vin",e.Year="year",e.ZipCode="zipcode",e))(O||{}),Yo=(e=>(e.Accumulator="accumulator",e.AddressJSON="address_json",e.BusinessClass="business_class",e.City="city",e.CommercialAutoACV="ca_acv",e.CommercialAutoCategory="ca_category",e.CommercialAutoCount="ca_count",e.CommercialAutoDrivetrain="ca_drivetrain",e.CommercialAutoFlow="ca_flow",e.CommercialAutoGCW="ca_gcw",e.CommercialAutoGVW="ca_gvw",e.CommercialAutoJson="ca_json",e.CommercialAutoMake="ca_make",e.CommercialAutoModel="ca_model",e.CommercialAutoMSRP="ca_msrp",e.CommercialAutoSubModel="ca_submodel",e.CommercialAutoTractor="ca_tractor",e.CommercialAutoVIN="ca_vin",e.CommercialAutoYear="ca_year",e.County="county",e.DriversLicenseState="drivers_license_state",e.Indent="indent",e.Increment="increment",e.JornayaLeadID="jornaya_lead_id",e.LinesOfBusiness="lines_of_business",e.LinesOfBusinessAdditional="lines_of_business_additional",e.MotorVehicleReportComplete="mvr_complete",e.MotorVehicleReportState="mvr_state",e.MotorVehicleReportCondition="mvr_report_condition",e.State="state",e.Street1="street_1",e.Street2="street_2",e.ZipCode="zipcode",e))(Yo||{}),zb=(e=>(e.CoverageLimit="coverage limit",e))(zb||{}),Wb=(e=>(e.MaximumDate="maxDate",e.MaximumValue="max",e.MinimumDate="minDate",e.MinimumValue="min",e.Pattern="pattern",e.Required="required",e))(Wb||{}),Hb=(e=>(e.Acord="acord",e.Acord125="acord_125",e.Acord126="acord_126",e.Acord130="acord_130",e))(Hb||{}),Zb=(e=>(e.BP_QUOTABLE="bp_quotable",e.OFFLINE="offline",e.ONLINE="online",e))(Zb||{}),Kb=(e=>(e.Pending="pending",e.Queued="queued",e.Executing="executing",e.Completed="completed",e.Failed="failed",e.Aborted="aborted",e))(Kb||{}),Yb=(e=>(e.ClueReport="ClueReport",e.MotorVehicleReport="MotorVehicleReport",e.RequestPrefill="RequestPrefill",e.DriverScore="DriverScore",e.JornayaAudit="JornayaAudit",e.DotNumber="DotNumber",e))(Yb||{}),cn=(e=>(e.Incomplete="incomplete",e.InProgress="in_progress",e.Completed="completed",e.Quoted="quoted",e))(cn||{}),Jb=(e=>(e.not_permitted_to_bind="not_permitted_to_bind",e.sent_to_wholesaler="sent_to_wholesaler",e))(Jb||{}),Xb=(e=>(e.dialog="dialog",e.wholesale_workflow="wholesale_workflow",e))(Xb||{}),ey=(e=>(e.NON_ADMITTED="non_admitted",e.WHOLESALER="wholesaler",e.INSTANT_ENROLLMENT="instant_enrollment",e.ADDITIONAL_MARKET="additional_market",e.OPT_IN_QUOTE_FLOW="opt_in_quote_flow",e))(ey||{});var ty=(e=>(e.Answer="answer",e.Application="application_form",e.QuoteRequest="quote_request",e))(ty||{}),Hs=(e=>(e.unsent="unsent",e.in_progress="in_progress",e.sent="sent",e.completed="completed",e.failed="failed",e.timed_out="timed_out",e.ineligible="ineligible",e.referral="referral",e.bound="bound",e))(Hs||{}),ry=(e=>(e.AddQuestionSet="application_forms_question_set",e.Answer="answer",e.Appetite="appetite",e.ApplicationForm="application_form",e.ApplicationFormPercentCompleted="application_form_percent_completed",e.ApplicationFormProcess="application_form_process",e.Batch="batch",e.CarrierProducts="eligible_carriers_products",e.Message="message",e.Notification="notification",e.QuoteRequest="quote_request",e.RealTimeEligibility="real_time_eligibility",e.RemoveQuestionSet="destroy_application_forms_question_set",e))(ry||{}),ny=(e=>(e.AcordForm="acord_form_pdf",e.AgentQuoteStart="agent_quote_start",e.ClientQuoteStart="client_quote_start",e.Storefront="storefront",e))(ny||{}),un=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Error="Error",e))(un||{});const oy={configurations:{},makes:{},models:{},selection:{},vin:{},years:{}},ay={list:[],isLoaded:!1};var iy=Object.defineProperty,Zs=Object.getOwnPropertySymbols,sy=Object.prototype.hasOwnProperty,cy=Object.prototype.propertyIsEnumerable,Ks=(e,t,r)=>t in e?iy(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Ys=(e,t)=>{for(var r in t||(t={}))sy.call(t,r)&&Ks(e,r,t[r]);if(Zs)for(var r of Zs(t))cy.call(t,r)&&Ks(e,r,t[r]);return e};Ys({},oy),Ys({},ay);var uy=Object.defineProperty,Js=Object.getOwnPropertySymbols,ly=Object.prototype.hasOwnProperty,dy=Object.prototype.propertyIsEnumerable,Xs=(e,t,r)=>t in e?uy(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Ge=(e,t)=>{for(var r in t||(t={}))ly.call(t,r)&&Xs(e,r,t[r]);if(Js)for(var r of Js(t))dy.call(t,r)&&Xs(e,r,t[r]);return e};const py={configurations:{},makes:{},models:{},selection:{},vins:{},years:{}},Jo={state:un.NotLoaded,map:{}},ec={state:un.NotLoaded},fy={form:un.NotLoaded},vy={answers:Ge({},Jo),application:Ge({},fy),bindQuotes:{},choices:{},carriersProducts:Ge({},ec),questionPools:Ge({},Jo),questionSets:Ge({},Jo),quotes:Ge({},ec),refilters:{},vehicles:Ge({},py)};var hy=Object.defineProperty,tc=Object.getOwnPropertySymbols,gy=Object.prototype.hasOwnProperty,my=Object.prototype.propertyIsEnumerable,rc=(e,t,r)=>t in e?hy(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,nc=(e,t)=>{for(var r in t||(t={}))gy.call(t,r)&&rc(e,r,t[r]);if(tc)for(var r of tc(t))my.call(t,r)&&rc(e,r,t[r]);return e};const by={auth:sr.NotLoaded,config:sr.NotLoaded,health:Ws.Healthy,state:sr.NotLoaded,user:sr.NotLoaded,socket:zs.NotConnected,socketId:""};nc({},vy),nc({},by);var yy=(e=>(e.QuestionSetSaved="QuestionSetSaved",e))(yy||{});function C(e){return e.uri?e.uri:e.env===sn.Production||e.env===void 0?"https://partner-engine.boldpenguin.com":e.env===sn.Development?"http://localhost:8080":`https://partner-engine-uat.${e.env}.boldpenguin.com`}var _y="10.25.0-beta.0",$y=Object.defineProperty,wy=Object.defineProperties,Ay=Object.getOwnPropertyDescriptors,oc=Object.getOwnPropertySymbols,Oy=Object.prototype.hasOwnProperty,Sy=Object.prototype.propertyIsEnumerable,ac=(e,t,r)=>t in e?$y(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,jy=(e,t)=>{for(var r in t||(t={}))Oy.call(t,r)&&ac(e,r,t[r]);if(oc)for(var r of oc(t))Sy.call(t,r)&&ac(e,r,t[r]);return e},Cy=(e,t)=>wy(e,Ay(t)),Z=(e=>(e.GET="GET",e.POST="POST",e.PUT="PUT",e.PATCH="PATCH",e.DELETE="DELETE",e))(Z||{});function P(e,t="GET",r){const n=new Headers({Accept:"application/json","Content-Type":"application/json","x-sdk-version":_y});e&&n.append("Authorization",`Bearer ${e}`);const o={credentials:"include",method:t,headers:n};return r&&(o.body=JSON.stringify(r)),o}function Me(e){let t="";const r=Object.entries(e!=null?e:{}).filter(([,n])=>n);return r.length>0&&(t=`?${r.map(([n,o])=>{const a=encodeURIComponent(n),i=encodeURIComponent(o);return `${a}=${i}`}).join("&")}`),t}const I=(e,t,r)=>new Promise((n,o)=>{fetch(e,t).then(a=>{let i;return r!=null&&r.expectsEmptyResponse?i=Ty:r!=null&&r.couldHaveEmptyResponse?i=Iy:r!=null&&r.expectBlob?i=qy:i=Ey,i(a,n,o)}).catch(a=>Py(a,o));});function Py(e,t){let r={error:e};e.error&&(r=e),t(r);}function ic(e,t,r){const n=Cy(jy({},t||{}),{error:e});r(n);}function Iy(e,t,r){return e.ok?t(cr(e)):r(cr(e))}function Ey(e,t,r){return e.json().then(n=>{const o=cr(e,n);(e.ok?t:r)(o);}).catch(n=>ic(n,e,r))}function Ty(e,t,r){const n=cr(e);(e.ok?t:r)(n);}function qy(e,t,r){return e.blob().then(n=>{const o=cr(e,n);(e.ok?t:r)(o);}).catch(n=>ic(n,e,r))}function cr(e,t){return {status:e.status,redirected:e.redirected,url:e.url,metadata:{searchTotalItems:Number(e.headers.get("x-total"))},parsedBody:t}}let xy=class Ig{static updateAnswer(t,r){const n=`${C(r)}/application_forms/${t.application_form_id}/answers/${t.id}`;return I(n,P(r.token,Z.PATCH,Ig.transformAnswerData(t)))}static transformAnswerData(t){let r;return t.question.field_type===O.Select?r=null:t.question.field_type===O.CheckBox?r=t.value?"1":null:r=t.value,{answer:{choice_id:t.choice_id,choice_ids:t.choice_ids,value:r,previous_version:t.version,question_id:t.question.id}}}},ze=class{static getApplicationForm(t,r){const n=`${C(r)}/application_forms/${t}`;return I(n,P(r.token))}static getLatestApplicationForm(t,r){const n=`${C(r)}/application_forms/${t}/latest`;return I(n,P(r.token))}static getApplicationFormEitScenario(t,r){const n=`${C(r)}/application_forms/${t}/export/eit_scenario`;return I(n,P(r.token),{expectBlob:!0})}static createExchangeApplicationForm(t,r){const n=`${C(r)}/exchange_application_forms`;return I(n,P(r.token,Z.POST,t))}static createApplicationForm(t,r){const n=`${C(r)}/application_forms`;return I(n,P(r.token,Z.POST,t))}static cloneApplicationForm(t,r){const n=`${C(r)}/application_forms/${t}/clones`;return I(n,P(r.token,Z.POST))}static updateApplicationForm(t,r){const n=`${C(r)}/application_forms/${t.id}`;return I(n,P(r.token,Z.PATCH,t))}static deleteApplicationForm(t,r){const n=`${C(r)}/application_forms/${t.id}`;return I(n,P(r.token,Z.DELETE))}static getApplicationFormFromToken(t,r){const n=`${C(r)}/tokens/${t}`;return I(n,P(r.token,Z.GET))}static getTokenForAuthGuestFlow(t){const r=C(t);return I(`${r}/tenants/${t.tenantId}/application_forms/guest`,P(void 0,Z.POST,{source:t.applicationSource,source_metadata:t.applicationMetadata,application_form:t.applicationForm}))}static searchApplicationForms(t,{sortColumn:r="",sortOrder:n="asc",pageNumber:o=0,pageSize:a=10,answers:i=[],q:s=""}={}){const c=Me({column:r,order:n,page:o.toString(),per_page:a.toString(),questions:this.serializeAnswers(i),q:s}),u=`${C(t)}/application_forms/${c}`;return I(u,P(t.token))}static getEligibilityResults(t,r){const n=`${C(r)}/application_forms/${t}/eligibility_results`;return I(n,P(r.token,Z.GET))}static updateApplicationToNextBucket(t,r,n){const o=`${C(n)}/application_forms/${t}/add_next_bucket`;return I(o,P(n.token,Z.POST,{product_codes:r}),{expectsEmptyResponse:!0})}static createCurrentTierSkips(t,r){const n=`${C(r)}/application_forms/${t.id}/current_tier_skips`;return I(n,P(r.token,Z.POST,t))}static serializeAnswers(t){const r={};for(const n of t)r[n.question.id]={value:n.value,choice_ids:n.choice_ids,choice:n.choice_id};return JSON.stringify(r)}},Xo=class{static getQuestionSet(t,r,n){const o=`${C(n)}/application_forms/${t.id}/application_forms_question_sets/${r.id}`;return I(o,P(n.token))}static updateQuestionSet(t,r){const n=`${C(r)}/application_forms/${t.application_form_id}/application_forms_question_sets/${t.id}`;return I(n,P(r.token,Z.PATCH,this.transformQuestionSetDataForUpdate(t)))}static createQuestionSet(t,r){const n=`${C(r)}/application_forms/${t.application_form_id}/application_forms_question_sets`;return I(n,P(r.token,Z.POST,this.transformQuestionSetDataForCreate(t)))}static transformQuestionSetDataForUpdate(t){return {application_forms_question_set:{is_completed:t.is_completed}}}static transformQuestionSetDataForCreate(t){return {application_forms_question_set:{question_set:{code:t.question_set.code}}}}};function sc(e){const t={};return Object.keys(e).forEach(r=>{e[r]&&(t[r]=e[r]);}),t}let cc=class Zi{static getChoices(t,r){const n=`${C(r)}/questions/${t}/choices`;return I(n,P(r.token))}static getChoice(t,r){const n=`${C(r)}/choices/${t}`;return I(n,P(r.token))}static addChoice(t,r){return I(`${C(r)}/choices`,P(r.token,Z.POST,Zi.toChoiceObject(t)))}static updateChoice(t,r){const n=`${C(r)}/choices/${t.id}`;return I(n,P(r.token,Z.PATCH,Zi.toChoiceObject(t)))}static searchChoices(t,r,n){const o=sc(r),a=`${C(n)}/questions/${t}/choices${Me(o)}`;return I(a,P(n.token))}static searchChoicesWithNudges(t,r,n){const o=sc(r),a=`${C(n)}/questions/${t}/choices${Me(o)}`;return I(a,P(n.token))}static toChoiceObject(t){return {choice:{value:t.value,alias:t.alias,question_id:t.question_id,order:t.order}}}},ky=class{static getFeatures(t){const r=`${C(t)}/features`;return I(r,P(t.token,Z.GET))}},uc=class{static addQuestionPool({application_id:t,config:r,question_pool_id:n,parent_answer_pool_id:o}){const a=`${C(r)}/application_forms/${t}/answer_pools`;return I(a,P(r.token,Z.POST,{question_pool_id:n,parent_answer_pool_id:o}))}static removeQuestionPool({application_id:t,config:r,answer_pool_id:n}){const o=`${C(r)}/application_forms/${t}/answer_pools/${n}`;return I(o,P(r.token,Z.DELETE))}},Dy=class{static getApplicationFormCarriersProducts(t,r){const n=`${C(r)}/application_forms/${t}/carriers_products`;return I(n,P(r.token))}},lc=class Eg{static getQuoteRequests(t,r){const n=`${C(r)}/application_forms/${t}/quote_requests`;return I(n,P(r.token))}static bindQuoteAction(t,r,n){const o=Me({bind_action:t.value}),a=`${C(n)}/application_forms/${r.application_form_id}/quote_requests/${r.id}/bind/${o}`;return I(a,P(n.token),{couldHaveEmptyResponse:!0})}static bindQuote(t,r,n){const o=`${C(n)}/application_forms/${r}/quote_requests/${t}/bind`;return I(o,P(n.token))}static quoteDetails(t,r){const n=`${C(r)}/application_forms/${t.application_form_id}/quote_requests/${t.id}/details`;return I(n,P(r.token))}static updateQuote(t,r){const n=`${C(r)}/application_forms/${t.application_form_id}/quote_requests/${t.id}`;return I(n,P(r.token,Z.PATCH,Eg.toQuoteRequestObject(t)))}static toQuoteRequestObject(t){return {quote_request:{focused:t.focused}}}},me=class{static getYears(t){const r=`${C(t)}/vehicles/years`;return I(r,P(t.token))}static getMakes(t,r,n){const o=`${C(r)}/vehicles/years/${t}/makes${Me({type:n==null?void 0:n.type,name:n==null?void 0:n.name})}`;return I(o,P(r.token))}static getModels(t,r,n,o){const a=`${C(n)}/vehicles/years/${t}/makes/${r}/models${Me({type:o})}`;return I(a,P(n.token))}static getConfigurations(t,r,n,o,a){const i=`${C(o)}/vehicles/years/${t}/makes/${r}/models/${n}/configurations${Me({type:a})}`;return I(i,P(o.token))}static getVinDetails(t,r){const n=`${C(r)}/vehicles/vins/${t}`;return I(n,P(r.token))}static selectVehicle(t){var r,n,o;const{appId:a,questionGroupId:i,configurationId:s,flowType:c,config:u,answerPoolId:l,type:d,vehicle:p}=t,h={question_group_id:i,configuration_id:s,flow:c,answer_pool_id:l};l||delete h.answer_pool_id;const v=`${C(u)}/application_forms/${a}/vehicles${Me({type:d,year:(r=p==null?void 0:p.yearId)!=null?r:void 0,make:(n=p==null?void 0:p.makeId)!=null?n:void 0,model:(o=p==null?void 0:p.modelId)!=null?o:void 0})}`;return I(v,P(u.token,Z.POST,h))}};var My=(e=>(e.LIGHT="Light",e.COLOR="Color",e))(My||{}),ln=(e=>(e.ApplicationForms="application_forms",e.PartnerPhoneCalls="partner_phone_calls",e.FeatureCheck="features",e.Questions="questions",e.Rules="rules",e.RuleExcludedChoices="rule_excluded_choices",e.SearchableQuestions="searchable_questions",e.SendToExchange="send_to_exchange",e.TakeOver="take_over",e))(ln||{}),dc=(e=>(e.NotConnected="NotConnected",e.Connected="Connected",e.Disconnected="Disconnected",e.Connecting="Connecting",e.Disconnecting="Disconnecting",e))(dc||{}),Ny=(e=>(e.Create="create",e.Index="index",e.Show="show",e.Update="update",e))(Ny||{}),pc=(e=>(e.Development="development",e.Alpha="alpha",e.Beta="beta",e.Production="prod",e))(pc||{}),Ly=(e=>(e.Accepted="Accepted",e.Expired="Expired",e.Not_Created="Not created",e.Queued="Queued to send",e.Sent="Sent",e))(Ly||{}),Ry=(e=>(e.INVITE="invite",e.RESET="reset",e))(Ry||{}),At=(e=>(e.token="token",e.cableToken="cable_token",e.user="user",e.state="state",e.expiresAt="expiresAt",e.refreshAt="refreshAt",e.impersonated="impersonated",e.terminalTheme="terminalTheme",e))(At||{}),ur=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Configured="Configured",e.Error="Error",e))(ur||{}),fc=(e=>(e.Healthy="Healthy",e.Error="Error",e))(fc||{}),Fy=(e=>(e.OAuth="OAuth",e.Login="Login",e))(Fy||{}),Qy=(e=>(e.storageKey="BP_SDK_ANALYTICS_VARS",e.additionalQuoteOnline="onlinebuy",e.additionalQuoteOffline="nobuy",e))(Qy||{}),Uy=(e=>(e.activeIdUpdate="activeIdUpdate",e.activeQuestionSetChange="activeQuestionSetChange",e.addItem="addItem",e.answerStateChanged="answerStateChanged",e.bpAnalyticsTrack="bpAnalyticsTrack",e.closePanel="closePanel",e.deleteItem="deleteItem",e.deletePool="deletePool",e.epFieldChanged="epFieldChanged",e.errorsUpdate="errorsUpdate",e.fakeValueUpdate="fakeValueUpdate",e.filterUpdate="filterUpdate",e.formLoaded="formLoaded",e.nudgeAnswer="nudgeAnswer",e.openPanel="openPanel",e.placesUpdate="placesUpdate",e.questionSetAdded="questionSetAdded",e.questionSetCompletionChanged="questionSetCompletionChanged",e.quoteFailure="quoteFailure",e.quotesChanged="quotesChanged",e.quoteSuccess="quoteSuccess",e.removeActiveAnswer="removeActiveAnswer",e.saveAnswer="saveAnswer",e.saveVehicle="saveVehicle",e.sdkError="sdkError",e.tabChanged="tabChanged",e.updateActiveAnswer="updateActiveAnswer",e.userAuthenticationChanged="userAuthenticationChanged",e.valueUpdate="valueUpdate",e.vinUpdate="vinUpdate",e.ymmcUpdate="ymmcUpdate",e))(Uy||{}),Vy=(e=>(e.vin="vin",e.ymm="ymm",e.trailers="Trailers",e))(Vy||{}),By=(e=>(e.trailers="trailers",e))(By||{}),Gy=(e=>(e.Configuration="configuration",e.Make="make",e.Model="model",e.Year="year",e))(Gy||{}),ea=(e=>(e.Address="address",e.Accordion="accordion",e.AccumulatePercentage="accumulate_percentage",e.Vehicle="vehicle",e.Driver="driver",e.Stepper="stepper",e.Step="step",e.Coverage="coverage",e.AutoCollapsedAccordion="auto_collapsed_accordion",e))(ea||{}),w=(e=>(e.Address="address",e.Anchor="anchor",e.CheckBox="checkbox",e.CoverageType="coverage_type",e.Currency="currency",e.Date="date",e.DeletePool="delete_pool",e.Dialog="dialog",e.DriversLicense="drivers_license",e.DropDown="dropdown",e.Email="email",e.FederalEmployeeIdentification="fein",e.JSON="json",e.Label="label",e.Masked="masked",e.Number="number",e.Object="object",e.Percentage="percentage",e.Phone="phone",e.Radio="radio",e.Select="select",e.SocialSecurityNumber="ssn",e.Step="step",e.Tenant="tenant",e.Text="text",e.Vehicle="vehicle",e.Url="url",e.VIN="vin",e.Year="year",e.ZipCode="zipcode",e))(w||{}),Ot=(e=>(e.Accumulator="accumulator",e.AddressJSON="address_json",e.BusinessClass="business_class",e.City="city",e.CommercialAutoACV="ca_acv",e.CommercialAutoCategory="ca_category",e.CommercialAutoCount="ca_count",e.CommercialAutoDrivetrain="ca_drivetrain",e.CommercialAutoFlow="ca_flow",e.CommercialAutoGCW="ca_gcw",e.CommercialAutoGVW="ca_gvw",e.CommercialAutoJson="ca_json",e.CommercialAutoMake="ca_make",e.CommercialAutoModel="ca_model",e.CommercialAutoMSRP="ca_msrp",e.CommercialAutoSubModel="ca_submodel",e.CommercialAutoTractor="ca_tractor",e.CommercialAutoVIN="ca_vin",e.CommercialAutoYear="ca_year",e.County="county",e.DriversLicenseState="drivers_license_state",e.Indent="indent",e.Increment="increment",e.JornayaLeadID="jornaya_lead_id",e.LinesOfBusiness="lines_of_business",e.LinesOfBusinessAdditional="lines_of_business_additional",e.MotorVehicleReportComplete="mvr_complete",e.MotorVehicleReportState="mvr_state",e.MotorVehicleReportCondition="mvr_report_condition",e.State="state",e.Street1="street_1",e.Street2="street_2",e.ZipCode="zipcode",e))(Ot||{}),zy=(e=>(e.CoverageLimit="coverage limit",e))(zy||{}),ee=(e=>(e.MaximumDate="maxDate",e.MaximumValue="max",e.MinimumDate="minDate",e.MinimumValue="min",e.Pattern="pattern",e.Required="required",e))(ee||{}),Wy=(e=>(e.Acord="acord",e.Acord125="acord_125",e.Acord126="acord_126",e.Acord130="acord_130",e))(Wy||{}),Hy=(e=>(e.BP_QUOTABLE="bp_quotable",e.OFFLINE="offline",e.ONLINE="online",e))(Hy||{}),Zy=(e=>(e.Pending="pending",e.Queued="queued",e.Executing="executing",e.Completed="completed",e.Failed="failed",e.Aborted="aborted",e))(Zy||{}),Ky=(e=>(e.ClueReport="ClueReport",e.MotorVehicleReport="MotorVehicleReport",e.RequestPrefill="RequestPrefill",e.DriverScore="DriverScore",e.JornayaAudit="JornayaAudit",e.DotNumber="DotNumber",e))(Ky||{}),dn=(e=>(e.Incomplete="incomplete",e.InProgress="in_progress",e.Completed="completed",e.Quoted="quoted",e))(dn||{}),Yy=(e=>(e.not_permitted_to_bind="not_permitted_to_bind",e.sent_to_wholesaler="sent_to_wholesaler",e))(Yy||{}),Jy=(e=>(e.dialog="dialog",e.wholesale_workflow="wholesale_workflow",e))(Jy||{}),Xy=(e=>(e.NON_ADMITTED="non_admitted",e.WHOLESALER="wholesaler",e.INSTANT_ENROLLMENT="instant_enrollment",e.ADDITIONAL_MARKET="additional_market",e.OPT_IN_QUOTE_FLOW="opt_in_quote_flow",e))(Xy||{});const e_=[w.Number,w.Currency,w.Percentage,w.Year];var t_=(e=>(e.Answer="answer",e.Application="application_form",e.QuoteRequest="quote_request",e))(t_||{}),vc=(e=>(e.unsent="unsent",e.in_progress="in_progress",e.sent="sent",e.completed="completed",e.failed="failed",e.timed_out="timed_out",e.ineligible="ineligible",e.referral="referral",e.bound="bound",e))(vc||{}),r_=(e=>(e.AddQuestionSet="application_forms_question_set",e.Answer="answer",e.Appetite="appetite",e.ApplicationForm="application_form",e.ApplicationFormPercentCompleted="application_form_percent_completed",e.ApplicationFormProcess="application_form_process",e.Batch="batch",e.CarrierProducts="eligible_carriers_products",e.Message="message",e.Notification="notification",e.QuoteRequest="quote_request",e.RealTimeEligibility="real_time_eligibility",e.RemoveQuestionSet="destroy_application_forms_question_set",e))(r_||{}),n_=(e=>(e.AcordForm="acord_form_pdf",e.AgentQuoteStart="agent_quote_start",e.ClientQuoteStart="client_quote_start",e.Storefront="storefront",e))(n_||{}),fe=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Error="Error",e))(fe||{});const o_={configurations:{},makes:{},models:{},selection:{},vin:{},years:{}},a_={list:[],isLoaded:!1};var i_=Object.defineProperty,hc=Object.getOwnPropertySymbols,s_=Object.prototype.hasOwnProperty,c_=Object.prototype.propertyIsEnumerable,gc=(e,t,r)=>t in e?i_(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,mc=(e,t)=>{for(var r in t||(t={}))s_.call(t,r)&&gc(e,r,t[r]);if(hc)for(var r of hc(t))c_.call(t,r)&&gc(e,r,t[r]);return e};mc({},o_),mc({},a_);var u_=Object.defineProperty,bc=Object.getOwnPropertySymbols,l_=Object.prototype.hasOwnProperty,d_=Object.prototype.propertyIsEnumerable,yc=(e,t,r)=>t in e?u_(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,We=(e,t)=>{for(var r in t||(t={}))l_.call(t,r)&&yc(e,r,t[r]);if(bc)for(var r of bc(t))d_.call(t,r)&&yc(e,r,t[r]);return e};const p_={configurations:{},makes:{},models:{},selection:{},vins:{},years:{}},ta={state:fe.NotLoaded,map:{}},_c={state:fe.NotLoaded},f_={form:fe.NotLoaded},v_={answers:We({},ta),application:We({},f_),bindQuotes:{},choices:{},carriersProducts:We({},_c),questionPools:We({},ta),questionSets:We({},ta),quotes:We({},_c),refilters:{},vehicles:We({},p_)};var h_=Object.defineProperty,$c=Object.getOwnPropertySymbols,g_=Object.prototype.hasOwnProperty,m_=Object.prototype.propertyIsEnumerable,wc=(e,t,r)=>t in e?h_(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Ac=(e,t)=>{for(var r in t||(t={}))g_.call(t,r)&&wc(e,r,t[r]);if($c)for(var r of $c(t))m_.call(t,r)&&wc(e,r,t[r]);return e};const b_={auth:ur.NotLoaded,config:ur.NotLoaded,health:fc.Healthy,state:ur.NotLoaded,user:ur.NotLoaded,socket:dc.NotConnected,socketId:""};Ac({},v_),Ac({},b_);var y_=(e=>(e.QuestionSetSaved="QuestionSetSaved",e))(y_||{}),pn=typeof globalThis!="undefined"?globalThis:typeof window!="undefined"?window:typeof global!="undefined"?global:typeof self!="undefined"?self:{};function lr(e){return e&&e.__esModule&&Object.prototype.hasOwnProperty.call(e,"default")?e.default:e}function __(e,t){for(var r=-1,n=e==null?0:e.length,o=Array(n);++r<n;)o[r]=t(e[r],r,e);return o}var ra=__,$_=Array.isArray,le=$_,w_=typeof pn=="object"&&pn&&pn.Object===Object&&pn,Oc=w_,A_=Oc,O_=typeof self=="object"&&self&&self.Object===Object&&self,S_=A_||O_||Function("return this")(),Oe=S_,j_=Oe,C_=j_.Symbol,dr=C_,Sc=dr,jc=Object.prototype,P_=jc.hasOwnProperty,I_=jc.toString,pr=Sc?Sc.toStringTag:void 0;function E_(e){var t=P_.call(e,pr),r=e[pr];try{e[pr]=void 0;var n=!0;}catch(a){}var o=I_.call(e);return n&&(t?e[pr]=r:delete e[pr]),o}var T_=E_,q_=Object.prototype,x_=q_.toString;function k_(e){return x_.call(e)}var D_=k_,Cc=dr,M_=T_,N_=D_,L_="[object Null]",R_="[object Undefined]",Pc=Cc?Cc.toStringTag:void 0;function F_(e){return e==null?e===void 0?R_:L_:Pc&&Pc in Object(e)?M_(e):N_(e)}var fr=F_;function Q_(e){return e!=null&&typeof e=="object"}var St=Q_,U_=fr,V_=St,B_="[object Symbol]";function G_(e){return typeof e=="symbol"||V_(e)&&U_(e)==B_}var fn=G_,z_=le,W_=fn,H_=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,Z_=/^\w*$/;function K_(e,t){if(z_(e))return !1;var r=typeof e;return r=="number"||r=="symbol"||r=="boolean"||e==null||W_(e)?!0:Z_.test(e)||!H_.test(e)||t!=null&&e in Object(t)}var na=K_;function Y_(e){var t=typeof e;return e!=null&&(t=="object"||t=="function")}var oa=Y_,J_=fr,X_=oa,e$="[object AsyncFunction]",t$="[object Function]",r$="[object GeneratorFunction]",n$="[object Proxy]";function o$(e){if(!X_(e))return !1;var t=J_(e);return t==t$||t==r$||t==e$||t==n$}var Ic=o$,a$=Oe,i$=a$["__core-js_shared__"],s$=i$,aa=s$,Ec=function(){var e=/[^.]+$/.exec(aa&&aa.keys&&aa.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}();function c$(e){return !!Ec&&Ec in e}var u$=c$,l$=Function.prototype,d$=l$.toString;function p$(e){if(e!=null){try{return d$.call(e)}catch(t){}try{return e+""}catch(t){}}return ""}var Tc=p$,f$=Ic,v$=u$,h$=oa,g$=Tc,m$=/[\\^$.*+?()[\]{}|]/g,b$=/^\[object .+?Constructor\]$/,y$=Function.prototype,_$=Object.prototype,$$=y$.toString,w$=_$.hasOwnProperty,A$=RegExp("^"+$$.call(w$).replace(m$,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function O$(e){if(!h$(e)||v$(e))return !1;var t=f$(e)?A$:b$;return t.test(g$(e))}var S$=O$;function j$(e,t){return e==null?void 0:e[t]}var C$=j$,P$=S$,I$=C$;function E$(e,t){var r=I$(e,t);return P$(r)?r:void 0}var He=E$,T$=He,q$=T$(Object,"create"),vn=q$,qc=vn;function x$(){this.__data__=qc?qc(null):{},this.size=0;}var k$=x$;function D$(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var M$=D$,N$=vn,L$="__lodash_hash_undefined__",R$=Object.prototype,F$=R$.hasOwnProperty;function Q$(e){var t=this.__data__;if(N$){var r=t[e];return r===L$?void 0:r}return F$.call(t,e)?t[e]:void 0}var U$=Q$,V$=vn,B$=Object.prototype,G$=B$.hasOwnProperty;function z$(e){var t=this.__data__;return V$?t[e]!==void 0:G$.call(t,e)}var W$=z$,H$=vn,Z$="__lodash_hash_undefined__";function K$(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=H$&&t===void 0?Z$:t,this}var Y$=K$,J$=k$,X$=M$,ew=U$,tw=W$,rw=Y$;function jt(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}jt.prototype.clear=J$,jt.prototype.delete=X$,jt.prototype.get=ew,jt.prototype.has=tw,jt.prototype.set=rw;var nw=jt;function ow(){this.__data__=[],this.size=0;}var aw=ow;function iw(e,t){return e===t||e!==e&&t!==t}var xc=iw,sw=xc;function cw(e,t){for(var r=e.length;r--;)if(sw(e[r][0],t))return r;return -1}var hn=cw,uw=hn,lw=Array.prototype,dw=lw.splice;function pw(e){var t=this.__data__,r=uw(t,e);if(r<0)return !1;var n=t.length-1;return r==n?t.pop():dw.call(t,r,1),--this.size,!0}var fw=pw,vw=hn;function hw(e){var t=this.__data__,r=vw(t,e);return r<0?void 0:t[r][1]}var gw=hw,mw=hn;function bw(e){return mw(this.__data__,e)>-1}var yw=bw,_w=hn;function $w(e,t){var r=this.__data__,n=_w(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this}var ww=$w,Aw=aw,Ow=fw,Sw=gw,jw=yw,Cw=ww;function Ct(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Ct.prototype.clear=Aw,Ct.prototype.delete=Ow,Ct.prototype.get=Sw,Ct.prototype.has=jw,Ct.prototype.set=Cw;var gn=Ct,Pw=He,Iw=Oe,Ew=Pw(Iw,"Map"),ia=Ew,kc=nw,Tw=gn,qw=ia;function xw(){this.size=0,this.__data__={hash:new kc,map:new(qw||Tw),string:new kc};}var kw=xw;function Dw(e){var t=typeof e;return t=="string"||t=="number"||t=="symbol"||t=="boolean"?e!=="__proto__":e===null}var Mw=Dw,Nw=Mw;function Lw(e,t){var r=e.__data__;return Nw(t)?r[typeof t=="string"?"string":"hash"]:r.map}var mn=Lw,Rw=mn;function Fw(e){var t=Rw(this,e).delete(e);return this.size-=t?1:0,t}var Qw=Fw,Uw=mn;function Vw(e){return Uw(this,e).get(e)}var Bw=Vw,Gw=mn;function zw(e){return Gw(this,e).has(e)}var Ww=zw,Hw=mn;function Zw(e,t){var r=Hw(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this}var Kw=Zw,Yw=kw,Jw=Qw,Xw=Bw,e0=Ww,t0=Kw;function Pt(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Pt.prototype.clear=Yw,Pt.prototype.delete=Jw,Pt.prototype.get=Xw,Pt.prototype.has=e0,Pt.prototype.set=t0;var sa=Pt,Dc=sa,r0="Expected a function";function ca(e,t){if(typeof e!="function"||t!=null&&typeof t!="function")throw new TypeError(r0);var r=function(){var n=arguments,o=t?t.apply(this,n):n[0],a=r.cache;if(a.has(o))return a.get(o);var i=e.apply(this,n);return r.cache=a.set(o,i)||a,i};return r.cache=new(ca.Cache||Dc),r}ca.Cache=Dc;var n0=ca,o0=n0,a0=500;function i0(e){var t=o0(e,function(n){return r.size===a0&&r.clear(),n}),r=t.cache;return t}var s0=i0,c0=s0,u0=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,l0=/\\(\\)?/g,d0=c0(function(e){var t=[];return e.charCodeAt(0)===46&&t.push(""),e.replace(u0,function(r,n,o,a){t.push(o?a.replace(l0,"$1"):n||r);}),t}),p0=d0,Mc=dr,f0=ra,v0=le,h0=fn,g0=1/0,Nc=Mc?Mc.prototype:void 0,Lc=Nc?Nc.toString:void 0;function Rc(e){if(typeof e=="string")return e;if(v0(e))return f0(e,Rc)+"";if(h0(e))return Lc?Lc.call(e):"";var t=e+"";return t=="0"&&1/e==-g0?"-0":t}var m0=Rc,b0=m0;function y0(e){return e==null?"":b0(e)}var _0=y0,$0=le,w0=na,A0=p0,O0=_0;function S0(e,t){return $0(e)?e:w0(e,t)?[e]:A0(O0(e))}var Fc=S0,j0=fn,C0=1/0;function P0(e){if(typeof e=="string"||j0(e))return e;var t=e+"";return t=="0"&&1/e==-C0?"-0":t}var bn=P0,I0=Fc,E0=bn;function T0(e,t){t=I0(t,e);for(var r=0,n=t.length;e!=null&&r<n;)e=e[E0(t[r++])];return r&&r==n?e:void 0}var ua=T0,q0=gn;function x0(){this.__data__=new q0,this.size=0;}var k0=x0;function D0(e){var t=this.__data__,r=t.delete(e);return this.size=t.size,r}var M0=D0;function N0(e){return this.__data__.get(e)}var L0=N0;function R0(e){return this.__data__.has(e)}var F0=R0,Q0=gn,U0=ia,V0=sa,B0=200;function G0(e,t){var r=this.__data__;if(r instanceof Q0){var n=r.__data__;if(!U0||n.length<B0-1)return n.push([e,t]),this.size=++r.size,this;r=this.__data__=new V0(n);}return r.set(e,t),this.size=r.size,this}var z0=G0,W0=gn,H0=k0,Z0=M0,K0=L0,Y0=F0,J0=z0;function It(e){var t=this.__data__=new W0(e);this.size=t.size;}It.prototype.clear=H0,It.prototype.delete=Z0,It.prototype.get=K0,It.prototype.has=Y0,It.prototype.set=J0;var Qc=It,X0="__lodash_hash_undefined__";function e1(e){return this.__data__.set(e,X0),this}var t1=e1;function r1(e){return this.__data__.has(e)}var n1=r1,o1=sa,a1=t1,i1=n1;function yn(e){var t=-1,r=e==null?0:e.length;for(this.__data__=new o1;++t<r;)this.add(e[t]);}yn.prototype.add=yn.prototype.push=a1,yn.prototype.has=i1;var la=yn;function s1(e,t){for(var r=-1,n=e==null?0:e.length;++r<n;)if(t(e[r],r,e))return !0;return !1}var c1=s1;function u1(e,t){return e.has(t)}var da=u1,l1=la,d1=c1,p1=da,f1=1,v1=2;function h1(e,t,r,n,o,a){var i=r&f1,s=e.length,c=t.length;if(s!=c&&!(i&&c>s))return !1;var u=a.get(e),l=a.get(t);if(u&&l)return u==t&&l==e;var d=-1,p=!0,h=r&v1?new l1:void 0;for(a.set(e,t),a.set(t,e);++d<s;){var v=e[d],g=t[d];if(n)var y=i?n(g,v,d,t,e,a):n(v,g,d,e,t,a);if(y!==void 0){if(y)continue;p=!1;break}if(h){if(!d1(t,function(m,$){if(!p1(h,$)&&(v===m||o(v,m,r,n,a)))return h.push($)})){p=!1;break}}else if(!(v===g||o(v,g,r,n,a))){p=!1;break}}return a.delete(e),a.delete(t),p}var Uc=h1,g1=Oe,m1=g1.Uint8Array,b1=m1;function y1(e){var t=-1,r=Array(e.size);return e.forEach(function(n,o){r[++t]=[o,n];}),r}var _1=y1;function $1(e){var t=-1,r=Array(e.size);return e.forEach(function(n){r[++t]=n;}),r}var pa=$1,Vc=dr,Bc=b1,w1=xc,A1=Uc,O1=_1,S1=pa,j1=1,C1=2,P1="[object Boolean]",I1="[object Date]",E1="[object Error]",T1="[object Map]",q1="[object Number]",x1="[object RegExp]",k1="[object Set]",D1="[object String]",M1="[object Symbol]",N1="[object ArrayBuffer]",L1="[object DataView]",Gc=Vc?Vc.prototype:void 0,fa=Gc?Gc.valueOf:void 0;function R1(e,t,r,n,o,a,i){switch(r){case L1:if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return !1;e=e.buffer,t=t.buffer;case N1:return !(e.byteLength!=t.byteLength||!a(new Bc(e),new Bc(t)));case P1:case I1:case q1:return w1(+e,+t);case E1:return e.name==t.name&&e.message==t.message;case x1:case D1:return e==t+"";case T1:var s=O1;case k1:var c=n&j1;if(s||(s=S1),e.size!=t.size&&!c)return !1;var u=i.get(e);if(u)return u==t;n|=C1,i.set(e,t);var l=A1(s(e),s(t),n,o,a,i);return i.delete(e),l;case M1:if(fa)return fa.call(e)==fa.call(t)}return !1}var F1=R1;function Q1(e,t){for(var r=-1,n=t.length,o=e.length;++r<n;)e[o+r]=t[r];return e}var zc=Q1,U1=zc,V1=le;function B1(e,t,r){var n=t(e);return V1(e)?n:U1(n,r(e))}var G1=B1;function z1(e,t){for(var r=-1,n=e==null?0:e.length,o=0,a=[];++r<n;){var i=e[r];t(i,r,e)&&(a[o++]=i);}return a}var Wc=z1;function W1(){return []}var H1=W1,Z1=Wc,K1=H1,Y1=Object.prototype,J1=Y1.propertyIsEnumerable,Hc=Object.getOwnPropertySymbols,X1=Hc?function(e){return e==null?[]:(e=Object(e),Z1(Hc(e),function(t){return J1.call(e,t)}))}:K1,eA=X1;function tA(e,t){for(var r=-1,n=Array(e);++r<e;)n[r]=t(r);return n}var rA=tA,nA=fr,oA=St,aA="[object Arguments]";function iA(e){return oA(e)&&nA(e)==aA}var sA=iA,Zc=sA,cA=St,Kc=Object.prototype,uA=Kc.hasOwnProperty,lA=Kc.propertyIsEnumerable,dA=Zc(function(){return arguments}())?Zc:function(e){return cA(e)&&uA.call(e,"callee")&&!lA.call(e,"callee")},va=dA,_n={exports:{}};function pA(){return !1}var fA=pA;(function(e,t){var r=Oe,n=fA,o=t&&!t.nodeType&&t,a=o&&!0&&e&&!e.nodeType&&e,i=a&&a.exports===o,s=i?r.Buffer:void 0,c=s?s.isBuffer:void 0,u=c||n;e.exports=u;})(_n,_n.exports);var Yc=_n.exports,vA=9007199254740991,hA=/^(?:0|[1-9]\d*)$/;function gA(e,t){var r=typeof e;return t=t==null?vA:t,!!t&&(r=="number"||r!="symbol"&&hA.test(e))&&e>-1&&e%1==0&&e<t}var Jc=gA,mA=9007199254740991;function bA(e){return typeof e=="number"&&e>-1&&e%1==0&&e<=mA}var ha=bA,yA=fr,_A=ha,$A=St,wA="[object Arguments]",AA="[object Array]",OA="[object Boolean]",SA="[object Date]",jA="[object Error]",CA="[object Function]",PA="[object Map]",IA="[object Number]",EA="[object Object]",TA="[object RegExp]",qA="[object Set]",xA="[object String]",kA="[object WeakMap]",DA="[object ArrayBuffer]",MA="[object DataView]",NA="[object Float32Array]",LA="[object Float64Array]",RA="[object Int8Array]",FA="[object Int16Array]",QA="[object Int32Array]",UA="[object Uint8Array]",VA="[object Uint8ClampedArray]",BA="[object Uint16Array]",GA="[object Uint32Array]",Q={};Q[NA]=Q[LA]=Q[RA]=Q[FA]=Q[QA]=Q[UA]=Q[VA]=Q[BA]=Q[GA]=!0,Q[wA]=Q[AA]=Q[DA]=Q[OA]=Q[MA]=Q[SA]=Q[jA]=Q[CA]=Q[PA]=Q[IA]=Q[EA]=Q[TA]=Q[qA]=Q[xA]=Q[kA]=!1;function zA(e){return $A(e)&&_A(e.length)&&!!Q[yA(e)]}var WA=zA;function HA(e){return function(t){return e(t)}}var ga=HA,$n={exports:{}};(function(e,t){var r=Oc,n=t&&!t.nodeType&&t,o=n&&!0&&e&&!e.nodeType&&e,a=o&&o.exports===n,i=a&&r.process,s=function(){try{var c=o&&o.require&&o.require("util").types;return c||i&&i.binding&&i.binding("util")}catch(u){}}();e.exports=s;})($n,$n.exports);var ZA=$n.exports,KA=WA,YA=ga,Xc=ZA,eu=Xc&&Xc.isTypedArray,JA=eu?YA(eu):KA,tu=JA,XA=rA,eO=va,tO=le,rO=Yc,nO=Jc,oO=tu,aO=Object.prototype,iO=aO.hasOwnProperty;function sO(e,t){var r=tO(e),n=!r&&eO(e),o=!r&&!n&&rO(e),a=!r&&!n&&!o&&oO(e),i=r||n||o||a,s=i?XA(e.length,String):[],c=s.length;for(var u in e)(t||iO.call(e,u))&&!(i&&(u=="length"||o&&(u=="offset"||u=="parent")||a&&(u=="buffer"||u=="byteLength"||u=="byteOffset")||nO(u,c)))&&s.push(u);return s}var cO=sO,uO=Object.prototype;function lO(e){var t=e&&e.constructor,r=typeof t=="function"&&t.prototype||uO;return e===r}var dO=lO;function pO(e,t){return function(r){return e(t(r))}}var fO=pO,vO=fO,hO=vO(Object.keys,Object),gO=hO,mO=dO,bO=gO,yO=Object.prototype,_O=yO.hasOwnProperty;function $O(e){if(!mO(e))return bO(e);var t=[];for(var r in Object(e))_O.call(e,r)&&r!="constructor"&&t.push(r);return t}var wO=$O,AO=Ic,OO=ha;function SO(e){return e!=null&&OO(e.length)&&!AO(e)}var wn=SO,jO=cO,CO=wO,PO=wn;function IO(e){return PO(e)?jO(e):CO(e)}var ma=IO,EO=G1,TO=eA,qO=ma;function xO(e){return EO(e,qO,TO)}var kO=xO,ru=kO,DO=1,MO=Object.prototype,NO=MO.hasOwnProperty;function LO(e,t,r,n,o,a){var i=r&DO,s=ru(e),c=s.length,u=ru(t),l=u.length;if(c!=l&&!i)return !1;for(var d=c;d--;){var p=s[d];if(!(i?p in t:NO.call(t,p)))return !1}var h=a.get(e),v=a.get(t);if(h&&v)return h==t&&v==e;var g=!0;a.set(e,t),a.set(t,e);for(var y=i;++d<c;){p=s[d];var m=e[p],$=t[p];if(n)var S=i?n($,m,p,t,e,a):n(m,$,p,e,t,a);if(!(S===void 0?m===$||o(m,$,r,n,a):S)){g=!1;break}y||(y=p=="constructor");}if(g&&!y){var A=e.constructor,q=t.constructor;A!=q&&"constructor"in e&&"constructor"in t&&!(typeof A=="function"&&A instanceof A&&typeof q=="function"&&q instanceof q)&&(g=!1);}return a.delete(e),a.delete(t),g}var RO=LO,FO=He,QO=Oe,UO=FO(QO,"DataView"),VO=UO,BO=He,GO=Oe,zO=BO(GO,"Promise"),WO=zO,HO=He,ZO=Oe,KO=HO(ZO,"Set"),nu=KO,YO=He,JO=Oe,XO=YO(JO,"WeakMap"),eS=XO,ba=VO,ya=ia,_a=WO,$a=nu,wa=eS,ou=fr,Et=Tc,au="[object Map]",tS="[object Object]",iu="[object Promise]",su="[object Set]",cu="[object WeakMap]",uu="[object DataView]",rS=Et(ba),nS=Et(ya),oS=Et(_a),aS=Et($a),iS=Et(wa),Ze=ou;(ba&&Ze(new ba(new ArrayBuffer(1)))!=uu||ya&&Ze(new ya)!=au||_a&&Ze(_a.resolve())!=iu||$a&&Ze(new $a)!=su||wa&&Ze(new wa)!=cu)&&(Ze=function(e){var t=ou(e),r=t==tS?e.constructor:void 0,n=r?Et(r):"";if(n)switch(n){case rS:return uu;case nS:return au;case oS:return iu;case aS:return su;case iS:return cu}return t});var sS=Ze,Aa=Qc,cS=Uc,uS=F1,lS=RO,lu=sS,du=le,pu=Yc,dS=tu,pS=1,fu="[object Arguments]",vu="[object Array]",An="[object Object]",fS=Object.prototype,hu=fS.hasOwnProperty;function vS(e,t,r,n,o,a){var i=du(e),s=du(t),c=i?vu:lu(e),u=s?vu:lu(t);c=c==fu?An:c,u=u==fu?An:u;var l=c==An,d=u==An,p=c==u;if(p&&pu(e)){if(!pu(t))return !1;i=!0,l=!1;}if(p&&!l)return a||(a=new Aa),i||dS(e)?cS(e,t,r,n,o,a):uS(e,t,c,r,n,o,a);if(!(r&pS)){var h=l&&hu.call(e,"__wrapped__"),v=d&&hu.call(t,"__wrapped__");if(h||v){var g=h?e.value():e,y=v?t.value():t;return a||(a=new Aa),o(g,y,r,n,a)}}return p?(a||(a=new Aa),lS(e,t,r,n,o,a)):!1}var hS=vS,gS=hS,gu=St;function mu(e,t,r,n,o){return e===t?!0:e==null||t==null||!gu(e)&&!gu(t)?e!==e&&t!==t:gS(e,t,r,n,mu,o)}var Oa=mu,mS=Qc,bS=Oa,yS=1,_S=2;function $S(e,t,r,n){var o=r.length,a=o,i=!n;if(e==null)return !a;for(e=Object(e);o--;){var s=r[o];if(i&&s[2]?s[1]!==e[s[0]]:!(s[0]in e))return !1}for(;++o<a;){s=r[o];var c=s[0],u=e[c],l=s[1];if(i&&s[2]){if(u===void 0&&!(c in e))return !1}else {var d=new mS;if(n)var p=n(u,l,c,e,t,d);if(!(p===void 0?bS(l,u,yS|_S,n,d):p))return !1}}return !0}var wS=$S,AS=oa;function OS(e){return e===e&&!AS(e)}var bu=OS,SS=bu,jS=ma;function CS(e){for(var t=jS(e),r=t.length;r--;){var n=t[r],o=e[n];t[r]=[n,o,SS(o)];}return t}var PS=CS;function IS(e,t){return function(r){return r==null?!1:r[e]===t&&(t!==void 0||e in Object(r))}}var yu=IS,ES=wS,TS=PS,qS=yu;function xS(e){var t=TS(e);return t.length==1&&t[0][2]?qS(t[0][0],t[0][1]):function(r){return r===e||ES(r,e,t)}}var kS=xS,DS=ua;function MS(e,t,r){var n=e==null?void 0:DS(e,t);return n===void 0?r:n}var NS=MS;function LS(e,t){return e!=null&&t in Object(e)}var RS=LS,FS=Fc,QS=va,US=le,VS=Jc,BS=ha,GS=bn;function zS(e,t,r){t=FS(t,e);for(var n=-1,o=t.length,a=!1;++n<o;){var i=GS(t[n]);if(!(a=e!=null&&r(e,i)))break;e=e[i];}return a||++n!=o?a:(o=e==null?0:e.length,!!o&&BS(o)&&VS(i,o)&&(US(e)||QS(e)))}var WS=zS,HS=RS,ZS=WS;function KS(e,t){return e!=null&&ZS(e,t,HS)}var YS=KS,JS=Oa,XS=NS,ej=YS,tj=na,rj=bu,nj=yu,oj=bn,aj=1,ij=2;function sj(e,t){return tj(e)&&rj(t)?nj(oj(e),t):function(r){var n=XS(r,e);return n===void 0&&n===t?ej(r,e):JS(t,n,aj|ij)}}var cj=sj;function uj(e){return e}var On=uj;function lj(e){return function(t){return t==null?void 0:t[e]}}var dj=lj,pj=ua;function fj(e){return function(t){return pj(t,e)}}var vj=fj,hj=dj,gj=vj,mj=na,bj=bn;function yj(e){return mj(e)?hj(bj(e)):gj(e)}var _j=yj,$j=kS,wj=cj,Aj=On,Oj=le,Sj=_j;function jj(e){return typeof e=="function"?e:e==null?Aj:typeof e=="object"?Oj(e)?wj(e[0],e[1]):$j(e):Sj(e)}var _u=jj;function Cj(e){return function(t,r,n){for(var o=-1,a=Object(t),i=n(t),s=i.length;s--;){var c=i[e?s:++o];if(r(a[c],c,a)===!1)break}return t}}var Pj=Cj,Ij=Pj,Ej=Ij(),Tj=Ej,qj=Tj,xj=ma;function kj(e,t){return e&&qj(e,t,xj)}var Dj=kj,Mj=wn;function Nj(e,t){return function(r,n){if(r==null)return r;if(!Mj(r))return e(r,n);for(var o=r.length,a=t?o:-1,i=Object(r);(t?a--:++a<o)&&n(i[a],a,i)!==!1;);return r}}var Lj=Nj,Rj=Dj,Fj=Lj,Qj=Fj(Rj),$u=Qj,Uj=$u,Vj=wn;function Bj(e,t){var r=-1,n=Vj(e)?Array(e.length):[];return Uj(e,function(o,a,i){n[++r]=t(o,a,i);}),n}var Gj=Bj;function zj(e,t){var r=e.length;for(e.sort(t);r--;)e[r]=e[r].value;return e}var Wj=zj,wu=fn;function Hj(e,t){if(e!==t){var r=e!==void 0,n=e===null,o=e===e,a=wu(e),i=t!==void 0,s=t===null,c=t===t,u=wu(t);if(!s&&!u&&!a&&e>t||a&&i&&c&&!s&&!u||n&&i&&c||!r&&c||!o)return 1;if(!n&&!a&&!u&&e<t||u&&r&&o&&!n&&!a||s&&r&&o||!i&&o||!c)return -1}return 0}var Zj=Hj,Kj=Zj;function Yj(e,t,r){for(var n=-1,o=e.criteria,a=t.criteria,i=o.length,s=r.length;++n<i;){var c=Kj(o[n],a[n]);if(c){if(n>=s)return c;var u=r[n];return c*(u=="desc"?-1:1)}}return e.index-t.index}var Jj=Yj,Sa=ra,Xj=ua,eC=_u,tC=Gj,rC=Wj,nC=ga,oC=Jj,aC=On,iC=le;function sC(e,t,r){t.length?t=Sa(t,function(a){return iC(a)?function(i){return Xj(i,a.length===1?a[0]:a)}:a}):t=[aC];var n=-1;t=Sa(t,nC(eC));var o=tC(e,function(a,i,s){var c=Sa(t,function(u){return u(a)});return {criteria:c,index:++n,value:a}});return rC(o,function(a,i){return oC(a,i,r)})}var cC=sC,uC=cC,Au=le;function lC(e,t,r,n){return e==null?[]:(Au(t)||(t=t==null?[]:[t]),r=n?void 0:r,Au(r)||(r=r==null?[]:[r]),uC(e,t,r))}var dC=lC,pC=lr(dC),Ou=typeof global=="object"&&global&&global.Object===Object&&global,fC=typeof self=="object"&&self&&self.Object===Object&&self,Se=Ou||fC||Function("return this")(),Ne=Se.Symbol,Su=Object.prototype,vC=Su.hasOwnProperty,hC=Su.toString,vr=Ne?Ne.toStringTag:void 0;function gC(e){var t=vC.call(e,vr),r=e[vr];try{e[vr]=void 0;var n=!0;}catch(a){}var o=hC.call(e);return n&&(t?e[vr]=r:delete e[vr]),o}var mC=Object.prototype,bC=mC.toString;function yC(e){return bC.call(e)}var _C="[object Null]",$C="[object Undefined]",ju=Ne?Ne.toStringTag:void 0;function hr(e){return e==null?e===void 0?$C:_C:ju&&ju in Object(e)?gC(e):yC(e)}function Tt(e){return e!=null&&typeof e=="object"}function wC(e,t){for(var r=-1,n=e==null?0:e.length,o=Array(n);++r<n;)o[r]=t(e[r],r,e);return o}var gr=Array.isArray;function Cu(e){var t=typeof e;return e!=null&&(t=="object"||t=="function")}function Pu(e){return e}var AC="[object AsyncFunction]",OC="[object Function]",SC="[object GeneratorFunction]",jC="[object Proxy]";function Iu(e){if(!Cu(e))return !1;var t=hr(e);return t==OC||t==SC||t==AC||t==jC}var ja=Se["__core-js_shared__"],Eu=function(){var e=/[^.]+$/.exec(ja&&ja.keys&&ja.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}();function CC(e){return !!Eu&&Eu in e}var PC=Function.prototype,IC=PC.toString;function Ke(e){if(e!=null){try{return IC.call(e)}catch(t){}try{return e+""}catch(t){}}return ""}var EC=/[\\^$.*+?()[\]{}|]/g,TC=/^\[object .+?Constructor\]$/,qC=Function.prototype,xC=Object.prototype,kC=qC.toString,DC=xC.hasOwnProperty,MC=RegExp("^"+kC.call(DC).replace(EC,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function NC(e){if(!Cu(e)||CC(e))return !1;var t=Iu(e)?MC:TC;return t.test(Ke(e))}function LC(e,t){return e==null?void 0:e[t]}function Ye(e,t){var r=LC(e,t);return NC(r)?r:void 0}var Ca=Ye(Se,"WeakMap");function RC(e,t,r){switch(r.length){case 0:return e.call(t);case 1:return e.call(t,r[0]);case 2:return e.call(t,r[0],r[1]);case 3:return e.call(t,r[0],r[1],r[2])}return e.apply(t,r)}function FC(){}var QC=800,UC=16,VC=Date.now;function BC(e){var t=0,r=0;return function(){var n=VC(),o=UC-(n-r);if(r=n,o>0){if(++t>=QC)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}function GC(e){return function(){return e}}var Tu=function(){try{var e=Ye(Object,"defineProperty");return e({},"",{}),e}catch(t){}}(),zC=Tu?function(e,t){return Tu(e,"toString",{configurable:!0,enumerable:!1,value:GC(t),writable:!0})}:Pu,WC=BC(zC),HC=WC;function ZC(e,t,r,n){for(var o=e.length,a=r+(n?1:-1);n?a--:++a<o;)if(t(e[a],a,e))return a;return -1}function KC(e){return e!==e}function YC(e,t,r){for(var n=r-1,o=e.length;++n<o;)if(e[n]===t)return n;return -1}function JC(e,t,r){return t===t?YC(e,t,r):ZC(e,KC,r)}function qu(e,t){var r=e==null?0:e.length;return !!r&&JC(e,t,0)>-1}var XC=9007199254740991,eP=/^(?:0|[1-9]\d*)$/;function tP(e,t){var r=typeof e;return t=t==null?XC:t,!!t&&(r=="number"||r!="symbol"&&eP.test(e))&&e>-1&&e%1==0&&e<t}function xu(e,t){return e===t||e!==e&&t!==t}var ku=Math.max;function rP(e,t,r){return t=ku(t===void 0?e.length-1:t,0),function(){for(var n=arguments,o=-1,a=ku(n.length-t,0),i=Array(a);++o<a;)i[o]=n[t+o];o=-1;for(var s=Array(t+1);++o<t;)s[o]=n[o];return s[t]=r(i),RC(e,this,s)}}function nP(e,t){return HC(rP(e,t,Pu),e+"")}var oP=9007199254740991;function Du(e){return typeof e=="number"&&e>-1&&e%1==0&&e<=oP}function Mu(e){return e!=null&&Du(e.length)&&!Iu(e)}var aP=Object.prototype;function iP(e){var t=e&&e.constructor,r=typeof t=="function"&&t.prototype||aP;return e===r}function sP(e,t){for(var r=-1,n=Array(e);++r<e;)n[r]=t(r);return n}var cP="[object Arguments]";function Nu(e){return Tt(e)&&hr(e)==cP}var Lu=Object.prototype,uP=Lu.hasOwnProperty,lP=Lu.propertyIsEnumerable,Ru=Nu(function(){return arguments}())?Nu:function(e){return Tt(e)&&uP.call(e,"callee")&&!lP.call(e,"callee")};function dP(){return !1}var Fu=typeof exports=="object"&&exports&&!exports.nodeType&&exports,Qu=Fu&&typeof module=="object"&&module&&!module.nodeType&&module,pP=Qu&&Qu.exports===Fu,Uu=pP?Se.Buffer:void 0,fP=Uu?Uu.isBuffer:void 0,Pa=fP||dP,vP="[object Arguments]",hP="[object Array]",gP="[object Boolean]",mP="[object Date]",bP="[object Error]",yP="[object Function]",_P="[object Map]",$P="[object Number]",wP="[object Object]",AP="[object RegExp]",OP="[object Set]",SP="[object String]",jP="[object WeakMap]",CP="[object ArrayBuffer]",PP="[object DataView]",IP="[object Float32Array]",EP="[object Float64Array]",TP="[object Int8Array]",qP="[object Int16Array]",xP="[object Int32Array]",kP="[object Uint8Array]",DP="[object Uint8ClampedArray]",MP="[object Uint16Array]",NP="[object Uint32Array]",U={};U[IP]=U[EP]=U[TP]=U[qP]=U[xP]=U[kP]=U[DP]=U[MP]=U[NP]=!0,U[vP]=U[hP]=U[CP]=U[gP]=U[PP]=U[mP]=U[bP]=U[yP]=U[_P]=U[$P]=U[wP]=U[AP]=U[OP]=U[SP]=U[jP]=!1;function LP(e){return Tt(e)&&Du(e.length)&&!!U[hr(e)]}function Vu(e){return function(t){return e(t)}}var Bu=typeof exports=="object"&&exports&&!exports.nodeType&&exports,mr=Bu&&typeof module=="object"&&module&&!module.nodeType&&module,RP=mr&&mr.exports===Bu,Ia=RP&&Ou.process,Gu=function(){try{var e=mr&&mr.require&&mr.require("util").types;return e||Ia&&Ia.binding&&Ia.binding("util")}catch(t){}}(),zu=Gu&&Gu.isTypedArray,Wu=zu?Vu(zu):LP,FP=Object.prototype,QP=FP.hasOwnProperty;function UP(e,t){var r=gr(e),n=!r&&Ru(e),o=!r&&!n&&Pa(e),a=!r&&!n&&!o&&Wu(e),i=r||n||o||a,s=i?sP(e.length,String):[],c=s.length;for(var u in e)(t||QP.call(e,u))&&!(i&&(u=="length"||o&&(u=="offset"||u=="parent")||a&&(u=="buffer"||u=="byteLength"||u=="byteOffset")||tP(u,c)))&&s.push(u);return s}function VP(e,t){return function(r){return e(t(r))}}var BP=VP(Object.keys,Object),GP=Object.prototype,zP=GP.hasOwnProperty;function WP(e){if(!iP(e))return BP(e);var t=[];for(var r in Object(e))zP.call(e,r)&&r!="constructor"&&t.push(r);return t}function HP(e){return Mu(e)?UP(e):WP(e)}var br=Ye(Object,"create");function ZP(){this.__data__=br?br(null):{},this.size=0;}function KP(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var YP="__lodash_hash_undefined__",JP=Object.prototype,XP=JP.hasOwnProperty;function eI(e){var t=this.__data__;if(br){var r=t[e];return r===YP?void 0:r}return XP.call(t,e)?t[e]:void 0}var tI=Object.prototype,rI=tI.hasOwnProperty;function nI(e){var t=this.__data__;return br?t[e]!==void 0:rI.call(t,e)}var oI="__lodash_hash_undefined__";function aI(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=br&&t===void 0?oI:t,this}function Je(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Je.prototype.clear=ZP,Je.prototype.delete=KP,Je.prototype.get=eI,Je.prototype.has=nI,Je.prototype.set=aI;function iI(){this.__data__=[],this.size=0;}function Sn(e,t){for(var r=e.length;r--;)if(xu(e[r][0],t))return r;return -1}var sI=Array.prototype,cI=sI.splice;function uI(e){var t=this.__data__,r=Sn(t,e);if(r<0)return !1;var n=t.length-1;return r==n?t.pop():cI.call(t,r,1),--this.size,!0}function lI(e){var t=this.__data__,r=Sn(t,e);return r<0?void 0:t[r][1]}function dI(e){return Sn(this.__data__,e)>-1}function pI(e,t){var r=this.__data__,n=Sn(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this}function je(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}je.prototype.clear=iI,je.prototype.delete=uI,je.prototype.get=lI,je.prototype.has=dI,je.prototype.set=pI;var yr=Ye(Se,"Map");function fI(){this.size=0,this.__data__={hash:new Je,map:new(yr||je),string:new Je};}function vI(e){var t=typeof e;return t=="string"||t=="number"||t=="symbol"||t=="boolean"?e!=="__proto__":e===null}function jn(e,t){var r=e.__data__;return vI(t)?r[typeof t=="string"?"string":"hash"]:r.map}function hI(e){var t=jn(this,e).delete(e);return this.size-=t?1:0,t}function gI(e){return jn(this,e).get(e)}function mI(e){return jn(this,e).has(e)}function bI(e,t){var r=jn(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this}function Xe(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Xe.prototype.clear=fI,Xe.prototype.delete=hI,Xe.prototype.get=gI,Xe.prototype.has=mI,Xe.prototype.set=bI;function Hu(e,t){for(var r=-1,n=t.length,o=e.length;++r<n;)e[o+r]=t[r];return e}var Zu=Ne?Ne.isConcatSpreadable:void 0;function yI(e){return gr(e)||Ru(e)||!!(Zu&&e&&e[Zu])}function Ku(e,t,r,n,o){var a=-1,i=e.length;for(r||(r=yI),o||(o=[]);++a<i;){var s=e[a];t>0&&r(s)?t>1?Ku(s,t-1,r,n,o):Hu(o,s):n||(o[o.length]=s);}return o}function _I(){this.__data__=new je,this.size=0;}function $I(e){var t=this.__data__,r=t.delete(e);return this.size=t.size,r}function wI(e){return this.__data__.get(e)}function AI(e){return this.__data__.has(e)}var OI=200;function SI(e,t){var r=this.__data__;if(r instanceof je){var n=r.__data__;if(!yr||n.length<OI-1)return n.push([e,t]),this.size=++r.size,this;r=this.__data__=new Xe(n);}return r.set(e,t),this.size=r.size,this}function Le(e){var t=this.__data__=new je(e);this.size=t.size;}Le.prototype.clear=_I,Le.prototype.delete=$I,Le.prototype.get=wI,Le.prototype.has=AI,Le.prototype.set=SI;function Yu(e,t){for(var r=-1,n=e==null?0:e.length,o=0,a=[];++r<n;){var i=e[r];t(i,r,e)&&(a[o++]=i);}return a}function jI(){return []}var CI=Object.prototype,PI=CI.propertyIsEnumerable,Ju=Object.getOwnPropertySymbols,II=Ju?function(e){return e==null?[]:(e=Object(e),Yu(Ju(e),function(t){return PI.call(e,t)}))}:jI;function EI(e,t,r){var n=t(e);return gr(e)?n:Hu(n,r(e))}function Xu(e){return EI(e,HP,II)}var Ea=Ye(Se,"DataView"),Ta=Ye(Se,"Promise"),qt=Ye(Se,"Set"),el="[object Map]",TI="[object Object]",tl="[object Promise]",rl="[object Set]",nl="[object WeakMap]",ol="[object DataView]",qI=Ke(Ea),xI=Ke(yr),kI=Ke(Ta),DI=Ke(qt),MI=Ke(Ca),et=hr;(Ea&&et(new Ea(new ArrayBuffer(1)))!=ol||yr&&et(new yr)!=el||Ta&&et(Ta.resolve())!=tl||qt&&et(new qt)!=rl||Ca&&et(new Ca)!=nl)&&(et=function(e){var t=hr(e),r=t==TI?e.constructor:void 0,n=r?Ke(r):"";if(n)switch(n){case qI:return ol;case xI:return el;case kI:return tl;case DI:return rl;case MI:return nl}return t});var al=et,il=Se.Uint8Array,NI="__lodash_hash_undefined__";function LI(e){return this.__data__.set(e,NI),this}function RI(e){return this.__data__.has(e)}function xt(e){var t=-1,r=e==null?0:e.length;for(this.__data__=new Xe;++t<r;)this.add(e[t]);}xt.prototype.add=xt.prototype.push=LI,xt.prototype.has=RI;function FI(e,t){for(var r=-1,n=e==null?0:e.length;++r<n;)if(t(e[r],r,e))return !0;return !1}function qa(e,t){return e.has(t)}var QI=1,UI=2;function sl(e,t,r,n,o,a){var i=r&QI,s=e.length,c=t.length;if(s!=c&&!(i&&c>s))return !1;var u=a.get(e),l=a.get(t);if(u&&l)return u==t&&l==e;var d=-1,p=!0,h=r&UI?new xt:void 0;for(a.set(e,t),a.set(t,e);++d<s;){var v=e[d],g=t[d];if(n)var y=i?n(g,v,d,t,e,a):n(v,g,d,e,t,a);if(y!==void 0){if(y)continue;p=!1;break}if(h){if(!FI(t,function(m,$){if(!qa(h,$)&&(v===m||o(v,m,r,n,a)))return h.push($)})){p=!1;break}}else if(!(v===g||o(v,g,r,n,a))){p=!1;break}}return a.delete(e),a.delete(t),p}function VI(e){var t=-1,r=Array(e.size);return e.forEach(function(n,o){r[++t]=[o,n];}),r}function xa(e){var t=-1,r=Array(e.size);return e.forEach(function(n){r[++t]=n;}),r}var BI=1,GI=2,zI="[object Boolean]",WI="[object Date]",HI="[object Error]",ZI="[object Map]",KI="[object Number]",YI="[object RegExp]",JI="[object Set]",XI="[object String]",eE="[object Symbol]",tE="[object ArrayBuffer]",rE="[object DataView]",cl=Ne?Ne.prototype:void 0,ka=cl?cl.valueOf:void 0;function nE(e,t,r,n,o,a,i){switch(r){case rE:if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return !1;e=e.buffer,t=t.buffer;case tE:return !(e.byteLength!=t.byteLength||!a(new il(e),new il(t)));case zI:case WI:case KI:return xu(+e,+t);case HI:return e.name==t.name&&e.message==t.message;case YI:case XI:return e==t+"";case ZI:var s=VI;case JI:var c=n&BI;if(s||(s=xa),e.size!=t.size&&!c)return !1;var u=i.get(e);if(u)return u==t;n|=GI,i.set(e,t);var l=sl(s(e),s(t),n,o,a,i);return i.delete(e),l;case eE:if(ka)return ka.call(e)==ka.call(t)}return !1}var oE=1,aE=Object.prototype,iE=aE.hasOwnProperty;function sE(e,t,r,n,o,a){var i=r&oE,s=Xu(e),c=s.length,u=Xu(t),l=u.length;if(c!=l&&!i)return !1;for(var d=c;d--;){var p=s[d];if(!(i?p in t:iE.call(t,p)))return !1}var h=a.get(e),v=a.get(t);if(h&&v)return h==t&&v==e;var g=!0;a.set(e,t),a.set(t,e);for(var y=i;++d<c;){p=s[d];var m=e[p],$=t[p];if(n)var S=i?n($,m,p,t,e,a):n(m,$,p,e,t,a);if(!(S===void 0?m===$||o(m,$,r,n,a):S)){g=!1;break}y||(y=p=="constructor");}if(g&&!y){var A=e.constructor,q=t.constructor;A!=q&&"constructor"in e&&"constructor"in t&&!(typeof A=="function"&&A instanceof A&&typeof q=="function"&&q instanceof q)&&(g=!1);}return a.delete(e),a.delete(t),g}var cE=1,ul="[object Arguments]",ll="[object Array]",Cn="[object Object]",uE=Object.prototype,dl=uE.hasOwnProperty;function lE(e,t,r,n,o,a){var i=gr(e),s=gr(t),c=i?ll:al(e),u=s?ll:al(t);c=c==ul?Cn:c,u=u==ul?Cn:u;var l=c==Cn,d=u==Cn,p=c==u;if(p&&Pa(e)){if(!Pa(t))return !1;i=!0,l=!1;}if(p&&!l)return a||(a=new Le),i||Wu(e)?sl(e,t,r,n,o,a):nE(e,t,c,r,n,o,a);if(!(r&cE)){var h=l&&dl.call(e,"__wrapped__"),v=d&&dl.call(t,"__wrapped__");if(h||v){var g=h?e.value():e,y=v?t.value():t;return a||(a=new Le),o(g,y,r,n,a)}}return p?(a||(a=new Le),sE(e,t,r,n,o,a)):!1}function pl(e,t,r,n,o){return e===t?!0:e==null||t==null||!Tt(e)&&!Tt(t)?e!==e&&t!==t:lE(e,t,r,n,pl,o)}function dE(e){return Tt(e)&&Mu(e)}function fl(e,t,r){for(var n=-1,o=e==null?0:e.length;++n<o;)if(r(t,e[n]))return !0;return !1}var pE=200;function fE(e,t,r,n){var o=-1,a=qu,i=!0,s=e.length,c=[],u=t.length;if(!s)return c;r&&(t=wC(t,Vu(r))),n?(a=fl,i=!1):t.length>=pE&&(a=qa,i=!1,t=new xt(t));e:for(;++o<s;){var l=e[o],d=r==null?l:r(l);if(l=n||l!==0?l:0,i&&d===d){for(var p=u;p--;)if(t[p]===d)continue e;c.push(l);}else a(t,d,n)||c.push(l);}return c}function vE(e){var t=e==null?0:e.length;return t?e[t-1]:void 0}function vl(e,t){return pl(e,t)}var hE=1/0,gE=qt&&1/xa(new qt([,-0]))[1]==hE?function(e){return new qt(e)}:FC,mE=200;function hl(e,t,r){var n=-1,o=qu,a=e.length,i=!0,s=[],c=s;if(r)i=!1,o=fl;else if(a>=mE){var u=t?null:gE(e);if(u)return xa(u);i=!1,o=qa,c=new xt;}else c=t?[]:s;e:for(;++n<a;){var l=e[n],d=t?t(l):l;if(l=r||l!==0?l:0,i&&d===d){for(var p=c.length;p--;)if(c[p]===d)continue e;t&&c.push(d),s.push(l);}else o(c,d,r)||(c!==s&&c.push(d),s.push(l));}return s}function bE(e,t,r){var n=e.length;if(n<2)return n?hl(e[0]):[];for(var o=-1,a=Array(n);++o<n;)for(var i=e[o],s=-1;++s<n;)s!=o&&(a[o]=fE(a[o]||i,e[s],t,r));return hl(Ku(a,1),t,r)}var yE=nP(function(e){var t=vE(e);return t=typeof t=="function"?t:void 0,bE(Yu(e,dE),void 0,t)}),_E=yE,_r={},Pn={};Object.defineProperty(Pn,"__esModule",{value:!0}),Pn.CA_DL=void 0,Pn.CA_DL={AB:[{regex:/^[0-9]{9}$/,description:"9 numbers"}],BC:[{regex:/^[0-9]{7}$/,description:"7 numbers"}],MB:[{regex:/^[A-Z0-9]{12}$/,description:"any combination of letters or numbers for a total of 12 characters"}],NB:[{regex:/^[0-9]{5,7}$/,description:"5-7 numbers"}],NL:[{regex:/^[A-Z]{1}[0-9]{9}$/,description:"1 letter followed by 9 numbers"}],NT:[{regex:/^[0-9]{6}$/,description:"6 numbers"}],NS:[{regex:/^[A-Z]{5}[0-9]{9}$/,description:"5 letters followed by 9 numbers"}],NU:[{regex:/^[0-9]{9}$/,description:"9 numbers"}],ON:[{regex:/^[A-Z]{1}[0-9]{14}$/,description:"1 letter followed by 14 numbers"}],PE:[{regex:/^[0-9]{5,6}$/,description:"5-6 numbers"}],QC:[{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],SK:[{regex:/^[0-9]{8}$/,description:"8 numbers"}],YT:[{regex:/^[0-9]{10}$/,description:"10 numbers"}]};var In={};Object.defineProperty(In,"__esModule",{value:!0}),In.US_DL=void 0,In.US_DL={AL:[{regex:/^[0-9]{1,8}$/,description:"1-8 numbers"}],AK:[{regex:/^[0-9]{1,7}$/,description:"1-7 numbers"}],AZ:[{regex:/^[A-Z]{1}[0-9]{8,9}$/,description:"1 letter followed by 8-9 numbers"},{regex:/^[A-Z]{2}[0-9]{2,5}$/,description:"2 letters followed by 2-5 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],AR:[{regex:/^[0-9]{4,9}$/,description:"4-9 numbers"}],CA:[{regex:/^[A-Z]{1}[0-9]{7}$/,description:"1 letter followed by 7 numbers"}],CO:[{regex:/^[0-9]{9}$/,description:"9 numbers"},{regex:/^[A-Z]{1}[0-9]{3,6}$/,description:"1 letter followed by 3-6 numbers"},{regex:/^[A-Z]{2}[0-9]{2,5}$/,description:"2 letters followed by 2-5 numbers"}],CT:[{regex:/^[0-9]{9}$/,description:"9 numbers"}],DE:[{regex:/^[0-9]{1,7}$/,description:"1-7 numbers"}],DC:[{regex:/^[0-9]{7}$/,description:"7 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],FL:[{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],GA:[{regex:/^[0-9]{7,9}$/,description:"7-9 numbers"}],HI:[{regex:/^[A-Z]{1}[0-9]{8}$/,description:"1 letter followed by 8 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],ID:[{regex:/^[A-Z]{2}[0-9]{6}[A-Z]{1}$/,description:"2 letters followed by 6 numbers followed by 1 letter"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],IL:[{regex:/^[A-Z]{1}[0-9]{11,12}$/,description:"1 letter followed by 11-12 numbers"}],IN:[{regex:/^[A-Z]{1}[0-9]{9}$/,description:"1 letter followed by 9 numbers"},{regex:/^[0-9]{9,10}$/,description:"9-10 numbers"}],IA:[{regex:/^[0-9]{9}$/,description:"9 numbers"},{regex:/^[0-9]{3}[A-Z]{2}[0-9]{4}$/,description:"3 numbers followed by 2 letters followed by 4 numbers"}],KS:[{regex:/^([A-Z]{1}[0-9]{1}){2}[A-Z]{1}$/,description:"1 letter then 1 number then 1 letter then 1 number then 1 letter"},{regex:/^[A-Z]{1}[0-9]{8}$/,description:"1 letter followed by 8 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],KY:[{regex:/^[A-Z]{1}[0-9]{8,9}$/,description:"1 letter followed by 8-9 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],LA:[{regex:/^[0-9]{1,9}$/,description:"1-9 numbers"}],ME:[{regex:/^[0-9]{7}$/,description:"7 numbers"},{regex:/^[0-9]{7}[A-Z]{1}$/,description:"7 numbers followed by 1 letter"},{regex:/^[0-9]{8}$/,description:"8 numbers"}],MD:[{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],MA:[{regex:/^[A-Z]{1}[0-9]{8}$/,description:"1 letter followed by 8 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],MI:[{regex:/^[A-Z]{1}[0-9]{10}$/,description:"1 letter followed by 10 numbers"},{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],MN:[{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],MS:[{regex:/^[0-9]{9}$/,description:"9 numbers"}],MO:[{regex:/^[A-Z]{1}[0-9]{5,9}$/,description:"1 letter followed by 5-9 numbers"},{regex:/^[A-Z]{1}[0-9]{6}[R]{1}$/,description:'1 letter followed by 6 numbers followed by "R"'},{regex:/^[0-9]{8}[A-Z]{2}$/,description:"8 numbers followed by 2 letters"},{regex:/^[0-9]{9}[A-Z]{1}$/,description:"9 numbers followed by 1 letter"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],MT:[{regex:/^[A-Z]{1}[0-9]{8}$/,description:"1 letter followed by 8 numbers"},{regex:/^[0-9]{13}$/,description:"13 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"},{regex:/^[0-9]{14}$/,description:"14 numbers"}],NE:[{regex:/^[A-Z]{1}[0-9]{6,8}$/,description:"1 letter followed by 6-8 numbers"}],NV:[{regex:/^[0-9]{9,10}$/,description:"9-10 numbers"},{regex:/^[0-9]{12}$/,description:"12 numbers"},{regex:/^[X]{1}[0-9]{8}$/,description:'"X" followed by 8 numbers'}],NH:[{regex:/^[0-9]{2}[A-Z]{3}[0-9]{5}$/,description:"2 numbers followed by 3 letters followed by 5 numbers"}],NJ:[{regex:/^[A-Z]{1}[0-9]{14}$/,description:"1 letter followed by 14 numbers"}],NM:[{regex:/^[0-9]{8,9}$/,description:"8-9 numbers"}],NY:[{regex:/^[A-Z]{1}[0-9]{7}$/,description:"1 letter followed by 7 numbers"},{regex:/^[A-Z]{1}[0-9]{18}$/,description:"1 letter followed by 18 numbers"},{regex:/^[0-9]{8,9}$/,description:"8-9 numbers"},{regex:/^[0-9]{16}$/,description:"16 numbers"},{regex:/^[A-Z]{8}$/,description:"8 letters"}],NC:[{regex:/^[0-9]{1,12}$/,description:"1-12 numbers"}],ND:[{regex:/^[A-Z]{3}[0-9]{6}$/,description:"3 letters followed by 6 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],OH:[{regex:/^[A-Z]{1}[0-9]{4,8}$/,description:"1 letter followed by 4-8 numbers"},{regex:/^[A-Z]{2}[0-9]{3,7}$/,description:"2 letters followed by 3-7 numbers"},{regex:/^[0-9]{8}$/,description:"8 numbers"}],OK:[{regex:/^[A-Z]{1}[0-9]{9}$/,description:"1 letter followed by 9 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],OR:[{regex:/^[0-9]{1,9}$/,description:"1-9 numbers"}],PA:[{regex:/^[0-9]{8}$/,description:"8 numbers"}],RI:[{regex:/^[0-9]{7}$/,description:"7 numbers"},{regex:/^[A-Z]{1}[0-9]{6}$/,description:"1 letter followed by 6 numbers"}],SC:[{regex:/^[0-9]{5,11}$/,description:"5-11 numbers"}],SD:[{regex:/^[0-9]{6,10}$/,description:"6-10 numbers"},{regex:/^[0-9]{12}$/,description:"12 numbers"}],TN:[{regex:/^[0-9]{7,9}$/,description:"7-9 numbers"}],TX:[{regex:/^[0-9]{7,8}$/,description:"7-8 numbers"}],UT:[{regex:/^[0-9]{4,10}$/,description:"4-10 numbers"}],VT:[{regex:/^[0-9]{8}$/,description:"8 numbers"},{regex:/^[0-9]{7}[A]$/,description:'7 numbers followed by "A"'}],VA:[{regex:/^[A-Z]{1}[0-9]{8,11}$/,description:"1 letter followed by 8-11 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],WA:[{regex:/^(?=.{12}$)[A-Z]{1,7}[A-Z0-9\\*]{4,11}$/,description:'1-7 letters followed by any combination of letters, numbers, or "*" for a total of 12 characters'}],WV:[{regex:/^[0-9]{7}$/,description:"7 numbers"},{regex:/^[A-Z]{1,2}[0-9]{5,6}$/,description:"1-2 letters followed by 5-6 numbers"}],WI:[{regex:/^[A-Z]{1}[0-9]{13}$/,description:"1 letter followed by 13 numbers"}],WY:[{regex:/^[0-9]{9,10}$/,description:"9-10 numbers"}]},Object.defineProperty(_r,"__esModule",{value:!0}),_r.getMatches=_r.isValid=void 0;var $E=Pn,wE=In;function AE(e,t){t===void 0&&(t={});var r=gl(e,t);return (r==null?void 0:r.length)>0}_r.isValid=AE;function gl(e,t){t===void 0&&(t={});var r=[],n,o;switch(t.country){case"CA":n=$E.CA_DL;break;case"US":default:n=wE.US_DL;break}return t.states?Array.isArray(t.states)||(o=[t.states]):o=Object.keys(n),o.forEach(function(a){var i=n[a];if(!i)throw new Error('Could not find state "'+a+'"!');i.forEach(function(s){var c=s.regex,u=s.description,l=t.ignoreCase?new RegExp(c,"i"):c;l.test(e)&&r.push({description:u,state:a});});}),r.length?r:null}_r.getMatches=gl;const OE=["email","number","url","zipcode","year"],SE=["ssn","fein","phone","currency","percentage"],jE=["radio","dropdown"],Da=e=>{let t=e;if(OE.includes(e))t="text";else if(SE.includes(e))t="masked";else if(jE.includes(e))return "dropdown";return t};function Ma(e){return ("0"+e.toString(16)).substr(-2)}function ml(e){const t=new Uint8Array((e-1)/2),r=window.crypto||window.msCrypto;if(!r)return "";r.getRandomValues(t);const n=Ma(Math.floor(Math.random()*6)+10).substr(-1);let o="";return e%2===0&&(o=Ma(Math.floor(Math.random()*6)+10).substr(-1)),n+Array.from(t,Ma).join("")+o}function CE(e,t){let r=!0;switch(typeof e){case"string":r=!0;break;case"object":Array.isArray(e)&&Array.isArray(t)?r=_E(e,t,vl).length>0:!Array.isArray(e)&&!Array.isArray(t)&&(r=!vl(e,t));break;case"number":r=!0;break}return r}function kt(e,t){let r=!0;return e===t?r=!1:e&&!t||!e&&t?r=!0:typeof e==typeof t&&(r=CE(e,t)),r}function PE(e){const t=e.question,r=e.choice_id||e.choice_ids;if(r.length>0){const n=t.choices.reduce((o,a)=>(o[a.id]=a,o),{});return Array.isArray(r)?r.map(o=>n[o].value):n[r].value}return e.value}function IE(e){var t;const r=e==null?void 0:e.question;if(!(r!=null&&r.field_type))return !!(e!=null&&e.value);switch(Da(r.field_type)){case w.DropDown:return r.multiple?!!((t=e.choice_ids)!=null&&t.length):!!e.choice_id;case w.Masked:case w.Text:default:return e.value!==null}}function EE(e,t){return t instanceof RegExp?e.filter(r=>t.test(r.question.code)):e.filter(r=>r.question.code.includes(t))}var ie={},Na={},$r={},La={};/*!
* cookie
* Copyright(c) 2012-2014 Roman Shtylman
* Copyright(c) 2015 Douglas Christopher Wilson
* MIT Licensed
*/La.parse=kE,La.serialize=DE;var TE=decodeURIComponent,qE=encodeURIComponent,xE=/; */,En=/^[\u0009\u0020-\u007e\u0080-\u00ff]+$/;function kE(e,t){if(typeof e!="string")throw new TypeError("argument str must be a string");for(var r={},n=t||{},o=e.split(xE),a=n.decode||TE,i=0;i<o.length;i++){var s=o[i],c=s.indexOf("=");if(!(c<0)){var u=s.substr(0,c).trim(),l=s.substr(++c,s.length).trim();l[0]=='"'&&(l=l.slice(1,-1)),r[u]==null&&(r[u]=ME(l,a));}}return r}function DE(e,t,r){var n=r||{},o=n.encode||qE;if(typeof o!="function")throw new TypeError("option encode is invalid");if(!En.test(e))throw new TypeError("argument name is invalid");var a=o(t);if(a&&!En.test(a))throw new TypeError("argument val is invalid");var i=e+"="+a;if(n.maxAge!=null){var s=n.maxAge-0;if(isNaN(s))throw new Error("maxAge should be a Number");i+="; Max-Age="+Math.floor(s);}if(n.domain){if(!En.test(n.domain))throw new TypeError("option domain is invalid");i+="; Domain="+n.domain;}if(n.path){if(!En.test(n.path))throw new TypeError("option path is invalid");i+="; Path="+n.path;}if(n.expires){if(typeof n.expires.toUTCString!="function")throw new TypeError("option expires is invalid");i+="; Expires="+n.expires.toUTCString();}if(n.httpOnly&&(i+="; HttpOnly"),n.secure&&(i+="; Secure"),n.sameSite){var c=typeof n.sameSite=="string"?n.sameSite.toLowerCase():n.sameSite;switch(c){case!0:i+="; SameSite=Strict";break;case"lax":i+="; SameSite=Lax";break;case"strict":i+="; SameSite=Strict";break;default:throw new TypeError("option sameSite is invalid")}}return i}function ME(e,t){try{return t(e)}catch(r){return e}}Object.defineProperty($r,"__esModule",{value:!0});var NE=function(){function e(t,r){for(var n=0;n<r.length;n++){var o=r[n];o.enumerable=o.enumerable||!1,o.configurable=!0,"value"in o&&(o.writable=!0),Object.defineProperty(t,o.key,o);}}return function(t,r,n){return r&&e(t.prototype,r),n&&e(t,n),t}}();$r.hasCookies=QE;var LE=La,Tn=RE(LE);function RE(e){return e&&e.__esModule?e:{default:e}}function FE(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}var Re="lS_",bl=function(){function e(){var t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};FE(this,e),this.cookieOptions=Object.assign({path:"/"},t),Re=t.prefix===void 0?Re:t.prefix;}return NE(e,[{key:"getItem",value:function(t){var r=Tn.default.parse(document.cookie);return !r||!r.hasOwnProperty(Re+t)?null:r[Re+t]}},{key:"setItem",value:function(t,r){return document.cookie=Tn.default.serialize(Re+t,r,this.cookieOptions),r}},{key:"removeItem",value:function(t){var r=Object.assign({},this.cookieOptions,{maxAge:-1});return document.cookie=Tn.default.serialize(Re+t,"",r),null}},{key:"clear",value:function(){var t=Tn.default.parse(document.cookie);for(var r in t)r.indexOf(Re)===0&&this.removeItem(r.substr(Re.length));return null}}]),e}();$r.default=bl;function QE(){var e=new bl;try{var t="__test";e.setItem(t,"1");var r=e.getItem(t);return e.removeItem(t),r==="1"}catch(n){return !1}}Object.defineProperty(Na,"__esModule",{value:!0}),Na.default=VE;var UE=$r,yl="__test";function _l(e){try{var t=window[e];return t.setItem(yl,"1"),t.removeItem(yl),!0}catch(r){return !1}}function VE(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"localStorage",t=String(e).replace(/storage$/i,"").toLowerCase();if(t==="local")return _l("localStorage");if(t==="session")return _l("sessionStorage");if(t==="cookie")return (0, UE.hasCookies)();if(t==="memory")return !0;throw new Error("Storage method `"+e+"` is not available.\n    Please use one of the following: localStorage, sessionStorage, cookieStorage, memoryStorage.")}var Ra={};Object.defineProperty(Ra,"__esModule",{value:!0});var BE=function(){function e(t,r){for(var n=0;n<r.length;n++){var o=r[n];o.enumerable=o.enumerable||!1,o.configurable=!0,"value"in o&&(o.writable=!0),Object.defineProperty(t,o.key,o);}}return function(t,r,n){return r&&e(t.prototype,r),n&&e(t,n),t}}();function GE(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}var zE=function(){function e(){GE(this,e),this._data={};}return BE(e,[{key:"getItem",value:function(t){return this._data.hasOwnProperty(t)?this._data[t]:null}},{key:"setItem",value:function(t,r){return this._data[t]=String(r)}},{key:"removeItem",value:function(t){return delete this._data[t]}},{key:"clear",value:function(){return this._data={}}}]),e}();Ra.default=zE,Object.defineProperty(ie,"__esModule",{value:!0}),ie.MemoryStorage=ie.CookieStorage=ie.isSupported=ie.storage=void 0;var WE=Na,qn=Fa(WE),HE=$r,$l=Fa(HE),ZE=Ra,wl=Fa(ZE);function Fa(e){return e&&e.__esModule?e:{default:e}}var Dt=null;(0, qn.default)("localStorage")?ie.storage=Dt=window.localStorage:(0, qn.default)("sessionStorage")?ie.storage=Dt=window.sessionStorage:(0, qn.default)("cookieStorage")?ie.storage=Dt=new $l.default:ie.storage=Dt=new wl.default;var Qa=ie.default=Dt;ie.storage=Dt,ie.isSupported=qn.default,ie.CookieStorage=$l.default,ie.MemoryStorage=wl.default;const xn="bp-sdk-";function wr(e){return Qa.getItem(`${xn}${e}`)}function Ua(e,t){Qa.setItem(`${xn}${e}`,t);}function KE(e){Qa.removeItem(`${xn}${e}`);}function YE(){Object.keys(At).forEach(e=>KE(e));}function JE(){var e;return JSON.parse((e=wr(At.user))!=null?e:"{}")}function XE(e){return (e==null?void 0:e.token_type)==="cookie"&&e!=null&&e.token_expires_at?e.token_expires_at*1e3:e!=null&&e.expires_in?Date.now()+e.expires_in*1e3:0}function eT(e){var t,r,n,o,a,i,s,c,u,l,d,p,h,v,g,y,m,$,S,A,q,ne,se,ae,pe,ce,re;return {token:(t=e==null?void 0:e.access_token)!=null?t:"",expiresAt:XE(e),email:(n=(r=e==null?void 0:e.extra)==null?void 0:r.raw_info.email)!=null?n:"",firstName:(a=(o=e==null?void 0:e.extra)==null?void 0:o.raw_info.first_name)!=null?a:"",lastName:(s=(i=e==null?void 0:e.extra)==null?void 0:i.raw_info.last_name)!=null?s:"",name:(u=(c=e==null?void 0:e.extra)==null?void 0:c.raw_info.name)!=null?u:"",permissions:(p=(d=(l=e==null?void 0:e.extra)==null?void 0:l.raw_info)==null?void 0:d.permissions)!=null?p:[],tenantLogoUrl:(g=(v=(h=e==null?void 0:e.extra)==null?void 0:h.raw_info)==null?void 0:v.tenant_logo_url)!=null?g:"",tenantName:($=(m=(y=e==null?void 0:e.extra)==null?void 0:y.raw_info)==null?void 0:m.tenant_name)!=null?$:"",tenantTheme:(q=(A=(S=e==null?void 0:e.extra)==null?void 0:S.raw_info)==null?void 0:A.tenant_theme)!=null?q:"",tenantWholesalerId:(ae=(se=(ne=e==null?void 0:e.extra)==null?void 0:ne.raw_info)==null?void 0:se.tenant_wholesaler_id)!=null?ae:"",user:e,userId:(re=(ce=(pe=e==null?void 0:e.extra)==null?void 0:pe.raw_info)==null?void 0:ce.user_id)!=null?re:""}}const tT=e=>{switch(e){case ln.TakeOver:return {model:"application_forms/clones",action:"create"};case ln.SendToExchange:return {model:"application_forms/application_forms_question_sets",action:"create"};case ln.FeatureCheck:return {model:"features",action:"index"};default:return {model:"",action:""}}};function rT({user:e=JE(),model:t,action:r,object:n}){var o,a;if(!((a=(o=e.extra)==null?void 0:o.raw_info)!=null&&a.permissions))return !1;let i=!1,s=n==null;for(const c of e.extra.raw_info.permissions){const[u,l,d,p]=c.split(":");if(d===t&&p===r)if(i=!0,n)switch(u){case"Tenant":l===n.tenant_id&&(s=!0);break;case"Owner":l===n.owner_id&&(s=!0);break;case"UserGroup":l===n.user_group_id&&(s=!0);break}else break}return i&&s}var nT=Object.defineProperty,oT=Object.defineProperties,aT=Object.getOwnPropertyDescriptors,Al=Object.getOwnPropertySymbols,iT=Object.prototype.hasOwnProperty,sT=Object.prototype.propertyIsEnumerable,Ol=(e,t,r)=>t in e?nT(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,cT=(e,t)=>{for(var r in t||(t={}))iT.call(t,r)&&Ol(e,r,t[r]);if(Al)for(var r of Al(t))sT.call(t,r)&&Ol(e,r,t[r]);return e},uT=(e,t)=>oT(e,aT(t)),lT=(e,t)=>t.reduce((r,n)=>uT(cT({},r),{[n]:e[n]}),{}),dT=Oa;function pT(e,t){return dT(e,t)}var fT=pT;lr(fT);function vT(e,t,r){switch(r.length){case 0:return e.call(t);case 1:return e.call(t,r[0]);case 2:return e.call(t,r[0],r[1]);case 3:return e.call(t,r[0],r[1],r[2])}return e.apply(t,r)}var hT=vT,gT=hT,Sl=Math.max;function mT(e,t,r){return t=Sl(t===void 0?e.length-1:t,0),function(){for(var n=arguments,o=-1,a=Sl(n.length-t,0),i=Array(a);++o<a;)i[o]=n[t+o];o=-1;for(var s=Array(t+1);++o<t;)s[o]=n[o];return s[t]=r(i),gT(e,this,s)}}var bT=mT;function yT(e){return function(){return e}}var _T=yT,$T=He,wT=function(){try{var e=$T(Object,"defineProperty");return e({},"",{}),e}catch(t){}}(),jl=wT,AT=_T,Cl=jl,OT=On,ST=Cl?function(e,t){return Cl(e,"toString",{configurable:!0,enumerable:!1,value:AT(t),writable:!0})}:OT,jT=ST,CT=800,PT=16,IT=Date.now;function ET(e){var t=0,r=0;return function(){var n=IT(),o=PT-(n-r);if(r=n,o>0){if(++t>=CT)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}var TT=ET,qT=jT,xT=TT,kT=xT(qT),DT=kT,MT=On,NT=bT,LT=DT;function RT(e,t){return LT(NT(e,t,MT),e+"")}var FT=RT;function QT(e,t,r,n){for(var o=e.length,a=r+(n?1:-1);n?a--:++a<o;)if(t(e[a],a,e))return a;return -1}var UT=QT;function VT(e){return e!==e}var BT=VT;function GT(e,t,r){for(var n=r-1,o=e.length;++n<o;)if(e[n]===t)return n;return -1}var zT=GT,WT=UT,HT=BT,ZT=zT;function KT(e,t,r){return t===t?ZT(e,t,r):WT(e,HT,r)}var YT=KT,JT=YT;function XT(e,t){var r=e==null?0:e.length;return !!r&&JT(e,t,0)>-1}var Pl=XT;function eq(e,t,r){for(var n=-1,o=e==null?0:e.length;++n<o;)if(r(t,e[n]))return !0;return !1}var Il=eq,tq=la,rq=Pl,nq=Il,oq=ra,aq=ga,iq=da,sq=200;function cq(e,t,r,n){var o=-1,a=rq,i=!0,s=e.length,c=[],u=t.length;if(!s)return c;r&&(t=oq(t,aq(r))),n?(a=nq,i=!1):t.length>=sq&&(a=iq,i=!1,t=new tq(t));e:for(;++o<s;){var l=e[o],d=r==null?l:r(l);if(l=n||l!==0?l:0,i&&d===d){for(var p=u;p--;)if(t[p]===d)continue e;c.push(l);}else a(t,d,n)||c.push(l);}return c}var uq=cq,El=dr,lq=va,dq=le,Tl=El?El.isConcatSpreadable:void 0;function pq(e){return dq(e)||lq(e)||!!(Tl&&e&&e[Tl])}var fq=pq,vq=zc,hq=fq;function ql(e,t,r,n,o){var a=-1,i=e.length;for(r||(r=hq),o||(o=[]);++a<i;){var s=e[a];t>0&&r(s)?t>1?ql(s,t-1,r,n,o):vq(o,s):n||(o[o.length]=s);}return o}var gq=ql;function mq(){}var bq=mq,Va=nu,yq=bq,_q=pa,$q=1/0,wq=Va&&1/_q(new Va([,-0]))[1]==$q?function(e){return new Va(e)}:yq,Aq=wq,Oq=la,Sq=Pl,jq=Il,Cq=da,Pq=Aq,Iq=pa,Eq=200;function Tq(e,t,r){var n=-1,o=Sq,a=e.length,i=!0,s=[],c=s;if(r)i=!1,o=jq;else if(a>=Eq){var u=t?null:Pq(e);if(u)return Iq(u);i=!1,o=Cq,c=new Oq;}else c=t?[]:s;e:for(;++n<a;){var l=e[n],d=t?t(l):l;if(l=r||l!==0?l:0,i&&d===d){for(var p=c.length;p--;)if(c[p]===d)continue e;t&&c.push(d),s.push(l);}else o(c,d,r)||(c!==s&&c.push(d),s.push(l));}return s}var xl=Tq,qq=uq,xq=gq,kl=xl;function kq(e,t,r){var n=e.length;if(n<2)return n?kl(e[0]):[];for(var o=-1,a=Array(n);++o<n;)for(var i=e[o],s=-1;++s<n;)s!=o&&(a[o]=qq(a[o]||i,e[s],t,r));return kl(xq(a,1),t,r)}var Dq=kq,Mq=wn,Nq=St;function Lq(e){return Nq(e)&&Mq(e)}var Rq=Lq;function Fq(e){var t=e==null?0:e.length;return t?e[t-1]:void 0}var Qq=Fq,Uq=Wc,Vq=FT,Bq=Dq,Gq=Rq,zq=Qq,Wq=Vq(function(e){var t=zq(e);return t=typeof t=="function"?t:void 0,Bq(Uq(e,Gq),void 0,t)}),Hq=Wq;lr(Hq);function Zq(e){var t,r;const n=new Map;if((r=(t=e==null?void 0:e.question_set)==null?void 0:t.question_groups)!=null&&r.length){const o=e.question_set.question_groups,a=o.flatMap(i=>i.question_ids);o.forEach(i=>{const s=i.parent_question_id,c=a.includes(s),u=i.question_ids.includes(s);c&&!u||n.set(s,Dn(e,i));});}return n}function Kq(e,t,r){const n=o=>!!t.answers.find(a=>a.question.id===r&&a.answer_pool_id===o);return e.reduce(([o,a],i)=>n(i)?[[...o,i],a]:[o,[...a,i]],[[],[]])}function Yq(e,t){let r=new Map;const n=(o,a)=>{const i=e.answers.find(c=>c.answer_pool_id===o),s=e.answers.find(c=>c.answer_pool_id===a);return (i.created_at instanceof Date?i.created_at.getTime():new Date(i.created_at).getTime())-(s.created_at instanceof Date?s.created_at.getTime():new Date(s.created_at).getTime())};return pC(Array.from(t),["1.parentQuestionAnswer.question.order","1.parentQuestionAnswer.created_at"]).forEach(([o,a])=>{const[i,s]=Kq(Array.from(new Set(a.answers.map(u=>u.answer_pool_id))),e,o);let c=0;i.sort(n).forEach(u=>{const l=kn({appFormQuestionSet:e,questionGroupsWithAns:a,parentQuestionAnswer:e.answers.find(d=>d.question.id===o&&d.answer_pool_id===u),answerPoolId:u,order:c++});r.set(l.parentQuestionAnswer.id,l);}),s.forEach(()=>{r=Dl(e,r,o,c);});}),r}function Dl(e,t,r,n){return e.answers.filter(o=>o.question.id===r).forEach(o=>{const a=e.question_set.question_groups.find(i=>i.parent_question_id===o.question.id);if(a){const i=Dn(e,a),s=kn({appFormQuestionSet:e,questionGroupsWithAns:i,parentQuestionAnswer:o,answerPoolId:o.answer_pool_id||null,order:n++});t.set(o.id,s);}}),t}function kn({appFormQuestionSet:e,questionGroupsWithAns:t,parentQuestionAnswer:r,answerPoolId:n,order:o}){const a=t.answers.filter(s=>s.answer_pool_id===n),i=Jq(e,t,n);return i.forEach((s,c)=>{a.find(u=>u.id===c)||a.push(s.parentQuestionAnswer);}),{answers:a.sort(Ba),group_type:t.group_type,nestedGroupsWithAnswers:i,label:t.label,order:o,parentQuestionAnswer:r,groupId:t.groupId,poolId:r.answer_pool_id,questionIds:t.questionIds}}function Jq(e,t,r){var n;const o=new Map;let a=0;return (n=t.nestedGroupsWithAnswers)==null||n.forEach((i,s)=>{const c=e.answers.find(u=>u.question.id===s&&u.answer_pool_id===r);if(c){const u=kn({appFormQuestionSet:e,questionGroupsWithAns:i,parentQuestionAnswer:c,answerPoolId:r,order:a++});o.set(u.parentQuestionAnswer.id,u);}else r?c||Xq(e,o,s,r,a):Dl(e,o,s,a);}),o}function Xq(e,t,r,n,o){return e.answers.filter(a=>a.question.id===r&&a.parent_answer_pool_id===n).forEach(a=>{const i=e.question_set.question_groups.find(s=>s.parent_question_id===a.question.id);if(i){const s=Dn(e,i),c=kn({appFormQuestionSet:e,questionGroupsWithAns:s,parentQuestionAnswer:a,answerPoolId:a.answer_pool_id||"",order:o++});t.set(a.id,c);}}),t}function Dn(e,t){const r=e.answers.filter(o=>t.question_ids.includes(o.question.id)&&o.display_type!=="private").sort(Ba),n=e.answers.find(o=>o.question.id===t.parent_question_id);return {answers:r,group_type:t.group_type,nestedGroupsWithAnswers:ex(e,t.id),label:t.label,order:0,parentQuestionAnswer:n,poolId:n==null?void 0:n.answer_pool_id,groupId:t.id,questionIds:t.question_ids}}function ex(e,t){const r=new Map,n=e.question_set.question_groups.find(o=>o.id===t);return e.question_set.question_groups.map(o=>o.parent_question_id).filter(o=>o!==(n==null?void 0:n.parent_question_id)).forEach(o=>{if(n!=null&&n.question_ids.includes(o)){const a=e.question_set.question_groups.find(i=>i.parent_question_id===o);a&&r.set(a.parent_question_id,Dn(e,a));}}),r}function Ba(e,t){return e.question.order===t.question.order?new Date(e.created_at).getTime()-new Date(t.created_at).getTime():e.question.order-t.question.order}function Ml(e){return e.group_type===ea.Stepper}function tx(e){return e.group_type===ea.Step}function rx(e,t){return e.parentQuestionAnswer.question.order-t.parentQuestionAnswer.question.order}var Nl=jl;function nx(e,t,r){t=="__proto__"&&Nl?Nl(e,t,{configurable:!0,enumerable:!0,value:r,writable:!0}):e[t]=r;}var ox=nx;function ax(e,t,r,n){for(var o=-1,a=e==null?0:e.length;++o<a;){var i=e[o];t(n,i,r(i),e);}return n}var ix=ax,sx=$u;function cx(e,t,r,n){return sx(e,function(o,a,i){t(n,o,r(o),i);}),n}var ux=cx,lx=ix,dx=ux,px=_u,fx=le;function vx(e,t){return function(r,n){var o=fx(r)?lx:dx,a=t?t():{};return o(r,e,px(n),a)}}var hx=vx,gx=ox,mx=hx,bx=Object.prototype,yx=bx.hasOwnProperty,_x=mx(function(e,t,r){yx.call(e,r)?e[r].push(t):gx(e,r,[t]);}),$x=_x;lr($x);var wx=xl;function Ax(e){return e&&e.length?wx(e):[]}var Ox=Ax;lr(Ox);const Ll=e=>e_.includes(e),Mt=e=>(t,r)=>{if(t)return r.push(e)},Mn=Mt(ee.Required),Ga=Mt(ee.MinimumValue),Sx=Mt(ee.MinimumDate),za=Mt(ee.MaximumValue),jx=Mt(ee.MaximumDate),Wa=Mt(ee.Pattern),Cx=e=>{const{field_type:t,reference_type:r,min_date:n,min_value:o,max_date:a,max_value:i,pattern:s}=e.question,{required:c}=e,u=[];switch(Da(t)){case w.Text:Mn(c,u),r===Ot.Accumulator?(za(o!=null,u),Ga(i!=null,u)):r!==Ot.Street1&&(Ga(o!=null,u),za(i!=null,u),Wa(s!=null,u));break;case w.Masked:Mn(c,u),Ga(o!=null,u),za(i!=null,u),Wa(s!=null,u);break;case w.Date:Mn(c,u),Sx(n!=null,u),jx(a!=null,u),Wa(s!=null,u);break;case w.DropDown:Mn(c,u);}return u},Rl=e=>e!=null&&e.question?Cx(e):[],Px=(e,t)=>{var r,n,o,a,i,s,c,u,l;let d;switch(t){case ee.Required:Da(e.question.field_type)===w.DropDown?d=!!((n=e.choice_id)!=null?n:((r=e.choice_ids)!=null?r:[]).length):d=e.value===0||e.value!=null&&e.value!="";break;case ee.MinimumValue:Ll(e.question.field_type)?d=+((o=e.value)!=null?o:0)>=e.question.min_value:d=((i=(a=e.value)!=null?a:"")==null?void 0:i.length)>=e.question.min_value;break;case ee.MaximumValue:Ll(e.question.field_type)?d=+((s=e.value)!=null?s:0)<=e.question.max_value:d=((u=(c=e.value)!=null?c:"")==null?void 0:u.length)<=e.question.max_value;break;case ee.MinimumDate:d=!!e.value&&new Date(e.value)>=new Date(e.question.min_date);break;case ee.MaximumDate:d=!!e.value&&new Date(e.value)<=new Date(e.question.max_date);break;case ee.Pattern:d=!!((l=e.value)!=null?l:"").toString().match(new RegExp(e.question.pattern));break;default:d=!0;}return d},Ix=e=>(t,r)=>Px(e,r)?t:[...t,r],Ha=(e,t=[])=>{if(!e.required&&!IE(e))return [];const r=t.reduce(Ix(e),[]);return r.includes(ee.Required)?[ee.Required]:r.includes(ee.Pattern)?[ee.Pattern]:r},Fl=e=>{const t=Rl(e);return Ha(e,t)};function Ql(e){var t;return e?((t=e.question)==null?void 0:t.reference_type)===Ot.Accumulator?!e.active||Fl(e).length===0:!e.active||e.is_completed||Fl(e).length===0:!0}function Ul(e){return (e==null?void 0:e.find(t=>!Ql(t)))===void 0}const Ex=["mqs_add_driver","mqs_add_vehicle"];function Tx(e){let t=!1;const r=e.question_set.question_pools.map(n=>n.parent_question_id);return r.length>0&&(t=!e.answers.some(n=>!r.includes(n.question.id)||!Ex.includes(n.question.code))),t}function Za(e){return e.answers.length===0?!0:Ul(e.answers)&&!Tx(e)}function qx(e,t){return (e!=null?e:[]).find(r=>r.question_set.code===t)}function xx(e,t){return window.dispatchEvent(new CustomEvent(`${xn}${e}`,{detail:t}))}function V(e){return !!(window&&window.dispatchEvent(new ErrorEvent("error",{error:e})))}var kx=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});function Dx(e,t){return kx(this,null,function*(){var r,n;Ua(At.token,e),Ua(At.cableToken,t);try{(n=(r=window.BpSdk)==null?void 0:r.store)==null||n.dispatch({type:"auth/Set Auth",payload:{token:e}});}catch(o){}xx(At.token,e);})}for(var Mx=[],Ka=0;Ka<256;++Ka)Mx.push((Ka+256).toString(16).substr(1));var Nx=Object.defineProperty,Lx=Object.defineProperties,Rx=Object.getOwnPropertyDescriptors,Vl=Object.getOwnPropertySymbols,Fx=Object.prototype.hasOwnProperty,Qx=Object.prototype.propertyIsEnumerable,Bl=(e,t,r)=>t in e?Nx(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Gl=(e,t)=>{for(var r in t||(t={}))Fx.call(t,r)&&Bl(e,r,t[r]);if(Vl)for(var r of Vl(t))Qx.call(t,r)&&Bl(e,r,t[r]);return e},zl=(e,t)=>Lx(e,Rx(t));const oe=(e,t,r)=>{const n=zl(Gl({},e.map),{[t]:r}),o=Object.values(n).reduce((a,i)=>a===fe.Error||i===fe.Error?fe.Error:a===fe.Loading||i===fe.Loading?fe.Loading:a,fe.Loaded);return zl(Gl({},e),{state:o,map:n})};var Ux=Object.defineProperty,Vx=Object.defineProperties,Bx=Object.getOwnPropertyDescriptors,Wl=Object.getOwnPropertySymbols,Gx=Object.prototype.hasOwnProperty,zx=Object.prototype.propertyIsEnumerable,Hl=(e,t,r)=>t in e?Ux(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Nn=(e,t)=>{for(var r in t||(t={}))Gx.call(t,r)&&Hl(e,r,t[r]);if(Wl)for(var r of Wl(t))zx.call(t,r)&&Hl(e,r,t[r]);return e},Ln=(e,t)=>Vx(e,Bx(t));const Zl=e=>e.reduce((t,r)=>Ln(Nn({},t),{[r.id]:r.value}),{}),Kl=e=>e.reduce((t,r)=>Ln(Nn({},t),{[r.id]:r.name}),{}),Yl=e=>e.reduce((t,r)=>Ln(Nn({},t),{[r.id]:r.name}),{}),Jl=e=>e.reduce((t,r)=>Ln(Nn({},t),{[r.id]:r.name}),{}),Ya=({configurations:e})=>e!=null?e:[],Wx=e=>e.application,Nt=f([Wx],e=>e.questionSets),de=f(Nt,e=>e.list),Ja=f(Nt,e=>e.active),be=f(de,Ja,(e,t)=>e.find(r=>r.question_set_id===t)),Xl=f(Nt,e=>e.idOfNextActiveQuestionSetNotInState);f(be,e=>{var t;return (t=e==null?void 0:e.question_set)==null?void 0:t.code});f(de,e=>e.filter(t=>t.is_active&&t.answers.some(r=>r.active)));f(de,e=>e.filter(t=>t.is_active&&t.is_completed));const ed=f(de,e=>e.filter(t=>t.is_active&&!t.is_completed));f([de,(e,t)=>t],(e,t)=>e.find(r=>r.id===t));const Jx=f([de,(e,t)=>t],(e,t)=>qx(e,t));f(Nt,e=>e.completedQuestionSetAnswerHasChanged);const ek=f(Nt,e=>e.parentQuestionSets);f([be,(e,t)=>t],(e,t)=>e==null?void 0:e.answers.find(r=>r.id===t));f([be,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e==null?void 0:e.answers)==null?void 0:r.filter(o=>(o==null?void 0:o.answer_pool_id)===t))!=null?n:[]});f([be,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e==null?void 0:e.answers)==null?void 0:r.filter(o=>(o==null?void 0:o.question.question_group_id)===t))!=null?n:[]});const ok=f([be,(e,t)=>t],(e,t)=>{var r;return (r=e==null?void 0:e.answers.filter(n=>n.question.question_group_id===t.groupId&&n.answer_pool_id===t.poolId))!=null?r:[]});f([de,(e,t)=>t],(e,t)=>t instanceof RegExp?e.flatMap(r=>r.answers).filter(r=>t.test(r.question.code)):e.flatMap(r=>r.answers).filter(r=>r.question.code.includes(t)));f([de,(e,t)=>t],(e,t)=>t instanceof RegExp?e.flatMap(r=>r.answers).filter(r=>t.test(r.question.reference_type)):e.flatMap(r=>r.answers).filter(r=>r.question.code.includes(t)));f([de],e=>{var t,r;const n=e.flatMap(a=>a.answers).find(a=>a.question.reference_type==="business_class"),o=(r=(t=n==null?void 0:n.question)==null?void 0:t.choices)==null?void 0:r.find(a=>a.id===(n==null?void 0:n.choice_id));return o==null?void 0:o.value});const td=e=>e.application,ye=f(td,e=>e.form),ck=f(ye,e=>e.id);f(ye,e=>{var t;return (t=e.features)!=null?t:[]});f(ye,e=>{var t;return (t=e.reference_id)!=null?t:""});f(ye,e=>e.percent_completed);f(td,e=>e.progress.meta.form.forwardProgressPercent);const fk=f(ye,e=>e.state);f(ye,e=>({source:e.source,metadata:e.source_metadata}));const rd=f(ye,e=>e.appetite),nd=e=>e.application.form.buckets;f(nd,ye,(e,t)=>e.find(r=>r.id===t.current_bucket_id));f(rd,e=>{const t=mk(e);return bk(t)});function mk(e){return e.flatMap(t=>t.carriers.filter(r=>r.is_enrolled))}function bk(e){const t=new Set;return e.reduce((r,n)=>(t.has(n.tenant_id)||(t.add(n.tenant_id),r.push(n)),r),[])}const J=e=>e.auth;f(J,e=>e==null?void 0:e.refreshCount);const od=f(J,e=>e==null?void 0:e.user);f(J,e=>e==null?void 0:e.token);f(J,e=>{var t;return (t=e==null?void 0:e.expiresAt)!=null?t:Date.now()-1});const ad=f(J,e=>{var t;return (t=e==null?void 0:e.expiresAt)!=null?t:Date.now()-1}),id=f(J,e=>e==null?void 0:e.userId);f(J,e=>e==null?void 0:e.email);f(J,e=>e==null?void 0:e.name);f(J,e=>e==null?void 0:e.firstName);f(J,e=>e==null?void 0:e.lastName);f(J,e=>e==null?void 0:e.tenantLogoUrl);f(J,e=>e==null?void 0:e.tenantName);f(J,e=>e==null?void 0:e.tenantTheme);f(J,e=>e==null?void 0:e.terminalTheme);const sd=f(ad,e=>e>Date.now()),Xa=e=>e.application.carriersProducts;f(Xa,e=>e.filter(t=>t.carrier.online));f(Xa,e=>e.filter(t=>!t.carrier.online));const qk=e=>e.status.remote.carriersProducts.state,cd=e=>e.application.choices,xk=f([cd,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e[t])==null?void 0:r.list)!=null?n:[]}),Ar=e=>e.config,ud=f([Ar,J],(e,t)=>({env:e.env,uri:e.uri,token:t.token}));f(Ar,e=>e.googleApiToken);f(Ar,e=>e.cableToken);const Mk=f(Ar,e=>e.debug||!1),Lt=f(Ar,e=>e.options),Nk=f(Lt,e=>!!e.enableAutocomplete);f(Lt,e=>!!e.enableTrailerSupport);const Rk=f(Lt,e=>!!e.enableDetailedRelevanceChips),Fk=f(Lt,e=>!!e.enableQuestionSetNavigation),Qk=f(Lt,e=>!!e.showParentQuestionSetsInNavigation);var Uk=Object.defineProperty,Vk=Object.defineProperties,Bk=Object.getOwnPropertyDescriptors,ld=Object.getOwnPropertySymbols,Gk=Object.prototype.hasOwnProperty,zk=Object.prototype.propertyIsEnumerable,dd=(e,t,r)=>t in e?Uk(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,pd=(e,t)=>{for(var r in t||(t={}))Gk.call(t,r)&&dd(e,r,t[r]);if(ld)for(var r of ld(t))zk.call(t,r)&&dd(e,r,t[r]);return e},fd=(e,t)=>Vk(e,Bk(t));const Rt=e=>e.application.form.eligible_carriers_products;f(Rt,e=>ei(e));const vd=f(Rt,e=>ti(e)),Rn=f(Rt,e=>e.filter(t=>t.market?t.market===Qo.BP_QUOTABLE:t.carrier.online));f(Rn,e=>ei(e));const hd=f(Rn,e=>ti(e)),Fn=f(Rt,e=>e.filter(t=>t.market?t.market===Qo.OFFLINE:!t.carrier.online));f(Fn,e=>ei(e));f(Fn,e=>ti(e));f([Rt,(e,t)=>t],(e,t)=>e.reduce((r,n)=>n.carrier.id===t?[...r,n.product]:r,[]));function ei(e){return Object.values(e.reduce((t,r)=>fd(pd({},t),{[r.carrier.id]:r.carrier}),{}))}function ti(e){const t=e.reduce((r,n)=>{const o=r.get(n.carrier.id);return o?o.products.push(n.product):r.set(n.carrier.id,fd(pd({},n.carrier),{products:[n.product]})),r},new Map);return Array.from(t.values())}const Qn=e=>e.application.form,gd=e=>e.status.remote,Xk=f([Qn],e=>e.locked);f([Qn],e=>e.taken_over);const t2=f([Qn],e=>e.cloned);f([Qn],e=>e.inactive);const n2=f([gd],e=>e.application.form===b.Loaded),md=f([gd],e=>e.application.form===b.Loading),Or=e=>e.application.form.messages||[];f([Or,(e,t)=>t],(e,t)=>e.filter(r=>r.component===$t.Answer&&r.metadata.answer_id===t));f([Or,be],(e,t)=>{const r=(t==null?void 0:t.answers.map(n=>n.id))||[];return !e.length||!r.length?[]:e.filter(n=>n.component===$t.Answer&&r.includes(n.metadata.answer_id||""))});f([Or,(e,t)=>t],(e,t)=>e.filter(r=>r.component===$t.QuoteRequest&&r.metadata.quote_request_id===t));const s2=f(Or,e=>e.filter(t=>t.component===$t.Application)),c2=e=>e.application.pendingProcesses,u2=f(c2,Ja,(e,t)=>e.some(r=>r.question_set_id===t&&(r.state===en.Pending||r.state===en.Queued||r.state===en.Executing)));var l2=Object.defineProperty,bd=Object.getOwnPropertySymbols,d2=Object.prototype.hasOwnProperty,p2=Object.prototype.propertyIsEnumerable,yd=(e,t,r)=>t in e?l2(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,f2=(e,t)=>{for(var r in t||(t={}))d2.call(t,r)&&yd(e,r,t[r]);if(bd)for(var r of bd(t))p2.call(t,r)&&yd(e,r,t[r]);return e};f(J,e=>e.permissions);const h2=f([id,ye],(e,t)=>t!=null&&e===t.owner_id),g2=f([od,(e,t)=>f2({},t)],(e,{permission:t,object:r,action:n})=>{const o=tT(t);return rT({user:e,object:r,action:n!=null?n:o.action,model:o.model||t})});f(J,e=>!!e.options.impersonated);f(be,(e,t)=>t,(e,t)=>{var r;if(e)return (r=e.question_set)==null?void 0:r.question_pools.find(n=>n.parent_question_id===t.question.id)});const tt=e=>e.application.quotes;f([tt,(e,t)=>t],(e,t)=>e.find(r=>r.id===t));f([tt,(e,t)=>t],(e,t)=>e.filter(r=>r.request_status===t));f([tt,(e,t)=>t],(e,t)=>e.filter(r=>r.products.includes(t)));f([tt,(e,t)=>t],(e,t)=>e.filter(r=>r.products.filter(n=>n.id===t).length));f([tt,(e,t)=>t],(e,t)=>e.filter(r=>r.products.filter(n=>n.name===t).length));const _d=e=>e.status.remote.quotes.state===b.Loading;f(md,_d,(e,t)=>e||!t);const S2=e=>e.application,ri=f([S2],e=>e.form.real_time_eligibility.carriers||[]);f([ri],e=>e.filter(t=>t.eligible));f([ri],e=>e.filter(t=>!t.eligible));const $d=f([de],e=>EE(e.flatMap(t=>t.answers),"coverage_type"));f([$d],e=>{const t=e.length?PE(e[0]):null;return Array.isArray(t)?t:t?[t]:[]});const ni=(e,t,r=0)=>({active:!0,annual_price:"",application_form_id:"",bindable:!0,calls_to_action:[],carrier:{code:e.code,id:e.id,logo_url:e.logo_url,name:e.name,online:e.online,labels:e.labels,opt_in_required:e.opt_in_required,opted_in:e.opted_in},carrier_bind_url:"",consumer_selected:!0,coverage:{disclaimer:"",limit_and_deductible:[],limit_and_deductible_default:[]},created_at:new Date,customizable:!1,downpayment:"",expired:!1,finished_at:new Date,focused:!1,id:e.id,monthly_price:"",number_of_installments:0,online:e.online||!1,order:r,owner_id:"",preferred:!1,price_details:{fees:[],new_total_premium:"",taxes:[],term:""},products:e.products,quote_number:"",recommended:!1,request_status:t,retrieve_url:"",semiannual_price:"",selected:!1,started_at:new Date,surcharge:0,tenant_id:"",translated_messages:[],updated_at:new Date,actions:[],attachments:[]}),wd=e=>{var t;return (t=(e==null?void 0:e.opt_in_required)&&!(e!=null&&e.opted_in))!=null?t:!1},Ad=(e,t)=>{const r=t.some(n=>n.question_set.tenant_id===(e==null?void 0:e.id));return wd(e)&&!r},Od="not_opted_in",Sd="incomplete_question_set",jd=f(ed,hd,(e,t)=>e.reduce((r,n)=>{const o=t.find(a=>a.id===n.question_set.tenant_id);return [...r,...o?[ni(o,Sd,null)]:[]]},[])),Cd=(e=[])=>{const t=[Uo.NON_ADMITTED,Uo.ADDITIONAL_MARKET];return e.some(r=>t.includes(r))},Pd=f(vd,e=>e.filter(t=>Cd(t.labels))),Id=f(Pd,de,(e,t)=>e.filter(r=>Ad(r,t)).map(r=>ni(r,Od))),Ed=f(Fn,Rn,(e,t)=>e.filter(r=>!t.some(n=>n.carrier.id===r.carrier.id&&n.product.id===r.product.id)).map(r=>({active:!0,annual_price:"",application_form_id:r.application_form_id,bindable:!0,calls_to_action:[],carrier:r.carrier,carrier_bind_url:"",consumer_selected:!0,coverage:{disclaimer:"",limit_and_deductible:[],limit_and_deductible_default:[]},created_at:new Date,customizable:!1,downpayment:"",expired:!1,finished_at:new Date,focused:!1,id:r.carriers_product_id,monthly_price:"",number_of_installments:0,online:!1,order:0,owner_id:"",preferred:!1,price_details:{fees:[],new_total_premium:"",taxes:[],term:""},products:[r.product],quote_number:"",recommended:!1,request_status:Vo.referral,retrieve_url:"",semiannual_price:"",selected:!1,started_at:new Date,surcharge:0,tenant_id:"",translated_messages:[],updated_at:new Date,actions:[],attachments:[]})));f(tt,jd,Id,Ed,(e,t,r,n)=>[...e||[],...t||[],...r||[],...n||[]]);const Un=e=>e.application.refilters;f([Un,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e[t])==null?void 0:r.list)!=null?n:[]});const T2=f([Un,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e[t])==null?void 0:r.list.filter(o=>o.value===null))!=null?n:[]});f([Un,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e[t])==null?void 0:r.list.filter(o=>o.value!==null))!=null?n:[]});const x2=e=>e.status.remote.answers.state,k2=e=>e.status.remote.answers.map,D2=e=>e.status.remote.application.form,Td=e=>e.status.remote,M2=f(Td,e=>e.questionPools.state);f(Td,(e,t)=>t,(e,t)=>e.questionPools.map[t]);const qd=e=>e.status.remote,L2=f(qd,e=>e.questionSets.state),oi=f(qd,e=>e.questionSets.map);f(oi,be,Xl,(e,t,r)=>r?!0:t?e[t.id]===b.Loading:!1);f(oi,be,(e,t)=>t,(e,t,r)=>t?e[r]===b.Loading:!1);const Q2=e=>e.status.remote,U2=f(Q2,e=>e.quotes.state),ve=e=>e.status.local;f(ve,e=>e.health);f(ve,e=>e.state);f(ve,e=>e.state===x.Loading);f(ve,e=>e.config===x.Configured);f(ve,e=>e.auth===x.Configured);const H2=f(ve,e=>e.user===x.Configured);f(ve,e=>e.auth===x.Configured);f(ve,e=>e.auth===x.Configured&&e.user===x.Configured);f(ve,e=>e.auth===x.Configured&&e.config===x.Configured&&e.user===x.Configured);f(ve,e=>e.socket);const X2=e=>e.viewState,Fe=f(X2,e=>e.vehicleOptions);f([(e,t)=>({state:e,questionGroupId:t})],({state:e,questionGroupId:t})=>({configurations:Ud(e,t),makes:Fd(e,t),models:Qd(e,t),selection:Vd(e,t),vin:Bd(e,t),years:Rd(e,t)}));const xd=f(Fe,e=>e.years),kd=f(Fe,e=>e.makes),Dd=f(Fe,e=>e.models),Md=f(Fe,e=>e.configurations),Nd=f(Fe,e=>e.selection),Ld=f(Fe,e=>e.vin),Rd=f([xd,(e,t)=>t],(e,t)=>e[t]),Fd=f([kd,(e,t)=>t],(e,t)=>e[t]),Qd=f([Dd,(e,t)=>t],(e,t)=>e[t]),Ud=f([Md,(e,t)=>t],(e,t)=>e[t]),Vd=f([Nd,(e,t)=>t],(e,t)=>e[t]),Bd=f([Ld,(e,t)=>t],(e,t)=>e[t]),tD=f([Fe,(e,t)=>t],(e,t)=>{var r,n,o,a,i,s,c,u,l,d,p,h,v,g;const y=e.selection,m={year:(n=(r=y==null?void 0:y[t])==null?void 0:r.yearId)!=null?n:"",make:"",model:"",configuration:""},$=(o=y[t])==null?void 0:o.makeId;$&&(m.make=(s=(i=(a=e.makes)==null?void 0:a[t])==null?void 0:i[$])!=null?s:"");const S=(c=y[t])==null?void 0:c.modelId;S&&(m.model=(d=(l=(u=e.models)==null?void 0:u[t])==null?void 0:l[S])!=null?d:"");const A=(p=y[t])==null?void 0:p.configurationId;return A&&(m.configuration=(g=(v=(h=e.configurations)==null?void 0:h[t])==null?void 0:v[A])!=null?g:""),m}),rD=e=>e.status.remote,rt=f(rD,e=>e.vehicles);f(rt,e=>e.configurations);f(rt,e=>e.makes);f(rt,e=>e.models);const Gd=f(rt,e=>e.selection),zd=f(rt,e=>e.vins);f(rt,e=>e.years);const sD=f([Gd,(e,t)=>t],(e,t)=>e[t]),cD=f([zd,(e,t)=>t],(e,t)=>e[t]),Wd=e=>e.viewState.features;f([Wd,(e,t)=>t],(e,t)=>e.list.includes(t));var lD=Object.defineProperty,dD=Object.defineProperties,pD=Object.getOwnPropertyDescriptors,Hd=Object.getOwnPropertySymbols,fD=Object.prototype.hasOwnProperty,vD=Object.prototype.propertyIsEnumerable,Zd=(e,t,r)=>t in e?lD(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Vn=(e,t)=>{for(var r in t||(t={}))fD.call(t,r)&&Zd(e,r,t[r]);if(Hd)for(var r of Hd(t))vD.call(t,r)&&Zd(e,r,t[r]);return e},Kd=(e,t)=>dD(e,pD(t)),Qe=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});function Yd(e){return Qe(this,null,function*(){var t,r;return (r=(t=window==null?void 0:window.BpSdk)==null?void 0:t.refreshGuestSetup(e))!=null?r:Promise.reject("SDK must be configured before loading a guest application")})}const nt="applicationForms",Sr=L(`${nt}/LoadOne`,(e,t)=>Qe(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield ze.getApplicationForm(r.id,r.config)).parsedBody}catch(o){const a=new M("Error loading app form",o);throw V(a),n(a),a}})),jr=L(`${nt}/LoadLatestOne`,(e,t)=>Qe(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield ze.getLatestApplicationForm(r.id,r.config)).parsedBody}catch(o){const a=new M("Error loading app form",o);throw V(a),n(a),a}})),ot=L(`${nt}/LoadOneFromToken`,(e,t)=>Qe(void 0,[e,t],function*(r,{rejectWithValue:n}){var o;try{const a=yield ze.getApplicationFormFromToken(r.token_id,r);if(a.parsedBody)return Ua(bt.cableToken,(o=a.parsedBody.application_form)==null?void 0:o.cable_token),a.parsedBody.application_form;throw new M("Application form does not exist on HTTP response body")}catch(a){const i=new M("Error loading app form from token",a);throw V(i),n(i),i}})),Bn=L(`${nt}/LoadGuest`,(e,t)=>Qe(void 0,[e,t],function*(r,{rejectWithValue:n,dispatch:o,getState:a}){try{const i=wr(bt.token),s=wr(bt.cableToken),c=a();if(i&&s&&sd(c))return o(ot(Kd(Vn({},r),{token:i,token_id:s})));const{parsedBody:u}=yield ze.getTokenForAuthGuestFlow(r);return yield Dx(u.token,u.token_id),yield Yd(Vn(Vn({},c.config),r)),o(ot(Kd(Vn({},r),{token:wr(bt.token),token_id:wr(bt.cableToken)})))}catch(i){const s=new M("Error loading guest application form",i);throw V(s),n(s)}})),Cr=L(`${nt}/Clone`,(e,t)=>Qe(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield ze.cloneApplicationForm(r.appFormId,r.config)).parsedBody}catch(o){const a=new M("Error cloning application form",o);throw V(a),n(a),a}})),Pr=L(`${nt}/CreateAndLoad`,(e,t)=>Qe(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield ze.createApplicationForm(r.data,r.config)).parsedBody}catch(o){const a=new M("Error loading app form",o);throw V(a),n(a),a}})),Jd=L(`${nt}/GetEligibilityResults`,(e,t)=>Qe(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield ze.getEligibilityResults(r.appId,r.config)).parsedBody}catch(o){const a=new M("Error getting eligibility results",o);throw V(a),n(a),a}})),ai=[Bs,ir],k=[...ai,Sr.pending,jr.pending,ot.pending,Cr.pending,Pr.pending],Xd=[Sr.pending,jr.pending,ot.pending,Cr.pending,Pr.pending],_e=[Sr.fulfilled,jr.fulfilled,ot.fulfilled,Cr.fulfilled,Pr.fulfilled],ii=[Sr.rejected,jr.rejected,ot.rejected,Cr.rejected,Pr.rejected];var hD=Object.defineProperty,gD=Object.defineProperties,mD=Object.getOwnPropertyDescriptors,ep=Object.getOwnPropertySymbols,bD=Object.prototype.hasOwnProperty,yD=Object.prototype.propertyIsEnumerable,tp=(e,t,r)=>t in e?hD(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Ce=(e,t)=>{for(var r in t||(t={}))bD.call(t,r)&&tp(e,r,t[r]);if(ep)for(var r of ep(t))yD.call(t,r)&&tp(e,r,t[r]);return e},Gn=(e,t)=>gD(e,mD(t));const si=Ce({},Jm),rp=E({name:"form",initialState:si,reducers:{handleMessage:(e,{payload:t})=>np(e,t),receiveAppetite:(e,{payload:t})=>op(e,t),receiveApplicationForm:(e,{payload:t})=>ap(e,t),receiveApplicationFormProgress:(e,{payload:t})=>ip(e,t),receiveCarrierProducts:(e,{payload:t})=>sp(e,t),receiveRealTimeEligibility:(e,{payload:t})=>cp(e,t)},extraReducers:e=>{e.addCase(Jd.fulfilled,(t,{payload:r})=>{kt(r,t.eligible_carriers_products)&&(t.eligible_carriers_products=r);}).addCase(ge,(t,{payload:r})=>{let n=Ce({},t);return r.events.forEach(o=>{switch(o.event){case T.Message:n=np(n,o.data);break;case T.Appetite:n=op(n,o.data);break;case T.ApplicationForm:n=ap(n,o.data);break;case T.ApplicationFormPercentCompleted:n=ip(n,o.data);break;case T.CarrierProducts:n=sp(n,o.data);break;case T.RealTimeEligibility:n=cp(n,o.data);break}}),n}).addMatcher(_(...k),t=>Ce({},si)).addMatcher(_(..._e),(t,{payload:r})=>{const n=lT(r,Object.keys(si));return Ce(Ce({},t),n)});}}),np=(e,t)=>{const r=[...e.messages],n=r.findIndex(o=>o.id===t.id);if(n===-1)return Gn(Ce({},e),{messages:t.active?[...e.messages,t]:[...e.messages]});{const o=r[n],a=new Date(o.updated_at);new Date(t.updated_at)>a&&(t.active?r.splice(n,1,t):r.splice(n,1));}return Gn(Ce({},e),{messages:r})},op=(e,t)=>kt(e.appetite,t.appetite)?Gn(Ce({},e),{appetite:t.appetite}):e,ap=(e,t)=>{const r=new Date(e.updated_at);return new Date(t.updated_at)>r?Gn(Ce({},e),{appetite:t.appetite,buckets:t.buckets,cloned:t.cloned,completed_at:t.completed_at,current_bucket_id:t.current_bucket_id,deleted_at:t.deleted_at,features:t.features,finished_quote_requests_count:t.finished_quote_requests_count,locked:t.locked,quote_requests_count:t.quote_requests_count,state:t.state,updated_at:t.updated_at,taken_over:t.taken_over,sent_to_wholesaler_at:t.sent_to_wholesaler_at,source:t.source,source_metadata:t.source_metadata,percent_completed:t.percent_completed,real_time_eligibility:t.real_time_eligibility,wholesaler_preferred_quotes:t.wholesaler_preferred_quotes}):e},ip=(e,t)=>(t.percent_completed!==e.percent_completed&&(e.percent_completed=t.percent_completed),e),sp=(e,t)=>(kt(t,e.eligible_carriers_products)&&(e.eligible_carriers_products=t),e),cp=(e,t)=>(kt(t,e.real_time_eligibility)&&(e.real_time_eligibility=t),e),{handleMessage:zn,receiveAppetite:up,receiveApplicationForm:Ir,receiveApplicationFormProgress:ci,receiveCarrierProducts:ui,receiveRealTimeEligibility:lp}=rp.actions;var _D=Object.defineProperty,dp=Object.getOwnPropertySymbols,$D=Object.prototype.hasOwnProperty,wD=Object.prototype.propertyIsEnumerable,pp=(e,t,r)=>t in e?_D(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,fp=(e,t)=>{for(var r in t||(t={}))$D.call(t,r)&&pp(e,r,t[r]);if(dp)for(var r of dp(t))wD.call(t,r)&&pp(e,r,t[r]);return e};const AD=[],vp=E({name:"notifications",initialState:AD,reducers:{addNewNotification:(e,{payload:t})=>{if(!t)return e;e.push(fp({id:ml(8)},t));},removeNotification:(e,{payload:t})=>{if(!t)return e;const r=e.findIndex(n=>n.id===t.id);r>-1&&e.splice(r,1);}},extraReducers:e=>{e.addCase(ge,(t,{payload:r})=>(r.events.forEach(n=>{n.event===T.Notification&&t.push(fp({id:ml(8)},n.data));}),t));}}),{addNewNotification:hp,removeNotification:OD}=vp.actions,SD=[],gp=E({name:"pendingProcesses",initialState:SD,reducers:{receivedApplicationFormPendingProcess:(e,{payload:t})=>{bp(e,t);}},extraReducers:e=>{e.addCase(Ir,(t,{payload:r})=>yp(t,r)).addCase(ge,(t,{payload:r})=>{let n=[...t];return r.events.forEach(o=>{switch(o.event){case T.ApplicationFormProcess:n=bp(n,o.data);break;case T.ApplicationForm:n=yp(n,o.data);break}}),n}),e.addMatcher(_(..._e),(t,{payload:r})=>{var n;return (n=r.pending_processes)!=null?n:[]});}}),{receivedApplicationFormPendingProcess:mp}=gp.actions,bp=(e,t)=>{if(!t)return [];const r=e.findIndex(n=>n.id===t.id);return r>=0?e[r]=t:e.push(t),e},yp=(e,t)=>{const{pending_processes:r=[]}=t;return r.length?e:[]};var _p=typeof global=="object"&&global&&global.Object===Object&&global,jD=typeof self=="object"&&self&&self.Object===Object&&self,$e=_p||jD||Function("return this")(),Ft=$e.Symbol,$p=Object.prototype,CD=$p.hasOwnProperty,PD=$p.toString,Er=Ft?Ft.toStringTag:void 0;function ID(e){var t=CD.call(e,Er),r=e[Er];try{e[Er]=void 0;var n=!0;}catch(a){}var o=PD.call(e);return n&&(t?e[Er]=r:delete e[Er]),o}var ED=Object.prototype,TD=ED.toString;function qD(e){return TD.call(e)}var xD="[object Null]",kD="[object Undefined]",wp=Ft?Ft.toStringTag:void 0;function Tr(e){return e==null?e===void 0?kD:xD:wp&&wp in Object(e)?ID(e):qD(e)}function qr(e){return e!=null&&typeof e=="object"}var li=Array.isArray;function xr(e){var t=typeof e;return e!=null&&(t=="object"||t=="function")}var DD="[object AsyncFunction]",MD="[object Function]",ND="[object GeneratorFunction]",LD="[object Proxy]";function Ap(e){if(!xr(e))return !1;var t=Tr(e);return t==MD||t==ND||t==DD||t==LD}var di=$e["__core-js_shared__"],Op=function(){var e=/[^.]+$/.exec(di&&di.keys&&di.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}();function RD(e){return !!Op&&Op in e}var FD=Function.prototype,QD=FD.toString;function at(e){if(e!=null){try{return QD.call(e)}catch(t){}try{return e+""}catch(t){}}return ""}var UD=/[\\^$.*+?()[\]{}|]/g,VD=/^\[object .+?Constructor\]$/,BD=Function.prototype,GD=Object.prototype,zD=BD.toString,WD=GD.hasOwnProperty,HD=RegExp("^"+zD.call(WD).replace(UD,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function ZD(e){if(!xr(e)||RD(e))return !1;var t=Ap(e)?HD:VD;return t.test(at(e))}function KD(e,t){return e==null?void 0:e[t]}function it(e,t){var r=KD(e,t);return ZD(r)?r:void 0}var pi=it($e,"WeakMap"),Sp=Object.create,YD=function(){function e(){}return function(t){if(!xr(t))return {};if(Sp)return Sp(t);e.prototype=t;var r=new e;return e.prototype=void 0,r}}();function JD(e,t){var r=-1,n=e.length;for(t||(t=Array(n));++r<n;)t[r]=e[r];return t}var jp=function(){try{var e=it(Object,"defineProperty");return e({},"",{}),e}catch(t){}}();function XD(e,t){for(var r=-1,n=e==null?0:e.length;++r<n&&t(e[r],r,e)!==!1;);return e}var eM=9007199254740991,tM=/^(?:0|[1-9]\d*)$/;function rM(e,t){var r=typeof e;return t=t==null?eM:t,!!t&&(r=="number"||r!="symbol"&&tM.test(e))&&e>-1&&e%1==0&&e<t}function Cp(e,t,r){t=="__proto__"&&jp?jp(e,t,{configurable:!0,enumerable:!0,value:r,writable:!0}):e[t]=r;}function Pp(e,t){return e===t||e!==e&&t!==t}var nM=Object.prototype,oM=nM.hasOwnProperty;function Ip(e,t,r){var n=e[t];(!(oM.call(e,t)&&Pp(n,r))||r===void 0&&!(t in e))&&Cp(e,t,r);}function Wn(e,t,r,n){var o=!r;r||(r={});for(var a=-1,i=t.length;++a<i;){var s=t[a],c=n?n(r[s],e[s],s,r,e):void 0;c===void 0&&(c=e[s]),o?Cp(r,s,c):Ip(r,s,c);}return r}var aM=9007199254740991;function Ep(e){return typeof e=="number"&&e>-1&&e%1==0&&e<=aM}function Tp(e){return e!=null&&Ep(e.length)&&!Ap(e)}var iM=Object.prototype;function fi(e){var t=e&&e.constructor,r=typeof t=="function"&&t.prototype||iM;return e===r}function sM(e,t){for(var r=-1,n=Array(e);++r<e;)n[r]=t(r);return n}var cM="[object Arguments]";function qp(e){return qr(e)&&Tr(e)==cM}var xp=Object.prototype,uM=xp.hasOwnProperty,lM=xp.propertyIsEnumerable,dM=qp(function(){return arguments}())?qp:function(e){return qr(e)&&uM.call(e,"callee")&&!lM.call(e,"callee")};function pM(){return !1}var kp=typeof exports=="object"&&exports&&!exports.nodeType&&exports,Dp=kp&&typeof module=="object"&&module&&!module.nodeType&&module,fM=Dp&&Dp.exports===kp,Mp=fM?$e.Buffer:void 0,vM=Mp?Mp.isBuffer:void 0,Np=vM||pM,hM="[object Arguments]",gM="[object Array]",mM="[object Boolean]",bM="[object Date]",yM="[object Error]",_M="[object Function]",$M="[object Map]",wM="[object Number]",AM="[object Object]",OM="[object RegExp]",SM="[object Set]",jM="[object String]",CM="[object WeakMap]",PM="[object ArrayBuffer]",IM="[object DataView]",EM="[object Float32Array]",TM="[object Float64Array]",qM="[object Int8Array]",xM="[object Int16Array]",kM="[object Int32Array]",DM="[object Uint8Array]",MM="[object Uint8ClampedArray]",NM="[object Uint16Array]",LM="[object Uint32Array]",B={};B[EM]=B[TM]=B[qM]=B[xM]=B[kM]=B[DM]=B[MM]=B[NM]=B[LM]=!0,B[hM]=B[gM]=B[PM]=B[mM]=B[IM]=B[bM]=B[yM]=B[_M]=B[$M]=B[wM]=B[AM]=B[OM]=B[SM]=B[jM]=B[CM]=!1;function RM(e){return qr(e)&&Ep(e.length)&&!!B[Tr(e)]}function vi(e){return function(t){return e(t)}}var Lp=typeof exports=="object"&&exports&&!exports.nodeType&&exports,kr=Lp&&typeof module=="object"&&module&&!module.nodeType&&module,FM=kr&&kr.exports===Lp,hi=FM&&_p.process,Qt=function(){try{var e=kr&&kr.require&&kr.require("util").types;return e||hi&&hi.binding&&hi.binding("util")}catch(t){}}(),Rp=Qt&&Qt.isTypedArray,QM=Rp?vi(Rp):RM,UM=Object.prototype,VM=UM.hasOwnProperty;function Fp(e,t){var r=li(e),n=!r&&dM(e),o=!r&&!n&&Np(e),a=!r&&!n&&!o&&QM(e),i=r||n||o||a,s=i?sM(e.length,String):[],c=s.length;for(var u in e)(t||VM.call(e,u))&&!(i&&(u=="length"||o&&(u=="offset"||u=="parent")||a&&(u=="buffer"||u=="byteLength"||u=="byteOffset")||rM(u,c)))&&s.push(u);return s}function Qp(e,t){return function(r){return e(t(r))}}var BM=Qp(Object.keys,Object),GM=Object.prototype,zM=GM.hasOwnProperty;function WM(e){if(!fi(e))return BM(e);var t=[];for(var r in Object(e))zM.call(e,r)&&r!="constructor"&&t.push(r);return t}function gi(e){return Tp(e)?Fp(e):WM(e)}function HM(e){var t=[];if(e!=null)for(var r in Object(e))t.push(r);return t}var ZM=Object.prototype,KM=ZM.hasOwnProperty;function YM(e){if(!xr(e))return HM(e);var t=fi(e),r=[];for(var n in e)n=="constructor"&&(t||!KM.call(e,n))||r.push(n);return r}function mi(e){return Tp(e)?Fp(e,!0):YM(e)}var Dr=it(Object,"create");function JM(){this.__data__=Dr?Dr(null):{},this.size=0;}function XM(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var eN="__lodash_hash_undefined__",tN=Object.prototype,rN=tN.hasOwnProperty;function nN(e){var t=this.__data__;if(Dr){var r=t[e];return r===eN?void 0:r}return rN.call(t,e)?t[e]:void 0}var oN=Object.prototype,aN=oN.hasOwnProperty;function iN(e){var t=this.__data__;return Dr?t[e]!==void 0:aN.call(t,e)}var sN="__lodash_hash_undefined__";function cN(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=Dr&&t===void 0?sN:t,this}function st(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}st.prototype.clear=JM,st.prototype.delete=XM,st.prototype.get=nN,st.prototype.has=iN,st.prototype.set=cN;function uN(){this.__data__=[],this.size=0;}function Hn(e,t){for(var r=e.length;r--;)if(Pp(e[r][0],t))return r;return -1}var lN=Array.prototype,dN=lN.splice;function pN(e){var t=this.__data__,r=Hn(t,e);if(r<0)return !1;var n=t.length-1;return r==n?t.pop():dN.call(t,r,1),--this.size,!0}function fN(e){var t=this.__data__,r=Hn(t,e);return r<0?void 0:t[r][1]}function vN(e){return Hn(this.__data__,e)>-1}function hN(e,t){var r=this.__data__,n=Hn(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this}function Pe(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Pe.prototype.clear=uN,Pe.prototype.delete=pN,Pe.prototype.get=fN,Pe.prototype.has=vN,Pe.prototype.set=hN;var Mr=it($e,"Map");function gN(){this.size=0,this.__data__={hash:new st,map:new(Mr||Pe),string:new st};}function mN(e){var t=typeof e;return t=="string"||t=="number"||t=="symbol"||t=="boolean"?e!=="__proto__":e===null}function Zn(e,t){var r=e.__data__;return mN(t)?r[typeof t=="string"?"string":"hash"]:r.map}function bN(e){var t=Zn(this,e).delete(e);return this.size-=t?1:0,t}function yN(e){return Zn(this,e).get(e)}function _N(e){return Zn(this,e).has(e)}function $N(e,t){var r=Zn(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this}function Ut(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Ut.prototype.clear=gN,Ut.prototype.delete=bN,Ut.prototype.get=yN,Ut.prototype.has=_N,Ut.prototype.set=$N;function Up(e,t){for(var r=-1,n=t.length,o=e.length;++r<n;)e[o+r]=t[r];return e}var Vp=Qp(Object.getPrototypeOf,Object);function wN(){this.__data__=new Pe,this.size=0;}function AN(e){var t=this.__data__,r=t.delete(e);return this.size=t.size,r}function ON(e){return this.__data__.get(e)}function SN(e){return this.__data__.has(e)}var jN=200;function CN(e,t){var r=this.__data__;if(r instanceof Pe){var n=r.__data__;if(!Mr||n.length<jN-1)return n.push([e,t]),this.size=++r.size,this;r=this.__data__=new Ut(n);}return r.set(e,t),this.size=r.size,this}function Vt(e){var t=this.__data__=new Pe(e);this.size=t.size;}Vt.prototype.clear=wN,Vt.prototype.delete=AN,Vt.prototype.get=ON,Vt.prototype.has=SN,Vt.prototype.set=CN;function PN(e,t){return e&&Wn(t,gi(t),e)}function IN(e,t){return e&&Wn(t,mi(t),e)}var Bp=typeof exports=="object"&&exports&&!exports.nodeType&&exports,Gp=Bp&&typeof module=="object"&&module&&!module.nodeType&&module,EN=Gp&&Gp.exports===Bp,zp=EN?$e.Buffer:void 0,Wp=zp?zp.allocUnsafe:void 0;function TN(e,t){if(t)return e.slice();var r=e.length,n=Wp?Wp(r):new e.constructor(r);return e.copy(n),n}function qN(e,t){for(var r=-1,n=e==null?0:e.length,o=0,a=[];++r<n;){var i=e[r];t(i,r,e)&&(a[o++]=i);}return a}function Hp(){return []}var xN=Object.prototype,kN=xN.propertyIsEnumerable,Zp=Object.getOwnPropertySymbols,bi=Zp?function(e){return e==null?[]:(e=Object(e),qN(Zp(e),function(t){return kN.call(e,t)}))}:Hp;function DN(e,t){return Wn(e,bi(e),t)}var MN=Object.getOwnPropertySymbols,Kp=MN?function(e){for(var t=[];e;)Up(t,bi(e)),e=Vp(e);return t}:Hp;function NN(e,t){return Wn(e,Kp(e),t)}function Yp(e,t,r){var n=t(e);return li(e)?n:Up(n,r(e))}function LN(e){return Yp(e,gi,bi)}function RN(e){return Yp(e,mi,Kp)}var yi=it($e,"DataView"),_i=it($e,"Promise"),$i=it($e,"Set"),Jp="[object Map]",FN="[object Object]",Xp="[object Promise]",ef="[object Set]",tf="[object WeakMap]",rf="[object DataView]",QN=at(yi),UN=at(Mr),VN=at(_i),BN=at($i),GN=at(pi),ct=Tr;(yi&&ct(new yi(new ArrayBuffer(1)))!=rf||Mr&&ct(new Mr)!=Jp||_i&&ct(_i.resolve())!=Xp||$i&&ct(new $i)!=ef||pi&&ct(new pi)!=tf)&&(ct=function(e){var t=Tr(e),r=t==FN?e.constructor:void 0,n=r?at(r):"";if(n)switch(n){case QN:return rf;case UN:return Jp;case VN:return Xp;case BN:return ef;case GN:return tf}return t});var wi=ct,zN=Object.prototype,WN=zN.hasOwnProperty;function HN(e){var t=e.length,r=new e.constructor(t);return t&&typeof e[0]=="string"&&WN.call(e,"index")&&(r.index=e.index,r.input=e.input),r}var nf=$e.Uint8Array;function Ai(e){var t=new e.constructor(e.byteLength);return new nf(t).set(new nf(e)),t}function ZN(e,t){var r=t?Ai(e.buffer):e.buffer;return new e.constructor(r,e.byteOffset,e.byteLength)}var KN=/\w*$/;function YN(e){var t=new e.constructor(e.source,KN.exec(e));return t.lastIndex=e.lastIndex,t}var of=Ft?Ft.prototype:void 0,af=of?of.valueOf:void 0;function JN(e){return af?Object(af.call(e)):{}}function XN(e,t){var r=t?Ai(e.buffer):e.buffer;return new e.constructor(r,e.byteOffset,e.length)}var eL="[object Boolean]",tL="[object Date]",rL="[object Map]",nL="[object Number]",oL="[object RegExp]",aL="[object Set]",iL="[object String]",sL="[object Symbol]",cL="[object ArrayBuffer]",uL="[object DataView]",lL="[object Float32Array]",dL="[object Float64Array]",pL="[object Int8Array]",fL="[object Int16Array]",vL="[object Int32Array]",hL="[object Uint8Array]",gL="[object Uint8ClampedArray]",mL="[object Uint16Array]",bL="[object Uint32Array]";function yL(e,t,r){var n=e.constructor;switch(t){case cL:return Ai(e);case eL:case tL:return new n(+e);case uL:return ZN(e,r);case lL:case dL:case pL:case fL:case vL:case hL:case gL:case mL:case bL:return XN(e,r);case rL:return new n;case nL:case iL:return new n(e);case oL:return YN(e);case aL:return new n;case sL:return JN(e)}}function _L(e){return typeof e.constructor=="function"&&!fi(e)?YD(Vp(e)):{}}var $L="[object Map]";function wL(e){return qr(e)&&wi(e)==$L}var sf=Qt&&Qt.isMap,AL=sf?vi(sf):wL,OL="[object Set]";function SL(e){return qr(e)&&wi(e)==OL}var cf=Qt&&Qt.isSet,jL=cf?vi(cf):SL,CL=1,PL=2,IL=4,uf="[object Arguments]",EL="[object Array]",TL="[object Boolean]",qL="[object Date]",xL="[object Error]",lf="[object Function]",kL="[object GeneratorFunction]",DL="[object Map]",ML="[object Number]",df="[object Object]",NL="[object RegExp]",LL="[object Set]",RL="[object String]",FL="[object Symbol]",QL="[object WeakMap]",UL="[object ArrayBuffer]",VL="[object DataView]",BL="[object Float32Array]",GL="[object Float64Array]",zL="[object Int8Array]",WL="[object Int16Array]",HL="[object Int32Array]",ZL="[object Uint8Array]",KL="[object Uint8ClampedArray]",YL="[object Uint16Array]",JL="[object Uint32Array]",R={};R[uf]=R[EL]=R[UL]=R[VL]=R[TL]=R[qL]=R[BL]=R[GL]=R[zL]=R[WL]=R[HL]=R[DL]=R[ML]=R[df]=R[NL]=R[LL]=R[RL]=R[FL]=R[ZL]=R[KL]=R[YL]=R[JL]=!0,R[xL]=R[lf]=R[QL]=!1;function Kn(e,t,r,n,o,a){var i,s=t&CL,c=t&PL,u=t&IL;if(r&&(i=o?r(e,n,o,a):r(e)),i!==void 0)return i;if(!xr(e))return e;var l=li(e);if(l){if(i=HN(e),!s)return JD(e,i)}else {var d=wi(e),p=d==lf||d==kL;if(Np(e))return TN(e,s);if(d==df||d==uf||p&&!o){if(i=c||p?{}:_L(e),!s)return c?NN(e,IN(i,e)):DN(e,PN(i,e))}else {if(!R[d])return o?e:{};i=yL(e,d,s);}}a||(a=new Vt);var h=a.get(e);if(h)return h;a.set(e,i),jL(e)?e.forEach(function(y){i.add(Kn(y,t,r,y,e,a));}):AL(e)&&e.forEach(function(y,m){i.set(m,Kn(y,t,r,m,e,a));});var v=u?c?RN:LN:c?mi:gi,g=l?void 0:v(e);return XD(g||e,function(y,m){g&&(m=y,y=e[m]),Ip(i,m,Kn(y,t,r,m,e,a));}),i}var XL=1,eR=4;function pf(e){return Kn(e,XL|eR)}const tR=(e,t)=>(e.parentQuestionSets=iR(t,e.parentQuestionSets),e.list=oR(e.list,t),e),rR=e=>e.filter(t=>ff(t)),ff=e=>{var t;return (t=e==null?void 0:e.question_set.question_groups)==null?void 0:t.some(r=>Ml(r))},nR=(e,t)=>{const r=e;return t.forEach(n=>{const o=gf(n),a=_f(e,n.order);r.splice(a,1,...o);}),r},vf=e=>!!e.parentQuestionSetId,hf=(e,t,r)=>{const n=r.list.filter(o=>o.parentQuestionSetId===e.id).filter(o=>!o.is_blank);return n.length&&n.findIndex(o=>o.question_set_id===t.question_set_id)===n.length-1?void 0:t};function oR(e,t){const r=e.filter(a=>a.parentQuestionSetId!==t.id),n=gf(t),o=_f(r,t.order);return r.splice(o,0,...n),r}function gf(e){const t=Yq(e,Zq(e));return Array.from(t.values()).filter(r=>Ml(r)).flatMap(r=>Array.from(r.nestedGroupsWithAnswers.values()).filter(n=>tx(n)).sort((n,o)=>rx(n,o)).map(n=>aR(n,e)))}function aR(e,t){const r=t.question_set.question_groups.find(a=>a.id===e.groupId),n=mf(e),o={answers:n,application_form_id:t.application_form_id,completed_at:null,created_at:t.created_at,id:e.groupId,is_active:t.is_active,is_blank:n.length===0,next_active_question_set_id:t.next_active_question_set_id,order:t.order,question_set:{code:e.parentQuestionAnswer.question.code,created_at:t.question_set.created_at,id:e.parentQuestionAnswer.id,name:e.label,owner_id:t.question_set.owner_id,question_groups:bf(t.question_set.question_groups,r),question_ids:yf(t.question_set.question_groups,r),question_pools:t.question_set.question_pools,subdomain:t.question_set.subdomain,tenant_id:t.question_set.tenant_id,title:e.label,updated_at:t.updated_at},question_set_id:e.parentQuestionAnswer.id,updated_at:t.updated_at,parentQuestionSetId:t.id};return o.is_completed=Za(o),o}function mf(e){let t=e.answers;return e.nestedGroupsWithAnswers.forEach(r=>{t=[...t,...mf(r)];}),t.sort(Ba)}function bf(e,t){if(!t)return [];let r=[];return e.forEach(n=>{t.question_ids.includes(n.parent_question_id)&&(r.push(n),r=[...r,...bf(e,n)]);}),r}function yf(e,t){if(!t)return [];let r=t==null?void 0:t.question_ids;return e.forEach(n=>{t.question_ids.includes(n.parent_question_id)&&(r=[...r,...yf(e,n)]);}),r}function _f(e,t){let r=e.findIndex(n=>n.order>=t);return r===-1&&(r=e.length),r}function iR(e,t){return [...t.filter(r=>r.question_set_id!==e.question_set_id),e]}const Yn=(e,t,r)=>{if(r){const n=e.list.findIndex(o=>o.question_set_id===r.question_set_id);return t==="backward"?sR(e.list,n):Of(e.list,n)}return Bt(e.list)},$f=(e,t,r)=>t?Yn(e,t,r||void 0):Bt(e.list),wf=(e,t,r,n)=>{const o=t.next_active_question_set_id;return o&&!n?Jn(e,o):r?Yn(e,r,n||t):Bt(e.list)},Bt=e=>{const t=e.filter(n=>Nr(n));if(t.length===0)return "";const r=t.find(n=>!n.is_completed);if(r)return r.question_set_id;{const n=e.filter(Nr);return n[n.length-1].question_set_id}};function Jn(e,t){var r;const n=e.list.findIndex(o=>o.id===t);if(n!==-1)return e.list[n].is_blank?Of(e.list,n):e.list[n].question_set_id;if(e.parentQuestionSets.find(o=>o.id===t))return (r=e.list.filter(Nr).find(o=>o.parentQuestionSetId===t))==null?void 0:r.question_set_id}function Af(e,t){const r=e.list.find(n=>n.question_set_id===e.active);return r&&!r.next_active_question_set_id&&r.is_completed&&r.order+1===t.order?Jn(e,t.id):e.active}function Nr(e){return e.is_active&&!e.is_blank&&e.answers.length>0}function sR(e,t){if(t<=0)return e[0].question_set_id;let r;for(let n=t-1;n>=0;n--)if(Nr(e[n])){r=e[n].question_set_id;break}return r!=null?r:e[t].question_set_id}function Of(e,t){if(t>=e.length-1)return e[e.length-1].question_set_id;let r;for(let n=t+1;n<e.length;n++)if(Nr(e[n])){r=e[n].question_set_id;break}return r!=null?r:e[t].question_set_id}var Xn=typeof globalThis!="undefined"?globalThis:typeof window!="undefined"?window:typeof global!="undefined"?global:typeof self!="undefined"?self:{};function cR(e){return e&&e.__esModule&&Object.prototype.hasOwnProperty.call(e,"default")?e.default:e}function uR(){this.__data__=[],this.size=0;}var lR=uR;function dR(e,t){return e===t||e!==e&&t!==t}var Sf=dR,pR=Sf;function fR(e,t){for(var r=e.length;r--;)if(pR(e[r][0],t))return r;return -1}var eo=fR,vR=eo,hR=Array.prototype,gR=hR.splice;function mR(e){var t=this.__data__,r=vR(t,e);if(r<0)return !1;var n=t.length-1;return r==n?t.pop():gR.call(t,r,1),--this.size,!0}var bR=mR,yR=eo;function _R(e){var t=this.__data__,r=yR(t,e);return r<0?void 0:t[r][1]}var $R=_R,wR=eo;function AR(e){return wR(this.__data__,e)>-1}var OR=AR,SR=eo;function jR(e,t){var r=this.__data__,n=SR(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this}var CR=jR,PR=lR,IR=bR,ER=$R,TR=OR,qR=CR;function Gt(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Gt.prototype.clear=PR,Gt.prototype.delete=IR,Gt.prototype.get=ER,Gt.prototype.has=TR,Gt.prototype.set=qR;var to=Gt,xR=to;function kR(){this.__data__=new xR,this.size=0;}var DR=kR;function MR(e){var t=this.__data__,r=t.delete(e);return this.size=t.size,r}var NR=MR;function LR(e){return this.__data__.get(e)}var RR=LR;function FR(e){return this.__data__.has(e)}var QR=FR,UR=typeof Xn=="object"&&Xn&&Xn.Object===Object&&Xn,jf=UR,VR=jf,BR=typeof self=="object"&&self&&self.Object===Object&&self,GR=VR||BR||Function("return this")(),we=GR,zR=we,WR=zR.Symbol,Oi=WR,Cf=Oi,Pf=Object.prototype,HR=Pf.hasOwnProperty,ZR=Pf.toString,Lr=Cf?Cf.toStringTag:void 0;function KR(e){var t=HR.call(e,Lr),r=e[Lr];try{e[Lr]=void 0;var n=!0;}catch(a){}var o=ZR.call(e);return n&&(t?e[Lr]=r:delete e[Lr]),o}var YR=KR,JR=Object.prototype,XR=JR.toString;function eF(e){return XR.call(e)}var tF=eF,If=Oi,rF=YR,nF=tF,oF="[object Null]",aF="[object Undefined]",Ef=If?If.toStringTag:void 0;function iF(e){return e==null?e===void 0?aF:oF:Ef&&Ef in Object(e)?rF(e):nF(e)}var ro=iF;function sF(e){var t=typeof e;return e!=null&&(t=="object"||t=="function")}var Rr=sF,cF=ro,uF=Rr,lF="[object AsyncFunction]",dF="[object Function]",pF="[object GeneratorFunction]",fF="[object Proxy]";function vF(e){if(!uF(e))return !1;var t=cF(e);return t==dF||t==pF||t==lF||t==fF}var Tf=vF,hF=we,gF=hF["__core-js_shared__"],mF=gF,Si=mF,qf=function(){var e=/[^.]+$/.exec(Si&&Si.keys&&Si.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}();function bF(e){return !!qf&&qf in e}var yF=bF,_F=Function.prototype,$F=_F.toString;function wF(e){if(e!=null){try{return $F.call(e)}catch(t){}try{return e+""}catch(t){}}return ""}var xf=wF,AF=Tf,OF=yF,SF=Rr,jF=xf,CF=/[\\^$.*+?()[\]{}|]/g,PF=/^\[object .+?Constructor\]$/,IF=Function.prototype,EF=Object.prototype,TF=IF.toString,qF=EF.hasOwnProperty,xF=RegExp("^"+TF.call(qF).replace(CF,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function kF(e){if(!SF(e)||OF(e))return !1;var t=AF(e)?xF:PF;return t.test(jF(e))}var DF=kF;function MF(e,t){return e==null?void 0:e[t]}var NF=MF,LF=DF,RF=NF;function FF(e,t){var r=RF(e,t);return LF(r)?r:void 0}var ut=FF,QF=ut,UF=we,VF=QF(UF,"Map"),ji=VF,BF=ut,GF=BF(Object,"create"),no=GF,kf=no;function zF(){this.__data__=kf?kf(null):{},this.size=0;}var WF=zF;function HF(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var ZF=HF,KF=no,YF="__lodash_hash_undefined__",JF=Object.prototype,XF=JF.hasOwnProperty;function eQ(e){var t=this.__data__;if(KF){var r=t[e];return r===YF?void 0:r}return XF.call(t,e)?t[e]:void 0}var tQ=eQ,rQ=no,nQ=Object.prototype,oQ=nQ.hasOwnProperty;function aQ(e){var t=this.__data__;return rQ?t[e]!==void 0:oQ.call(t,e)}var iQ=aQ,sQ=no,cQ="__lodash_hash_undefined__";function uQ(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=sQ&&t===void 0?cQ:t,this}var lQ=uQ,dQ=WF,pQ=ZF,fQ=tQ,vQ=iQ,hQ=lQ;function zt(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}zt.prototype.clear=dQ,zt.prototype.delete=pQ,zt.prototype.get=fQ,zt.prototype.has=vQ,zt.prototype.set=hQ;var gQ=zt,Df=gQ,mQ=to,bQ=ji;function yQ(){this.size=0,this.__data__={hash:new Df,map:new(bQ||mQ),string:new Df};}var _Q=yQ;function $Q(e){var t=typeof e;return t=="string"||t=="number"||t=="symbol"||t=="boolean"?e!=="__proto__":e===null}var wQ=$Q,AQ=wQ;function OQ(e,t){var r=e.__data__;return AQ(t)?r[typeof t=="string"?"string":"hash"]:r.map}var oo=OQ,SQ=oo;function jQ(e){var t=SQ(this,e).delete(e);return this.size-=t?1:0,t}var CQ=jQ,PQ=oo;function IQ(e){return PQ(this,e).get(e)}var EQ=IQ,TQ=oo;function qQ(e){return TQ(this,e).has(e)}var xQ=qQ,kQ=oo;function DQ(e,t){var r=kQ(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this}var MQ=DQ,NQ=_Q,LQ=CQ,RQ=EQ,FQ=xQ,QQ=MQ;function Wt(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Wt.prototype.clear=NQ,Wt.prototype.delete=LQ,Wt.prototype.get=RQ,Wt.prototype.has=FQ,Wt.prototype.set=QQ;var UQ=Wt,VQ=to,BQ=ji,GQ=UQ,zQ=200;function WQ(e,t){var r=this.__data__;if(r instanceof VQ){var n=r.__data__;if(!BQ||n.length<zQ-1)return n.push([e,t]),this.size=++r.size,this;r=this.__data__=new GQ(n);}return r.set(e,t),this.size=r.size,this}var HQ=WQ,ZQ=to,KQ=DR,YQ=NR,JQ=RR,XQ=QR,e9=HQ;function Ht(e){var t=this.__data__=new ZQ(e);this.size=t.size;}Ht.prototype.clear=KQ,Ht.prototype.delete=YQ,Ht.prototype.get=JQ,Ht.prototype.has=XQ,Ht.prototype.set=e9;var t9=Ht;function r9(e,t){for(var r=-1,n=e==null?0:e.length;++r<n&&t(e[r],r,e)!==!1;);return e}var n9=r9,o9=ut,a9=function(){try{var e=o9(Object,"defineProperty");return e({},"",{}),e}catch(t){}}(),i9=a9,Mf=i9;function s9(e,t,r){t=="__proto__"&&Mf?Mf(e,t,{configurable:!0,enumerable:!0,value:r,writable:!0}):e[t]=r;}var Nf=s9,c9=Nf,u9=Sf,l9=Object.prototype,d9=l9.hasOwnProperty;function p9(e,t,r){var n=e[t];(!(d9.call(e,t)&&u9(n,r))||r===void 0&&!(t in e))&&c9(e,t,r);}var Lf=p9,f9=Lf,v9=Nf;function h9(e,t,r,n){var o=!r;r||(r={});for(var a=-1,i=t.length;++a<i;){var s=t[a],c=n?n(r[s],e[s],s,r,e):void 0;c===void 0&&(c=e[s]),o?v9(r,s,c):f9(r,s,c);}return r}var ao=h9;function g9(e,t){for(var r=-1,n=Array(e);++r<e;)n[r]=t(r);return n}var m9=g9;function b9(e){return e!=null&&typeof e=="object"}var Fr=b9,y9=ro,_9=Fr,$9="[object Arguments]";function w9(e){return _9(e)&&y9(e)==$9}var A9=w9,Rf=A9,O9=Fr,Ff=Object.prototype,S9=Ff.hasOwnProperty,j9=Ff.propertyIsEnumerable,C9=Rf(function(){return arguments}())?Rf:function(e){return O9(e)&&S9.call(e,"callee")&&!j9.call(e,"callee")},P9=C9,I9=Array.isArray,Ci=I9,io={exports:{}};function E9(){return !1}var T9=E9;(function(e,t){var r=we,n=T9,o=t&&!t.nodeType&&t,a=o&&!0&&e&&!e.nodeType&&e,i=a&&a.exports===o,s=i?r.Buffer:void 0,c=s?s.isBuffer:void 0,u=c||n;e.exports=u;})(io,io.exports);var Qf=io.exports,q9=9007199254740991,x9=/^(?:0|[1-9]\d*)$/;function k9(e,t){var r=typeof e;return t=t==null?q9:t,!!t&&(r=="number"||r!="symbol"&&x9.test(e))&&e>-1&&e%1==0&&e<t}var D9=k9,M9=9007199254740991;function N9(e){return typeof e=="number"&&e>-1&&e%1==0&&e<=M9}var Uf=N9,L9=ro,R9=Uf,F9=Fr,Q9="[object Arguments]",U9="[object Array]",V9="[object Boolean]",B9="[object Date]",G9="[object Error]",z9="[object Function]",W9="[object Map]",H9="[object Number]",Z9="[object Object]",K9="[object RegExp]",Y9="[object Set]",J9="[object String]",X9="[object WeakMap]",eU="[object ArrayBuffer]",tU="[object DataView]",rU="[object Float32Array]",nU="[object Float64Array]",oU="[object Int8Array]",aU="[object Int16Array]",iU="[object Int32Array]",sU="[object Uint8Array]",cU="[object Uint8ClampedArray]",uU="[object Uint16Array]",lU="[object Uint32Array]",G={};G[rU]=G[nU]=G[oU]=G[aU]=G[iU]=G[sU]=G[cU]=G[uU]=G[lU]=!0,G[Q9]=G[U9]=G[eU]=G[V9]=G[tU]=G[B9]=G[G9]=G[z9]=G[W9]=G[H9]=G[Z9]=G[K9]=G[Y9]=G[J9]=G[X9]=!1;function dU(e){return F9(e)&&R9(e.length)&&!!G[L9(e)]}var pU=dU;function fU(e){return function(t){return e(t)}}var Pi=fU,so={exports:{}};(function(e,t){var r=jf,n=t&&!t.nodeType&&t,o=n&&!0&&e&&!e.nodeType&&e,a=o&&o.exports===n,i=a&&r.process,s=function(){try{var c=o&&o.require&&o.require("util").types;return c||i&&i.binding&&i.binding("util")}catch(u){}}();e.exports=s;})(so,so.exports);var Ii=so.exports,vU=pU,hU=Pi,Vf=Ii,Bf=Vf&&Vf.isTypedArray,gU=Bf?hU(Bf):vU,mU=gU,bU=m9,yU=P9,_U=Ci,$U=Qf,wU=D9,AU=mU,OU=Object.prototype,SU=OU.hasOwnProperty;function jU(e,t){var r=_U(e),n=!r&&yU(e),o=!r&&!n&&$U(e),a=!r&&!n&&!o&&AU(e),i=r||n||o||a,s=i?bU(e.length,String):[],c=s.length;for(var u in e)(t||SU.call(e,u))&&!(i&&(u=="length"||o&&(u=="offset"||u=="parent")||a&&(u=="buffer"||u=="byteLength"||u=="byteOffset")||wU(u,c)))&&s.push(u);return s}var Gf=jU,CU=Object.prototype;function PU(e){var t=e&&e.constructor,r=typeof t=="function"&&t.prototype||CU;return e===r}var Ei=PU;function IU(e,t){return function(r){return e(t(r))}}var zf=IU,EU=zf,TU=EU(Object.keys,Object),qU=TU,xU=Ei,kU=qU,DU=Object.prototype,MU=DU.hasOwnProperty;function NU(e){if(!xU(e))return kU(e);var t=[];for(var r in Object(e))MU.call(e,r)&&r!="constructor"&&t.push(r);return t}var LU=NU,RU=Tf,FU=Uf;function QU(e){return e!=null&&FU(e.length)&&!RU(e)}var Wf=QU,UU=Gf,VU=LU,BU=Wf;function GU(e){return BU(e)?UU(e):VU(e)}var Ti=GU,zU=ao,WU=Ti;function HU(e,t){return e&&zU(t,WU(t),e)}var ZU=HU;function KU(e){var t=[];if(e!=null)for(var r in Object(e))t.push(r);return t}var YU=KU,JU=Rr,XU=Ei,eV=YU,tV=Object.prototype,rV=tV.hasOwnProperty;function nV(e){if(!JU(e))return eV(e);var t=XU(e),r=[];for(var n in e)n=="constructor"&&(t||!rV.call(e,n))||r.push(n);return r}var oV=nV,aV=Gf,iV=oV,sV=Wf;function cV(e){return sV(e)?aV(e,!0):iV(e)}var qi=cV,uV=ao,lV=qi;function dV(e,t){return e&&uV(t,lV(t),e)}var pV=dV,co={exports:{}};(function(e,t){var r=we,n=t&&!t.nodeType&&t,o=n&&!0&&e&&!e.nodeType&&e,a=o&&o.exports===n,i=a?r.Buffer:void 0,s=i?i.allocUnsafe:void 0;function c(u,l){if(l)return u.slice();var d=u.length,p=s?s(d):new u.constructor(d);return u.copy(p),p}e.exports=c;})(co,co.exports);var fV=co.exports;function vV(e,t){var r=-1,n=e.length;for(t||(t=Array(n));++r<n;)t[r]=e[r];return t}var hV=vV;function gV(e,t){for(var r=-1,n=e==null?0:e.length,o=0,a=[];++r<n;){var i=e[r];t(i,r,e)&&(a[o++]=i);}return a}var mV=gV;function bV(){return []}var Hf=bV,yV=mV,_V=Hf,$V=Object.prototype,wV=$V.propertyIsEnumerable,Zf=Object.getOwnPropertySymbols,AV=Zf?function(e){return e==null?[]:(e=Object(e),yV(Zf(e),function(t){return wV.call(e,t)}))}:_V,xi=AV,OV=ao,SV=xi;function jV(e,t){return OV(e,SV(e),t)}var CV=jV;function PV(e,t){for(var r=-1,n=t.length,o=e.length;++r<n;)e[o+r]=t[r];return e}var Kf=PV,IV=zf,EV=IV(Object.getPrototypeOf,Object),Yf=EV,TV=Kf,qV=Yf,xV=xi,kV=Hf,DV=Object.getOwnPropertySymbols,MV=DV?function(e){for(var t=[];e;)TV(t,xV(e)),e=qV(e);return t}:kV,Jf=MV,NV=ao,LV=Jf;function RV(e,t){return NV(e,LV(e),t)}var FV=RV,QV=Kf,UV=Ci;function VV(e,t,r){var n=t(e);return UV(e)?n:QV(n,r(e))}var Xf=VV,BV=Xf,GV=xi,zV=Ti;function WV(e){return BV(e,zV,GV)}var HV=WV,ZV=Xf,KV=Jf,YV=qi;function JV(e){return ZV(e,YV,KV)}var XV=JV,eB=ut,tB=we,rB=eB(tB,"DataView"),nB=rB,oB=ut,aB=we,iB=oB(aB,"Promise"),sB=iB,cB=ut,uB=we,lB=cB(uB,"Set"),dB=lB,pB=ut,fB=we,vB=pB(fB,"WeakMap"),hB=vB,ki=nB,Di=ji,Mi=sB,Ni=dB,Li=hB,ev=ro,Zt=xf,tv="[object Map]",gB="[object Object]",rv="[object Promise]",nv="[object Set]",ov="[object WeakMap]",av="[object DataView]",mB=Zt(ki),bB=Zt(Di),yB=Zt(Mi),_B=Zt(Ni),$B=Zt(Li),lt=ev;(ki&&lt(new ki(new ArrayBuffer(1)))!=av||Di&&lt(new Di)!=tv||Mi&&lt(Mi.resolve())!=rv||Ni&&lt(new Ni)!=nv||Li&&lt(new Li)!=ov)&&(lt=function(e){var t=ev(e),r=t==gB?e.constructor:void 0,n=r?Zt(r):"";if(n)switch(n){case mB:return av;case bB:return tv;case yB:return rv;case _B:return nv;case $B:return ov}return t});var Ri=lt,wB=Object.prototype,AB=wB.hasOwnProperty;function OB(e){var t=e.length,r=new e.constructor(t);return t&&typeof e[0]=="string"&&AB.call(e,"index")&&(r.index=e.index,r.input=e.input),r}var SB=OB,jB=we,CB=jB.Uint8Array,PB=CB,iv=PB;function IB(e){var t=new e.constructor(e.byteLength);return new iv(t).set(new iv(e)),t}var Fi=IB,EB=Fi;function TB(e,t){var r=t?EB(e.buffer):e.buffer;return new e.constructor(r,e.byteOffset,e.byteLength)}var qB=TB,xB=/\w*$/;function kB(e){var t=new e.constructor(e.source,xB.exec(e));return t.lastIndex=e.lastIndex,t}var DB=kB,sv=Oi,cv=sv?sv.prototype:void 0,uv=cv?cv.valueOf:void 0;function MB(e){return uv?Object(uv.call(e)):{}}var NB=MB,LB=Fi;function RB(e,t){var r=t?LB(e.buffer):e.buffer;return new e.constructor(r,e.byteOffset,e.length)}var FB=RB,QB=Fi,UB=qB,VB=DB,BB=NB,GB=FB,zB="[object Boolean]",WB="[object Date]",HB="[object Map]",ZB="[object Number]",KB="[object RegExp]",YB="[object Set]",JB="[object String]",XB="[object Symbol]",e3="[object ArrayBuffer]",t3="[object DataView]",r3="[object Float32Array]",n3="[object Float64Array]",o3="[object Int8Array]",a3="[object Int16Array]",i3="[object Int32Array]",s3="[object Uint8Array]",c3="[object Uint8ClampedArray]",u3="[object Uint16Array]",l3="[object Uint32Array]";function d3(e,t,r){var n=e.constructor;switch(t){case e3:return QB(e);case zB:case WB:return new n(+e);case t3:return UB(e,r);case r3:case n3:case o3:case a3:case i3:case s3:case c3:case u3:case l3:return GB(e,r);case HB:return new n;case ZB:case JB:return new n(e);case KB:return VB(e);case YB:return new n;case XB:return BB(e)}}var p3=d3,f3=Rr,lv=Object.create,v3=function(){function e(){}return function(t){if(!f3(t))return {};if(lv)return lv(t);e.prototype=t;var r=new e;return e.prototype=void 0,r}}(),h3=v3,g3=h3,m3=Yf,b3=Ei;function y3(e){return typeof e.constructor=="function"&&!b3(e)?g3(m3(e)):{}}var _3=y3,$3=Ri,w3=Fr,A3="[object Map]";function O3(e){return w3(e)&&$3(e)==A3}var S3=O3,j3=S3,C3=Pi,dv=Ii,pv=dv&&dv.isMap,P3=pv?C3(pv):j3,I3=P3,E3=Ri,T3=Fr,q3="[object Set]";function x3(e){return T3(e)&&E3(e)==q3}var k3=x3,D3=k3,M3=Pi,fv=Ii,vv=fv&&fv.isSet,N3=vv?M3(vv):D3,L3=N3,R3=t9,F3=n9,Q3=Lf,U3=ZU,V3=pV,B3=fV,G3=hV,z3=CV,W3=FV,H3=HV,Z3=XV,K3=Ri,Y3=SB,J3=p3,X3=_3,eG=Ci,tG=Qf,rG=I3,nG=Rr,oG=L3,aG=Ti,iG=qi,sG=1,cG=2,uG=4,hv="[object Arguments]",lG="[object Array]",dG="[object Boolean]",pG="[object Date]",fG="[object Error]",gv="[object Function]",vG="[object GeneratorFunction]",hG="[object Map]",gG="[object Number]",mv="[object Object]",mG="[object RegExp]",bG="[object Set]",yG="[object String]",_G="[object Symbol]",$G="[object WeakMap]",wG="[object ArrayBuffer]",AG="[object DataView]",OG="[object Float32Array]",SG="[object Float64Array]",jG="[object Int8Array]",CG="[object Int16Array]",PG="[object Int32Array]",IG="[object Uint8Array]",EG="[object Uint8ClampedArray]",TG="[object Uint16Array]",qG="[object Uint32Array]",F={};F[hv]=F[lG]=F[wG]=F[AG]=F[dG]=F[pG]=F[OG]=F[SG]=F[jG]=F[CG]=F[PG]=F[hG]=F[gG]=F[mv]=F[mG]=F[bG]=F[yG]=F[_G]=F[IG]=F[EG]=F[TG]=F[qG]=!0,F[fG]=F[gv]=F[$G]=!1;function uo(e,t,r,n,o,a){var i,s=t&sG,c=t&cG,u=t&uG;if(r&&(i=o?r(e,n,o,a):r(e)),i!==void 0)return i;if(!nG(e))return e;var l=eG(e);if(l){if(i=Y3(e),!s)return G3(e,i)}else {var d=K3(e),p=d==gv||d==vG;if(tG(e))return B3(e,s);if(d==mv||d==hv||p&&!o){if(i=c||p?{}:X3(e),!s)return c?W3(e,V3(i,e)):z3(e,U3(i,e))}else {if(!F[d])return o?e:{};i=J3(e,d,s);}}a||(a=new R3);var h=a.get(e);if(h)return h;a.set(e,i),oG(e)?e.forEach(function(y){i.add(uo(y,t,r,y,e,a));}):rG(e)&&e.forEach(function(y,m){i.set(m,uo(y,t,r,m,e,a));});var v=u?c?Z3:H3:c?iG:aG,g=l?void 0:v(e);return F3(g||e,function(y,m){g&&(m=y,y=e[m]),Q3(i,m,uo(y,t,r,m,e,a));}),i}var xG=uo,kG=xG,DG=1,MG=4;function NG(e){return kG(e,DG|MG)}var LG=NG,RG=cR(LG),FG=Object.defineProperty,QG=Object.defineProperties,UG=Object.getOwnPropertyDescriptors,bv=Object.getOwnPropertySymbols,VG=Object.prototype.hasOwnProperty,BG=Object.prototype.propertyIsEnumerable,yv=(e,t,r)=>t in e?FG(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,GG=(e,t)=>{for(var r in t||(t={}))VG.call(t,r)&&yv(e,r,t[r]);if(bv)for(var r of bv(t))BG.call(t,r)&&yv(e,r,t[r]);return e},zG=(e,t)=>QG(e,UG(t)),WG=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});function _v(e){return zG(GG({},RG(e)),{updated_at:new Date().toISOString()})}const X=L("answers/Update",(e,t)=>WG(void 0,[e,t],function*(r,{getState:n,rejectWithValue:o,dispatch:a}){try{const i=n(),s=Ha(r.answer,Rl(r.answer));return i.queues.answers.outbound[r.answer.id]?s.length?_v(r.answer):(yield xy.updateAnswer(r.answer,r.config)).parsedBody:(a(on(r.answer)),_v(r.answer))}catch(i){const s=new M("Error updating answer",i);throw V(s),o(s)}}));var HG=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const Kt=L("quoteableCarriersProduct/Load",(e,t)=>HG(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield Dy.getApplicationFormCarriersProducts(r.appFormId,r.config)).parsedBody}catch(o){const a=new M("Error getting carrier products",o);throw V(a),n(a),a}}));var $v=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const Qr=L("choices/Search",(e,t)=>$v(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield cc.searchChoices(r.questionId,{filter:r.filter},r.config)).parsedBody}catch(o){const a=new M("Error searching choices",o);throw V(a),n(a),a}})),Ie=L("choicesWithNudges/Search",(e,t)=>$v(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield cc.searchChoicesWithNudges(r.questionId,{filter:r.filter,refilter:r.refilter},r.config)).parsedBody}catch(o){const a=new M("Error searching choices",o);throw V(a),n(a),a}}));var ZG=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const wv=L("features/Search",(e,t)=>ZG(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield ky.getFeatures(r.config)).parsedBody}catch(o){const a=new M("Error getting features",o);throw V(a),n(a),a}}));var KG=Object.defineProperty,YG=Object.defineProperties,JG=Object.getOwnPropertyDescriptors,Av=Object.getOwnPropertySymbols,XG=Object.prototype.hasOwnProperty,ez=Object.prototype.propertyIsEnumerable,Ov=(e,t,r)=>t in e?KG(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,tz=(e,t)=>{for(var r in t||(t={}))XG.call(t,r)&&Ov(e,r,t[r]);if(Av)for(var r of Av(t))ez.call(t,r)&&Ov(e,r,t[r]);return e},rz=(e,t)=>YG(e,JG(t)),Sv=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const jv="questionPools",lo=L(`${jv}/Add`,(e,t)=>Sv(void 0,[e,t],function*(r,{getState:n,rejectWithValue:o}){var a,i;try{const s=n();if(r.parent_question_pool_id){const c=(a=s.application.questionSets.list.filter(u=>u.question_set.question_pools))==null?void 0:a.find(u=>u.question_set.question_pools.findIndex(l=>l.id===r.parent_question_pool_id)!==-1);c&&(r=rz(tz({},r),{parent_answer_pool_id:(i=c.answers.find(u=>u.answer_pool_id===r.selected_answer_pool_id))==null?void 0:i.answer_pool_id}));}return (yield uc.addQuestionPool(r)).parsedBody}catch(s){const c=new M("Error adding question pool",s);return V(c),o(c)}})),po=L(`${jv}/Remove`,(e,t)=>Sv(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const o=yield uc.removeQuestionPool(r);return r.callback&&r.callback(),o.parsedBody}catch(o){const a=new M("Error adding question pool",o);return V(a),n(a)}}));var nz=Object.defineProperty,oz=Object.defineProperties,az=Object.getOwnPropertyDescriptors,Cv=Object.getOwnPropertySymbols,iz=Object.prototype.hasOwnProperty,sz=Object.prototype.propertyIsEnumerable,Pv=(e,t,r)=>t in e?nz(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Iv=(e,t)=>{for(var r in t||(t={}))iz.call(t,r)&&Pv(e,r,t[r]);if(Cv)for(var r of Cv(t))sz.call(t,r)&&Pv(e,r,t[r]);return e},Ev=(e,t)=>oz(e,az(t)),Qi=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const Ui="questionSets",Ur=L(`${Ui}/LoadOne`,(e,t)=>Qi(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield Xo.getQuestionSet(r.appForm,{id:r.id},r.config)).parsedBody}catch(o){const a=new M("Error loading single question set",o);throw V(a),n(a),a}})),cz=L(`${Ui}/Create`,(e,t)=>Qi(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield Xo.createQuestionSet(r.appFormQuestionSet,r.config)).parsedBody}catch(o){const a=new M("Error loading single question set",o);throw V(a),n(a),a}})),Vr=L(`${Ui}/Save`,(e,t)=>Qi(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const o=r.parentQuestionSet?Ev(Iv({},r.parentQuestionSet),{is_completed:!0}):Ev(Iv({},r.appFormQuestionSet),{is_completed:!0});return (yield Xo.updateQuestionSet(o,r.config)).parsedBody}catch(o){const a=new M("Error saving question set",o);return V(a),n(a)}}));var Tv=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const Br=L("bindQuote/Bind",(e,t)=>Tv(void 0,[e,t],function*(r,{rejectWithValue:n}){var o;try{const a=yield lc.bindQuote(r.quoteId,r.appFormId,r.config);return r.afterResponse&&r.afterResponse(((o=a.parsedBody)==null?void 0:o.retrieve_url)||""),a.parsedBody}catch(a){const i=new M("Error binding quote request",a);throw V(i),n(i),i}})),fo=L("bindQuote/action",(e,t)=>Tv(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const{action:o,quote:a,config:i}=r;return yield lc.bindQuoteAction(o,a,i)}catch(o){const a=new M("Error binding quote request",o);throw n(a),a}}));var dt=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const Yt="vehicleOptions",Ee=L(`${Yt}/LoadFull`,(e,t)=>dt(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const o=me.getYears(r.config),a=me.getMakes(r.year,r.config,{type:r.type}),i=me.getModels(r.year,r.makeId,r.config,r.type),s=me.getConfigurations(r.year,r.makeId,r.modelId,r.config,r.type),{parsedBody:c}=yield o,{parsedBody:u}=yield a,{parsedBody:l}=yield i,{parsedBody:d}=yield s;return {years:c,makes:u,models:l,configurations:d}}catch(o){const a=new M("Error loading full vehicle options",o);throw V(a),n(a),a}})),N=L(`${Yt}/LoadYears`,(e,t)=>dt(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield me.getYears(r.config)).parsedBody}catch(o){const a=new M("Error loading vehicle years",o);throw V(a),n(a),a}})),H=L(`${Yt}/LoadMakes`,(e,t)=>dt(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield me.getMakes(r.year,r.config,{type:r.type})).parsedBody}catch(o){const a=new M("Error loading vehicle makes",o);throw V(a),n(a),a}})),te=L(`${Yt}/LoadModels`,(e,t)=>dt(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield me.getModels(r.year,r.makeId,r.config,r.type)).parsedBody}catch(o){const a=new M("Error loading vehicle makes",o);throw V(a),n(a),a}})),Gr=L(`${Yt}/LoadConfigurations`,(e,t)=>dt(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield me.getConfigurations(r.year,r.makeId,r.modelId,r.config,r.type)).parsedBody}catch(o){const a=new M("Error loading vehicle configurations",o);throw V(a),n(a),a}})),D=L(`${Yt}/LoadVin`,(e,t)=>dt(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return yield me.getVinDetails(r.vin,r.config)}catch(o){const a=new M("Error loading VIN",o);throw V(a),n(a),a}})),vo=L("selectVehicle/Save",(e,t)=>dt(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const o=yield me.selectVehicle(r);return o&&r.callback&&r.callback(),o.parsedBody}catch(o){const a=new M("Error saving vehicle selection",o);throw V(a),n(a),a}}));var uz=Object.defineProperty,lz=Object.defineProperties,dz=Object.getOwnPropertyDescriptors,qv=Object.getOwnPropertySymbols,pz=Object.prototype.hasOwnProperty,fz=Object.prototype.propertyIsEnumerable,xv=(e,t,r)=>t in e?uz(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,pt=(e,t)=>{for(var r in t||(t={}))pz.call(t,r)&&xv(e,r,t[r]);if(qv)for(var r of qv(t))fz.call(t,r)&&xv(e,r,t[r]);return e},ho=(e,t)=>lz(e,dz(t));const kv=pt({},eb),Dv=E({name:"questionSets",initialState:kv,reducers:{switchActiveQuestionSet:(e,t)=>{const r=e.list.find(n=>n.question_set_id===e.active);r&&vf(r)&&(r.is_completed=Za(r)),e.active=Yn(e,t.payload,r);},updateActiveQuestionSet:(e,{payload:t})=>{Vv(e),e.active=t;},goToCarrierQuestionSet:(e,{payload:t})=>{const r=e.list.find(n=>n.question_set.tenant_id===t);r&&(Vv(e),e.active=r.question_set_id);},markActiveQuestionSetCompleted:e=>{const t=e.list.findIndex(r=>r.question_set_id===e.active);t!==-1&&(e.list[t].is_completed=Ul(e.list[t].answers));},receiveQuestionSet:(e,{payload:t})=>Rv(e,t),removeQuestionSet:(e,{payload:t})=>Qv(e,t),receiveUpdatedAnswer:(e,{payload:t})=>Fv(e,t),updateActiveAnswerId:(e,{payload:t})=>{const r=ho(pt({},e),{activeAnswerId:t,pendingActiveAnswerUpdate:void 0});return e.pendingActiveAnswerUpdate?Te(r,e.pendingActiveAnswerUpdate):r},removeActiveAnswerId:e=>{const t=ho(pt({},e),{activeAnswerId:"",pendingActiveAnswerUpdate:void 0});return e.pendingActiveAnswerUpdate?Te(t,e.pendingActiveAnswerUpdate):t}},extraReducers:e=>{e.addCase(Vr.fulfilled,(t,{payload:r,meta:n})=>{if(Cz(t,r))return t;Tz(r)?_z(t,n):$z(t,r,n);}).addCase(X.pending,(t,{meta:r})=>Te(t,ho(pt({},r.arg.answer),{version:r.arg.answer.version+1}),!1)).addCase(X.rejected,(t,{meta:r})=>Bv(t,r.arg.answer)?t:Te(t,r.arg.answer,!1)).addCase(X.fulfilled,(t,{payload:r})=>Bv(t,r)?t:Te(t,r)).addCase(Zo,(t,{payload:r})=>{const n=ho(pt({},Te(t,r,!1)),{pendingActiveAnswerUpdate:void 0}),o=Bi(t,r);if(o!==-1){const a=n.list[o];a.is_completed=!1;}return n}).addCase(Ur.fulfilled,(t,{payload:r})=>Vi(t,r,!1)).addCase(ge,(t,{payload:r})=>{let n=pt({},t);return r.events.forEach(o=>{switch(o.event){case T.AddQuestionSet:n=Rv(n,o.data);break;case T.RemoveQuestionSet:n=Qv(n,o.data);break;case T.Answer:n=Fv(n,o.data);break}}),n}),e.addMatcher(_(..._e),(t,{payload:r})=>{const n=rR(r.application_forms_question_sets);t.list=wz(r.application_forms_question_sets,n),t.active=Bt(t.list),t.activeAnswerId="",t.parentQuestionSets=n;}).addMatcher(_(...k),t=>pt({},kv));}}),{switchActiveQuestionSet:vz,updateActiveQuestionSet:hz,receiveQuestionSet:Mv,removeQuestionSet:Nv,receiveUpdatedAnswer:Lv,updateActiveAnswerId:gz,removeActiveAnswerId:mz,goToCarrierQuestionSet:bz,markActiveQuestionSetCompleted:yz}=Dv.actions,_z=(e,t)=>{e.completedQuestionSetAnswerHasChanged=!(e.completedQuestionSetAnswerHasChanged&&Gi(e.list));const r=t.arg.parentQuestionSet?hf(t.arg.parentQuestionSet,t.arg.appFormQuestionSet,e):void 0;e.active=$f(e,t.arg.navigationDirection,r);},$z=(e,t,r)=>{const n=Vi(e,t,!1);e.list=n.list,e.parentQuestionSets=n.parentQuestionSets;const o=r.arg.parentQuestionSet?hf(r.arg.parentQuestionSet,r.arg.appFormQuestionSet,e):void 0,a=wf(n,t,r.arg.navigationDirection,o);a?e.active=a:t.next_active_question_set_id?e.idOfNextActiveQuestionSetNotInState=t.next_active_question_set_id:e.active=Bt(n.list);},Rv=(e,t)=>{const r=Vi(e,t);return e.list=r.list,e.completedQuestionSetAnswerHasChanged=r.completedQuestionSetAnswerHasChanged,e.parentQuestionSets=r.parentQuestionSets,e.idOfNextActiveQuestionSetNotInState===t.id?(e.active=Jn(e,e.idOfNextActiveQuestionSetNotInState),e.idOfNextActiveQuestionSetNotInState=void 0):e.active=Af(e,t),e},Fv=(e,t)=>Te(e,t),Qv=(e,t)=>(e.list=e.list.filter(r=>r.id!==t.id&&r.parentQuestionSetId!==t.id),e.parentQuestionSets=e.parentQuestionSets.filter(r=>r.id!==t.id),e);function Te(e,t,r=!0){const n=Bi(e,t);if(n!==-1){const o=pf(e),{answer:a,index:i}=Pz(o,t),s=o.list[n];if(a)qz(a,t)&&(Ez(t)?xz(e,o,a,t,r):s.answers.splice(i,1));else if(t.active&&t.display_type!=="private"){const c=kz(s,t);s.answers.splice(c,0,t);}return s.parentQuestionSetId&&(s.is_blank=s.answers.length===0),o}return e}function Vi(e,t,r=!0){let n=pf(e),o=n.completedQuestionSetAnswerHasChanged;if(ff(t))n=tR(n,t);else {const a=n.list.findIndex(i=>i.question_set_id===t.question_set_id);if(a>-1){const i=n.list[a],s=!e.completedQuestionSetAnswerHasChanged&&Uv(i.is_completed,t.is_completed);if(i.completed_at=t.completed_at,i.is_active=t.is_active,i.is_completed=t.is_completed,i.order=t.order,i.updated_at=t.updated_at,i.is_blank=t.is_blank,r)for(const c of t.answers)n=Te(n,c);s&&(o=!0);}else n.list=Sz(n.list,t);}return n.completedQuestionSetAnswerHasChanged&&Gi(n.list)&&(o=!1),n.completedQuestionSetAnswerHasChanged=o,n}function wz(e,t){return nR(e,t)}function Az(e,t){return e.findIndex(r=>r.question_set_id===t)}function Oz(e,t,r){const n=t.list.findIndex(o=>o.answers.find(a=>a.question.id===r));if(n===-1){const o=e.question_set.question_groups.find(a=>a.question_ids.includes(r));if(o){const a=t.list.findIndex(i=>i.question_set.question_groups.find(s=>s.id===o.id));return a===-1?t.list.findIndex(i=>i.id===o.id):a}}return n}const Uv=(e,t)=>!!e&&!t;function Bi(e,t){const r=e.parentQuestionSets.find(n=>n.question_set_id===t.question.question_set_id);return r?Oz(r,e,t.question.id):Az(e.list,t.question.question_set_id)}function Vv(e){const t=e.list.find(r=>r.question_set_id===e.active);t&&vf(t)&&(t.is_completed=Za(t));}function Sz(e,t){const r=jz(e,t.order);return e.splice(r,0,t),e}function jz(e,t){let r=e.findIndex(n=>n.order>=t);return r===-1&&(r=e.length),r}function Gi(e){return e.length?e.slice(0,-1).every(t=>t.is_completed):!1}function Cz(e,t){var r;return !!t.application_form_id&&((r=e.list[0])==null?void 0:r.application_form_id)!==t.application_form_id}function Pz(e,t){const r=Bi(e,t);return Iz(t,r>=0?e.list[r]:void 0)}function Iz(e,t){const r=(a,i)=>({answer:i&&a>=0?i.answers[a]:void 0,index:a}),n=(a,i)=>{const s=c=>u=>u.id===c;return i.findIndex(s(a))};let o=-1;return t&&(o=n(e.id,t.answers)),r(o,t)}function Ez(e){return e.active&&e.display_type!=="private"}function Bv(e,t){var r;return ((r=e.list[0])==null?void 0:r.application_form_id)!==t.application_form_id}function Tz(e){return !e.id}function qz(e,t){const r=new Date(e.updated_at);return new Date(t.updated_at)>=r&&t.version>=e.version||t.created_at===e.created_at}function xz(e,t,r,n,o){e.activeAnswerId!==n.id?Object.assign(r,n):(Object.assign(r,n,{value:r.value,choice_id:r.choice_id,choice_ids:r.choice_ids,updated_at:r.updated_at}),o&&(t.pendingActiveAnswerUpdate=n));}function kz(e,t){let r=e.answers.length;for(let n=0;n<e.answers.length;n++)if(e.answers[n].question.order>=t.question.order){r=n;break}return r}const Dz=[],Gv=E({name:"quotes",initialState:Dz,reducers:{receiveQuoteRequest:(e,{payload:t})=>Wv(e,t)},extraReducers:e=>{e.addCase(ge,(t,{payload:r})=>{let n=[...t];return r.events.forEach(o=>{switch(o.event){case T.QuoteRequest:n=Wv(n,o.data);break}}),n}).addMatcher(_(..._e),(t,{payload:r})=>r!=null&&r.quote_requests?r.quote_requests:t).addMatcher(_(...k),t=>[]);}}),{receiveQuoteRequest:zv}=Gv.actions,Wv=(e,t)=>{const r=e.findIndex(n=>n.id===t.id);if(r===-1){if(t.active)return e.push(t),e}else {const n=e[r],o=new Date(n.updated_at);if(new Date(t.updated_at)>=o)return t.active?e[r]=t:e.splice(r,1),e}return e},Mz=(e,t,r="")=>{const{data:n,event:o}=e;switch(o){case T.Appetite:t.dispatch(up(n));break;case T.AddQuestionSet:t.dispatch(Mv(n));break;case T.RemoveQuestionSet:t.dispatch(Nv(n));break;case T.ApplicationForm:t.dispatch(Ir(n));break;case T.Message:t.dispatch(zn(n));break;case T.QuoteRequest:t.dispatch(zv(n));break;case T.Answer:t.dispatch(Lv(n));break;case T.CarrierProducts:t.dispatch(ui(n.length>0?n.filter(a=>a.application_form_id===r):n));break;case T.ApplicationFormPercentCompleted:t.dispatch(ci(n));break;case T.ApplicationFormProcess:t.dispatch(mp(n));break;case T.RealTimeEligibility:t.dispatch(lp(n));break;case T.Notification:t.dispatch(hp(n));break;case T.Batch:t.dispatch(ge(n));break}},Nz=(e,t)=>{if(!(e!=null&&e.data))return !0;switch(e.event){case T.ApplicationForm:return e.data.id!==t;case T.ApplicationFormPercentCompleted:return e.data.application_form_id!==t;case T.CarrierProducts:return e.data.length>0?e.data.filter(r=>r.application_form_id===t).length===0:!1;case T.Notification:return !1;case T.Batch:return !1;case T.Appetite:default:return e.data.application_form_id!==t}},Lz=(e,t,r)=>{Nz(t,r)||Mz(t,e,r);},Rz=e=>t=>r=>{Lz(e,r,t);};let Hv;const Zv=(e,t,r)=>{e===mt.Disconnected&&console.error("Websocket has disconnected. Attempting to reconnect."),r.dispatch(an({status:e,appId:t}));},Fz=(e,t)=>()=>Zv(mt.Connected,e,t),Qz=(e,t)=>()=>Zv(mt.Disconnected,e,t),Uz=(e,t,r=Hv)=>{r!=null&&r.disconnect();const n=Rz(e),{config:{wsBaseUri:o,options:{enableWebSockets:a}}}=e.getState(),{payload:{cable_token:i,id:s}}=t;a&&o&&(Hv=r=new Tb({wsBaseUri:o,token:i,id:s}),r.start(),r.subscribe("ApplicationFormChannel",{id:s},{received:n(s),connected:Fz(s,e),disconnected:Qz(s,e)}));},Vz=e=>t=>r=>(_e.findIndex(n=>n.type===r.type)>=0&&(e.dispatch(Ko({appId:r.payload.id})),e.dispatch(an({status:mt.Connecting,appId:r.payload.id})),Uz(e,r)),t(r)),Bz=e=>t=>r=>{const{config:{debug:n}}=e.getState();if(n){console.group(r.type),console.info("dispatching",r);const o=t(r);return console.log("next state",e.getState()),console.groupEnd(),o}return t(r)};var Gz=Object.defineProperty,zz=Object.defineProperties,Wz=Object.getOwnPropertyDescriptors,Kv=Object.getOwnPropertySymbols,Hz=Object.prototype.hasOwnProperty,Zz=Object.prototype.propertyIsEnumerable,Yv=(e,t,r)=>t in e?Gz(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Jt=(e,t)=>{for(var r in t||(t={}))Hz.call(t,r)&&Yv(e,r,t[r]);if(Kv)for(var r of Kv(t))Zz.call(t,r)&&Yv(e,r,t[r]);return e},Kz=(e,t)=>zz(e,Wz(t));const Jv=Jt({},Wm),Xv=E({name:"auth",initialState:Jv,reducers:{setAuth:(e,{payload:t})=>Kz(Jt(Jt({},e),t),{refreshCount:e.refreshCount+1}),setUser:(e,{payload:t})=>Jt(Jt({},e),eT(t)),setImpersonatedOnAuth:(e,{payload:t})=>{e.options.impersonated=t.sudo;},logout:e=>Jt({},Jv),setTheme:(e,{payload:t})=>{e.terminalTheme=t;}}}),{setAuth:eh,setUser:th,setImpersonatedOnAuth:Yz,logout:zr,setTheme:Jz}=Xv.actions;var Xz=Object.defineProperty,eW=Object.defineProperties,tW=Object.getOwnPropertyDescriptors,rh=Object.getOwnPropertySymbols,rW=Object.prototype.hasOwnProperty,nW=Object.prototype.propertyIsEnumerable,nh=(e,t,r)=>t in e?Xz(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Xt=(e,t)=>{for(var r in t||(t={}))rW.call(t,r)&&nh(e,r,t[r]);if(rh)for(var r of rh(t))nW.call(t,r)&&nh(e,r,t[r]);return e},oW=(e,t)=>eW(e,tW(t));const oh=Xt({},Km),ah=E({name:"config",initialState:oh,reducers:{setConfig:(e,{payload:t})=>oW(Xt(Xt({},e),t),{options:Xt(Xt({},e.options),t.options)}),setWebSocketConfig:(e,{payload:t})=>{e.options.enableWebSockets=t.enableWebsocket;}},extraReducers:e=>{e.addCase(ir,t=>Xt({},oh)).addMatcher(_(..._e),(t,{payload:r})=>{t.cableToken=r.cable_token;});}}),{setConfig:ih,setWebSocketConfig:aW}=ah.actions;var iW=Object.defineProperty,sh=Object.getOwnPropertySymbols,sW=Object.prototype.hasOwnProperty,cW=Object.prototype.propertyIsEnumerable,ch=(e,t,r)=>t in e?iW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,uh=(e,t)=>{for(var r in t||(t={}))sW.call(t,r)&&ch(e,r,t[r]);if(sh)for(var r of sh(t))cW.call(t,r)&&ch(e,r,t[r]);return e};const lh=uh({},Ns),dh=E({name:"status/Local",initialState:lh,reducers:{setLocalStatusState:(e,{payload:t})=>{e.state=t;},setLocalStatusHealth:(e,{payload:t})=>{e.health=t;}},extraReducers:e=>{e.addCase(ir,t=>uh({},lh)).addCase(eh,t=>{t.auth=x.Configured;}).addCase(ih,t=>{t.config=x.Configured;}).addCase(th,t=>{t.user=x.Configured;}).addCase(Us,t=>{t.auth=x.Error,t.user=x.Error;}).addCase(zr,t=>{t.auth=x.NotLoaded,t.user=x.NotLoaded;}).addCase(an,(t,{payload:r})=>{t.socket=r.appId===t.socketId?r.status:t.socket;}).addCase(Ko,(t,{payload:r})=>{t.socketId=r.appId;});}}),{setLocalStatusState:go,setLocalStatusHealth:mo}=dh.actions;function ph(e,t,r){t(X({answer:r,config:ud(e)}));}function uW(e,t){switch(t.type){case X.fulfilled.type:return e.application.form.id!==t.payload.application_form_id;default:return !1}}function fh(e,t){const r=e.queues.answers.queue.length,n=Object.keys(e.queues.answers.outbound).length;r&&!n&&ph(e,t.dispatch,e.queues.answers.queue[0]);}function lW(e,t,r){const n=e.queues.answers.queue.length,o=Object.keys(e.queues.answers.outbound).length;!n&&!o?ph(e,t.dispatch,r):t.dispatch(on(r));}function dW(e,t,r){e.queues.answers.queue[0]&&e.queues.answers.queue[0].id!==r.payload.id&&fh(e,t);}const pW=e=>t=>r=>{const n=e.getState();if(uW(n,r))return t(r);switch(r.type){case X.fulfilled.type:dW(n,e,r);break;case X.pending.type:case go.type:case mo.type:case zn.type:case zr.type:break;case Zo.type:lW(n,e,r.payload);break;default:fh(n,e);break}return t(r)};var fW=Object.defineProperty,vW=Object.defineProperties,hW=Object.getOwnPropertyDescriptors,vh=Object.getOwnPropertySymbols,gW=Object.prototype.hasOwnProperty,mW=Object.prototype.propertyIsEnumerable,hh=(e,t,r)=>t in e?fW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Wr=(e,t)=>{for(var r in t||(t={}))gW.call(t,r)&&hh(e,r,t[r]);if(vh)for(var r of vh(t))mW.call(t,r)&&hh(e,r,t[r]);return e},zi=(e,t)=>vW(e,hW(t));const gh=Wr({},Ym),mh=E({name:"error",initialState:gh,reducers:{updateErrors:(e,{payload:t})=>zi(Wr({},e),{[t.answer.id]:t.errors}),removeError:(e,{payload:t})=>zi(Wr({},e),{[t]:[]})},extraReducers:e=>{e.addCase(X.pending,(t,{meta:r})=>{const{answer:n}=r.arg,o=Ha(n);t[n.id]=o;}).addCase(X.rejected,(t,{meta:r,payload:n})=>{var o,a,i;const s=(a=(o=n.response)==null?void 0:o.parsedBody)!=null?a:[],c=r.arg.answer.id,u=(i=t[c])!=null?i:[];return zi(Wr({},t),{[c]:[...u,...s]})}),e.addMatcher(_(...k),t=>Wr({},gh));}}),{updateErrors:bW,removeError:bh}=mh.actions,yW=e=>t=>t.question.reference_type===Xr.Accumulator&&_W(t.id,e)&&Ql(t),_W=(e,t)=>t[e]!==void 0,$W=(e,t)=>{if(!e)return;const{errors:r,questionSets:{list:n}}=e;n.flatMap(o=>o.answers).filter(o=>yW(r)(o)).forEach(o=>{t.dispatch(bh(o.id));});},wW=e=>t=>r=>{if(r.type===X.fulfilled.type){const{application:n}=e.getState();$W(n,e);}return t(r)},yh=e=>[e.status.remote.application.form,e.status.remote.questionSets.state,e.status.remote.questionPools.state,e.status.remote.answers.state,...Object.values(e.status.remote.bindQuotes),...Object.values(e.status.remote.choices),...Object.values(e.status.remote.refilters),...Object.values(e.status.remote.vehicles.configurations),...Object.values(e.status.remote.vehicles.makes),...Object.values(e.status.remote.vehicles.models),...Object.values(e.status.remote.vehicles.selection),...Object.values(e.status.remote.vehicles.vins),...Object.values(e.status.remote.vehicles.years)],AW=e=>[...yh(e),e.status.remote.quotes.state],OW=e=>e.status.remote.application.form,SW=e=>!!(e.auth.user&&e.config.clientId),jW=e=>yh(e).reduce((t,r)=>{switch(r){case b.Error:t=x.Configured;break;case b.NotLoaded:t=x.Configured;break;case b.Loading:t!==x.Configured&&(t=x.Loading);break;case b.Loaded:t!==x.Configured&&t!==x.Loading&&(t=x.Loaded);break}return t},x.Loaded),CW=e=>e===b.Error?yt.Error:yt.Healthy,PW=e=>AW(e).reduce((t,r)=>t===yt.Error?t:CW(r),yt.Healthy),IW=e=>{let t=x.NotLoaded;if(SW(e)){t=x.Configured;const r=OW(e);r===b.Loaded?t=jW(e):r===b.Loading&&(t=x.Loading);}return t},EW=e=>t=>r=>{const n=e.getState();switch(r.type){case go.type:case mo.type:return t(r)}const o=IW(n),a=PW(n);return o!==n.status.local.state&&e.dispatch(go(o)),a!==n.status.local.health&&e.dispatch(mo(a)),t(r)},TW=new M().name;function qW(e){return e.payload&&e.payload.error&&TW===e.payload.error.name}const xW=e=>t=>r=>{if(r.type===zr.type)return YE(),t(r);const n=e.getState().config.debug;if(qW(r)){const o=r.payload.error.response;if((o==null?void 0:o.status)===401){n&&console.error("User is trying to make an unauthenticated request, logging out."),e.dispatch(zr());const a={id:"logoutExpired",tenant_id:"",owner_id:"",target:"Your login has expired. Please log in again to continue.",application_form_id:"",active:!0,created_at:new Date,updated_at:new Date,code:"",metadata:{},component:$t.Application};e.dispatch(zn(a));}}return t(r)};var kW=[wW,Bz,pW,EW,xW,Vz];const DW=[],MW=E({name:"carriersProducts",initialState:DW,reducers:{},extraReducers:e=>{e.addCase(ge,(t,{payload:r})=>{let n=[...t];return r.events.forEach(o=>{o.event===T.CarrierProducts&&kt(o.data,t)&&(n=o.data);}),n}),e.addMatcher(_(ui,Kt.fulfilled),(t,{payload:r})=>kt(r,t)?r:t),e.addMatcher(_(Kt.pending,...k),()=>[]);}});var NW=Object.defineProperty,_h=Object.getOwnPropertySymbols,LW=Object.prototype.hasOwnProperty,RW=Object.prototype.propertyIsEnumerable,$h=(e,t,r)=>t in e?NW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,wh=(e,t)=>{for(var r in t||(t={}))LW.call(t,r)&&$h(e,r,t[r]);if(_h)for(var r of _h(t))RW.call(t,r)&&$h(e,r,t[r]);return e};const Ah=wh({},Hm),FW=E({name:"choice",initialState:Ah,reducers:{},extraReducers:e=>{e.addCase(Qr.fulfilled,(t,r)=>{var n;const o=r.meta.arg,a=t[o.questionId];if(a&&a.timestamp>o.timestamp)return t;t[o.questionId]={list:(n=r.payload)!=null?n:[],timestamp:o.timestamp};}).addCase(Ie.fulfilled,(t,r)=>{var n;const o=r.meta.arg,a=t[o.questionId];if(a&&a.timestamp>o.timestamp)return t;t[o.questionId]={list:(n=r.payload.choices)!=null?n:[],timestamp:o.timestamp};}),e.addMatcher(_(...k),t=>wh({},Ah));}});var QW=Object.defineProperty,Oh=Object.getOwnPropertySymbols,UW=Object.prototype.hasOwnProperty,VW=Object.prototype.propertyIsEnumerable,Sh=(e,t,r)=>t in e?QW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,jh=(e,t)=>{for(var r in t||(t={}))UW.call(t,r)&&Sh(e,r,t[r]);if(Oh)for(var r of Oh(t))VW.call(t,r)&&Sh(e,r,t[r]);return e};const Ch=jh({},Xm),bo=(e,t)=>{e.meta.form.percent=t,t>e.meta.form.forwardProgressPercent&&(e.meta.form.forwardProgressPercent=t);},BW=E({name:"progress",initialState:Ch,reducers:{},extraReducers:e=>{e.addCase(Ir,(t,{payload:r})=>{bo(t,r.percent_completed);}).addCase(ci,(t,{payload:r})=>{bo(t,r.percent_completed);}).addCase(ge,(t,{payload:r})=>(r.events.forEach(n=>{switch(n.event){case T.ApplicationForm:case T.ApplicationFormPercentCompleted:bo(t,n.data.percent_completed);break}}),t)),e.addMatcher(_(..._e),(t,{payload:r})=>{bo(t,r.percent_completed);}).addMatcher(_(...k),t=>jh({},Ch));}});var GW=Object.defineProperty,Ph=Object.getOwnPropertySymbols,zW=Object.prototype.hasOwnProperty,WW=Object.prototype.propertyIsEnumerable,Ih=(e,t,r)=>t in e?GW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Eh=(e,t)=>{for(var r in t||(t={}))zW.call(t,r)&&Ih(e,r,t[r]);if(Ph)for(var r of Ph(t))WW.call(t,r)&&Ih(e,r,t[r]);return e};const Th=Eh({},sb),HW=E({name:"queues",initialState:Th,reducers:{},extraReducers:e=>{e.addCase(on,(t,{payload:r})=>{const n=t.answers.queue.findIndex(o=>o.id===r.id);n===-1?t.answers.queue.push(r):t.answers.queue[n]=r;}).addCase(X.pending,(t,{meta:r})=>{if(!t.answers.outbound[r.arg.answer.id]){t.answers.outbound[r.arg.answer.id]=r.arg.answer;const n=t.answers.queue.findIndex(o=>o.id===r.arg.answer.id);n!==-1&&t.answers.queue.splice(n,1);}}).addCase(X.fulfilled,(t,{payload:r})=>{const n=t.answers.queue.find(o=>o.id===r.id);n&&(n.version=r.version);}),e.addMatcher(_(X.fulfilled,X.rejected),(t,{meta:r})=>{t.answers.outbound[r.arg.answer.id]&&delete t.answers.outbound[r.arg.answer.id];}).addMatcher(_(...k),()=>Eh({},Th));}});var ZW=Object.defineProperty,qh=Object.getOwnPropertySymbols,KW=Object.prototype.hasOwnProperty,YW=Object.prototype.propertyIsEnumerable,xh=(e,t,r)=>t in e?ZW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,kh=(e,t)=>{for(var r in t||(t={}))KW.call(t,r)&&xh(e,r,t[r]);if(qh)for(var r of qh(t))YW.call(t,r)&&xh(e,r,t[r]);return e};const Dh=kh({},tb),JW=E({name:"quoteBind",initialState:Dh,reducers:{},extraReducers:e=>{e.addCase(Br.fulfilled,(t,{payload:r,meta:n})=>{const{retrieve_url:o}=r;t[n.arg.quoteId]=o;}),e.addMatcher(_(...k),t=>kh({},Dh));}});var XW=Object.defineProperty,Mh=Object.getOwnPropertySymbols,e4=Object.prototype.hasOwnProperty,t4=Object.prototype.propertyIsEnumerable,Nh=(e,t,r)=>t in e?XW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Lh=(e,t)=>{for(var r in t||(t={}))e4.call(t,r)&&Nh(e,r,t[r]);if(Mh)for(var r of Mh(t))t4.call(t,r)&&Nh(e,r,t[r]);return e};const Rh=Lh({},rb),r4=E({name:"refilters",initialState:Rh,reducers:{},extraReducers:e=>{e.addCase(Ie.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=t[a.questionId];if(i&&i.timestamp>a.timestamp)return t;t[a.questionId]={list:(o=r==null?void 0:r.refilter)!=null?o:[],timestamp:a.timestamp};}),e.addMatcher(_(...k),t=>Lh({},Rh));}}),n4={},o4=E({name:"status/remote/vehicles/configurations",initialState:n4,reducers:{},extraReducers:e=>{e.addCase(Gr.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.NotLoaded;}).addCase(Ee.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loaded;}).addCase(Gr.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Error;}),e.addMatcher(_(te.pending,H.pending,N.pending,D.pending,te.rejected,te.fulfilled,H.rejected,H.fulfilled,N.rejected,N.fulfilled,D.rejected,D.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=b.NotLoaded;}).addMatcher(_(...k),t=>({}));}}),a4={},i4=E({name:"status/remote/vehicles/makes",initialState:a4,reducers:{},extraReducers:e=>{e.addCase(H.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loading;}).addCase(H.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loaded;}).addCase(H.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Error;}),e.addMatcher(_(N.pending,D.pending,N.rejected,N.fulfilled,D.rejected,D.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=b.NotLoaded;}).addMatcher(_(...k),t=>({}));}}),s4={},c4=E({name:"status/remote/vehicles/models",initialState:s4,reducers:{},extraReducers:e=>{e.addCase(te.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loading;}).addCase(te.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loaded;}).addCase(te.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Error;}),e.addMatcher(_(N.pending,N.rejected,N.fulfilled,H.pending,H.rejected,H.fulfilled,D.pending,D.rejected,D.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=b.NotLoaded;}).addMatcher(_(...k),t=>({}));}}),u4={},l4=E({name:"status/remote/vehicles/selections",initialState:u4,reducers:{},extraReducers:e=>{e.addCase(vo.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loading;}).addCase(vo.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loaded;}).addCase(vo.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Error;}).addCase(Gs,(t,{payload:r})=>{t[r.questionGroupId]=b.NotLoaded;}),e.addMatcher(_(...k),t=>({}));}}),d4={},p4=E({name:"status/remote/vehicles/vins",initialState:d4,reducers:{},extraReducers:e=>{e.addCase(D.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loading;}).addCase(D.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loaded;}).addCase(D.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Error;}),e.addMatcher(_(N.pending,N.rejected,N.fulfilled,H.pending,H.rejected,H.fulfilled,te.pending,te.rejected,te.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=b.NotLoaded;}).addMatcher(_(...k),t=>({}));}}),f4={},v4=E({name:"status/remote/vehicles/years",initialState:f4,reducers:{},extraReducers:e=>{e.addCase(N.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loading;}).addCase(N.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Loaded;}).addCase(N.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=b.Error;}),e.addMatcher(_(D.pending,D.rejected,D.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=b.NotLoaded;}).addMatcher(_(...k),t=>({}));}});var h4=combineReducers({configurations:o4.reducer,makes:i4.reducer,models:c4.reducer,selection:l4.reducer,vins:p4.reducer,years:v4.reducer}),g4=Object.defineProperty,Fh=Object.getOwnPropertySymbols,m4=Object.prototype.hasOwnProperty,b4=Object.prototype.propertyIsEnumerable,Qh=(e,t,r)=>t in e?g4(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Uh=(e,t)=>{for(var r in t||(t={}))m4.call(t,r)&&Qh(e,r,t[r]);if(Fh)for(var r of Fh(t))b4.call(t,r)&&Qh(e,r,t[r]);return e};const Vh=Uh({},wt),y4=E({name:"status/remote/answers",initialState:Vh,reducers:{},extraReducers:e=>{e.addCase(X.pending,(t,{meta:r})=>oe(t,r.arg.answer.id,b.Loading)).addCase(X.fulfilled,(t,{meta:r})=>oe(t,r.arg.answer.id,b.Loaded)).addCase(X.rejected,(t,{meta:r})=>oe(t,r.arg.answer.id,b.Error)),e.addMatcher(_(...k),t=>Uh({},Vh));}});var _4=Object.defineProperty,Bh=Object.getOwnPropertySymbols,$4=Object.prototype.hasOwnProperty,w4=Object.prototype.propertyIsEnumerable,Gh=(e,t,r)=>t in e?_4(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,A4=(e,t)=>{for(var r in t||(t={}))$4.call(t,r)&&Gh(e,r,t[r]);if(Bh)for(var r of Bh(t))w4.call(t,r)&&Gh(e,r,t[r]);return e};const O4=A4({},xs),S4=E({name:"status/remote/applicationForm",initialState:O4,reducers:{},extraReducers:e=>{e.addMatcher(_(...ai),t=>{t.form=b.NotLoaded;}).addMatcher(_(...Xd,Bn.pending),t=>{t.form=b.Loading;}).addMatcher(_(..._e,Bn.fulfilled),t=>{t.form=b.Loaded;}).addMatcher(_(...ii,Bn.rejected),t=>{t.form=b.Error;});}}),j4={},C4=E({name:"status/remote/bindQuotes",initialState:j4,reducers:{},extraReducers:e=>{e.addCase(Br.pending,(t,{meta:r})=>{t[r.arg.quoteId]=b.Loading;}).addCase(Br.fulfilled,(t,{meta:r})=>{t[r.arg.quoteId]=b.Loaded;}).addCase(Br.rejected,(t,{meta:r})=>{t[r.arg.quoteId]=b.Error;}).addCase(fo.pending,(t,{meta:r})=>{t[r.arg.quote.id]=b.Loading;}).addCase(fo.fulfilled,(t,{meta:r})=>{t[r.arg.quote.id]=b.Loaded;}).addCase(fo.rejected,(t,{meta:r})=>{t[r.arg.quote.id]=b.Error;}),e.addMatcher(_(...k),t=>({}));}});var P4=Object.defineProperty,zh=Object.getOwnPropertySymbols,I4=Object.prototype.hasOwnProperty,E4=Object.prototype.propertyIsEnumerable,Wh=(e,t,r)=>t in e?P4(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Hh=(e,t)=>{for(var r in t||(t={}))I4.call(t,r)&&Wh(e,r,t[r]);if(zh)for(var r of zh(t))E4.call(t,r)&&Wh(e,r,t[r]);return e};const Zh=Hh({},tn),T4=E({name:"status/remote/carriersProducts",initialState:Zh,reducers:{},extraReducers:e=>{e.addCase(Kt.pending,t=>{t.state=b.Loading;}).addCase(Kt.fulfilled,t=>{t.state=b.Loaded;}).addCase(Kt.rejected,t=>{t.state=b.Error;}),e.addMatcher(_(...k),t=>Hh({},Zh));}}),q4={},x4=E({name:"status/remote/choices",initialState:q4,reducers:{},extraReducers:e=>{e.addCase(Ie.pending,(t,{meta:r})=>{t[r.arg.questionId]=b.Loading;}).addCase(Ie.fulfilled,(t,{meta:r})=>{t[r.arg.questionId]=b.Loaded;}).addCase(Ie.rejected,(t,{meta:r})=>{t[r.arg.questionId]=b.Error;}),e.addMatcher(_(...k),t=>({}));}});var k4=Object.defineProperty,Kh=Object.getOwnPropertySymbols,D4=Object.prototype.hasOwnProperty,M4=Object.prototype.propertyIsEnumerable,Yh=(e,t,r)=>t in e?k4(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Jh=(e,t)=>{for(var r in t||(t={}))D4.call(t,r)&&Yh(e,r,t[r]);if(Kh)for(var r of Kh(t))M4.call(t,r)&&Yh(e,r,t[r]);return e};const Xh=Jh({},wt),N4=E({name:"status/remote/questionPool",initialState:Xh,reducers:{},extraReducers:e=>{e.addCase(lo.pending,(t,{meta:r})=>oe(t,r.arg.question_pool_id,b.Loading)).addCase(lo.fulfilled,(t,{meta:r})=>oe(t,r.arg.question_pool_id,b.Loaded)).addCase(lo.rejected,(t,{meta:r})=>oe(t,r.arg.question_pool_id,b.Error)).addCase(po.pending,(t,{meta:r})=>oe(t,r.arg.answer_pool_id,b.Loading)).addCase(po.fulfilled,(t,{meta:r})=>oe(t,r.arg.answer_pool_id,b.Loaded)).addCase(po.rejected,(t,{meta:r})=>oe(t,r.arg.answer_pool_id,b.Error)),e.addMatcher(_(...k),t=>Jh({},Xh));}});var L4=Object.defineProperty,eg=Object.getOwnPropertySymbols,R4=Object.prototype.hasOwnProperty,F4=Object.prototype.propertyIsEnumerable,tg=(e,t,r)=>t in e?L4(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,rg=(e,t)=>{for(var r in t||(t={}))R4.call(t,r)&&tg(e,r,t[r]);if(eg)for(var r of eg(t))F4.call(t,r)&&tg(e,r,t[r]);return e};const ng=rg({},wt),Q4=E({name:"status/remote/questionSets",initialState:ng,reducers:{},extraReducers:e=>{e.addCase(Ur.pending,(t,{meta:r})=>oe(t,r.arg.id,b.Loading)).addCase(Ur.fulfilled,(t,{meta:r})=>oe(t,r.arg.id,b.Loaded)).addCase(Ur.rejected,(t,{meta:r})=>oe(t,r.arg.id,b.Error)),e.addCase(Vr.pending,(t,{meta:r})=>oe(t,r.arg.appFormQuestionSet.id,b.Loading)).addCase(Vr.fulfilled,(t,{meta:r})=>oe(t,r.arg.appFormQuestionSet.id,b.Loaded)).addCase(Vr.rejected,(t,{meta:r})=>oe(t,r.arg.appFormQuestionSet.id,b.Error)),e.addMatcher(_(...k),t=>rg({},ng));}});var U4=Object.defineProperty,og=Object.getOwnPropertySymbols,V4=Object.prototype.hasOwnProperty,B4=Object.prototype.propertyIsEnumerable,ag=(e,t,r)=>t in e?U4(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,ig=(e,t)=>{for(var r in t||(t={}))V4.call(t,r)&&ag(e,r,t[r]);if(og)for(var r of og(t))B4.call(t,r)&&ag(e,r,t[r]);return e};const sg=ig({},Cs),G4=E({name:"viewState/Features",initialState:sg,reducers:{},extraReducers:e=>{e.addCase(wv.fulfilled,(t,{payload:r})=>{t.list=r,t.isLoaded=!0;}).addCase(ir,t=>ig({},sg));}});var z4=Object.defineProperty,W4=Object.defineProperties,H4=Object.getOwnPropertyDescriptors,cg=Object.getOwnPropertySymbols,Z4=Object.prototype.hasOwnProperty,K4=Object.prototype.propertyIsEnumerable,ug=(e,t,r)=>t in e?z4(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Y4=(e,t)=>{for(var r in t||(t={}))Z4.call(t,r)&&ug(e,r,t[r]);if(cg)for(var r of cg(t))K4.call(t,r)&&ug(e,r,t[r]);return e},J4=(e,t)=>W4(e,H4(t));const X4={},e8=E({name:"viewState/vehicleOptions/Configurations",initialState:X4,reducers:{},extraReducers:e=>{e.addCase(Ee.fulfilled,(t,{payload:r,meta:n})=>{const o=n.arg,a=Ya(r);t[o.questionGroupId]=Jl(a);}).addCase(Gr.fulfilled,(t,{payload:r,meta:n})=>{t[n.arg.questionGroupId]=Jl(r);}).addCase(D.fulfilled,(t,{payload:r,meta:n})=>{const o=r,a=o==null?void 0:o.parsedBody;if(!r)return t;const i=Ya(a);t[n.arg.questionGroupId]=i.reduce((s,c)=>J4(Y4({},s),{[c.id]:c.name}),{});}),e.addMatcher(_(Gr.pending,te.pending,H.pending,N.pending,D.pending,te.fulfilled,H.fulfilled,N.fulfilled,te.rejected,H.rejected,N.rejected,D.rejected),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]={};}).addMatcher(_(...k),t=>({}));}}),t8={},r8=E({name:"viewState/vehicleOptions/Makes",initialState:t8,reducers:{},extraReducers:e=>{e.addCase(Ee.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r.makes)!=null?o:[];t[a.questionGroupId]=Kl(i);}).addCase(H.fulfilled,(t,{payload:r,meta:n})=>{const o=n.arg,a=r!=null?r:[];t[o.questionGroupId]=Kl(a);}),e.addMatcher(_(H.rejected,N.pending,D.pending,N.fulfilled,D.rejected,D.fulfilled),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]={};}).addMatcher(_(...k),t=>({}));}}),n8={},o8=E({name:"viewState/vehicleOptions/Models",initialState:n8,reducers:{},extraReducers:e=>{e.addCase(Ee.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r.models)!=null?o:[];t[a.questionGroupId]=Yl(i);}).addCase(te.fulfilled,(t,{payload:r,meta:n})=>{const o=n.arg,a=r!=null?r:[];t[o.questionGroupId]=Yl(a);}),e.addMatcher(_(H.rejected,H.fulfilled,N.rejected,N.fulfilled,D.rejected,D.fulfilled),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]={};}).addMatcher(_(...k),t=>({}));}});var a8=Object.defineProperty,lg=Object.getOwnPropertySymbols,i8=Object.prototype.hasOwnProperty,s8=Object.prototype.propertyIsEnumerable,dg=(e,t,r)=>t in e?a8(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,pg=(e,t)=>{for(var r in t||(t={}))i8.call(t,r)&&dg(e,r,t[r]);if(lg)for(var r of lg(t))s8.call(t,r)&&dg(e,r,t[r]);return e};const c8={},fg=E({name:"viewState/vehicleOptions/Selection",initialState:c8,reducers:{setVehicleSelection:(e,{payload:t})=>{e[t.questionGroupId]=pg(pg({},e[t.questionGroupId]),t.selection);}},extraReducers:e=>{e.addMatcher(_(...k),t=>({}));}}),{setVehicleSelection:u8}=fg.actions;var l8=Object.defineProperty,d8=Object.defineProperties,p8=Object.getOwnPropertyDescriptors,vg=Object.getOwnPropertySymbols,f8=Object.prototype.hasOwnProperty,v8=Object.prototype.propertyIsEnumerable,hg=(e,t,r)=>t in e?l8(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,er=(e,t)=>{for(var r in t||(t={}))f8.call(t,r)&&hg(e,r,t[r]);if(vg)for(var r of vg(t))v8.call(t,r)&&hg(e,r,t[r]);return e},Wi=(e,t)=>d8(e,p8(t));const gg=3,h8={},g8=E({name:"viewState/vehicleOptions/Vin",initialState:h8,reducers:{},extraReducers:e=>{e.addCase(D.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=r,i=a==null?void 0:a.parsedBody,s=n.arg;if(!i)return t;const c=Ya(i),u=(o=t[s.questionGroupId])!=null?o:er({},Bo);if(c.length===0){const l=u.errorCount+1;return t[s.questionGroupId]=Wi(er({},u),{errorCount:l,isAvailable:l<gg,lastResponse:a}),t}t[s.questionGroupId]=Wi(er({},u),{errorCount:0,isAvailable:!0,lastResponse:void 0});}).addCase(D.rejected,(t,{meta:r})=>{var n;const o=r.arg,a=(n=t[o.questionGroupId])!=null?n:er({},Bo),i=a.errorCount+1;t[o.questionGroupId]=Wi(er({},a),{errorCount:i,isAvailable:i<gg,lastResponse:void 0});}),e.addMatcher(_(Ee.pending,te.pending,H.pending,N.pending,Ee.rejected,Ee.fulfilled,te.rejected,te.fulfilled,H.rejected,H.fulfilled,N.rejected,N.fulfilled),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]=er({},Bo);}).addMatcher(_(...k),t=>({}));}}),m8={},b8=E({name:"viewState/vehicleOptions/Years",initialState:m8,reducers:{},extraReducers:e=>{e.addCase(Ee.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r.years)!=null?o:[];t[a.questionGroupId]=Zl(i);}).addCase(N.fulfilled,(t,{payload:r,meta:n})=>{const o=n.arg,a=r!=null?r:[];t[o.questionGroupId]=Zl(a);}),e.addMatcher(_(N.pending,D.pending,D.rejected,D.fulfilled),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]={};}).addMatcher(_(...k),t=>({}));}});var y8=combineReducers({vehicleOptions:combineReducers({configurations:e8.reducer,makes:r8.reducer,models:o8.reducer,selection:fg.reducer,vin:g8.reducer,years:b8.reducer}),features:G4.reducer}),_8=Object.defineProperty,mg=Object.getOwnPropertySymbols,$8=Object.prototype.hasOwnProperty,w8=Object.prototype.propertyIsEnumerable,bg=(e,t,r)=>t in e?_8(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,yg=(e,t)=>{for(var r in t||(t={}))$8.call(t,r)&&bg(e,r,t[r]);if(mg)for(var r of mg(t))w8.call(t,r)&&bg(e,r,t[r]);return e};const _g=yg({},tn),A8=E({name:"status/remote/quotes",initialState:_g,reducers:{},extraReducers:e=>{e.addCase(Ir,(t,{payload:r})=>wg(t,r)).addCase(Vs,(t,{payload:r})=>{t.state=b.Loaded,t.updated_at=r.updatedAt;}).addCase(ge,(t,{payload:r})=>{r.events.forEach(n=>{if(n.event===T.ApplicationForm)return wg(t,n.data)});}),e.addMatcher(_(...k),t=>yg({},_g)).addMatcher(_(..._e),(t,{payload:r})=>{t.state=$g(r);}).addMatcher(_(...ii),t=>{t.state=b.Error;});}}),$g=e=>{let t=b.NotLoaded;return e.state===_t.Completed&&!e.taken_over&&!e.cloned?t=e.quote_requests_count?b.Loading:b.Loaded:e.state===_t.Quoted&&(t=e.finished_quote_requests_count===e.quote_requests_count?b.Loaded:b.Loading),t},wg=(e,t)=>{const r=new Date(e.updated_at||0),n=new Date(t.updated_at);return n>r?{state:$g(t),updated_at:n}:e},O8={},S8=E({name:"status/remote/refilters",initialState:O8,reducers:{},extraReducers:e=>{e.addMatcher(_(Qr.pending,Ie.pending),(t,{meta:r})=>{t[r.arg.questionId]=b.Loading;}).addMatcher(_(Qr.fulfilled,Ie.fulfilled),(t,{meta:r})=>{t[r.arg.questionId]=b.Loaded;}).addMatcher(_(Qr.rejected,Ie.rejected),(t,{meta:r})=>{t[r.arg.questionId]=b.Error;}).addMatcher(_(...k),t=>({}));}});var j8=combineReducers({vehicles:h4,answers:y4.reducer,application:S4.reducer,bindQuotes:C4.reducer,carriersProducts:T4.reducer,choices:x4.reducer,questionPools:N4.reducer,questionSets:Q4.reducer,quotes:A8.reducer,refilters:S8.reducer}),C8=Object.defineProperty,P8=Object.defineProperties,I8=Object.getOwnPropertyDescriptors,Ag=Object.getOwnPropertySymbols,E8=Object.prototype.hasOwnProperty,T8=Object.prototype.propertyIsEnumerable,Og=(e,t,r)=>t in e?C8(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Sg=(e,t)=>{for(var r in t||(t={}))E8.call(t,r)&&Og(e,r,t[r]);if(Ag)for(var r of Ag(t))T8.call(t,r)&&Og(e,r,t[r]);return e},q8=(e,t)=>P8(e,I8(t));const x8=combineReducers({form:rp.reducer,choices:FW.reducer,errors:mh.reducer,pendingProcesses:gp.reducer,progress:BW.reducer,questionSets:Dv.reducer,quotes:Gv.reducer,quoteBind:JW.reducer,carriersProducts:MW.reducer,refilters:r4.reducer}),k8=combineReducers({remote:j8,local:dh.reducer});let D8="production";const Hi=gm({reducer:{application:x8,auth:Xv.reducer,queues:HW.reducer,config:ah.reducer,notifications:vp.reducer,viewState:y8,status:k8},middleware:e=>e({thunk:!0,immutableCheck:D8!=="production",serializableCheck:!1}).concat([...kW]),devTools:!0}),M8=()=>{var e,t,r,n;return window&&!((e=window.BpSdk)!=null&&e.store)&&(window.BpSdk=q8(Sg({},(t=window.BpSdk)!=null?t:{}),{store:Hi})),(n=(r=window==null?void 0:window.BpSdk)==null?void 0:r.store)!=null?n:Sg({},Hi)};

exports.Bd = Bd;
exports.Bn = Bn;
exports.Br = Br;
exports.Cr = Cr;
exports.D = D;
exports.D2 = D2;
exports.Ee = Ee;
exports.Fd = Fd;
exports.Fk = Fk;
exports.Gr = Gr;
exports.Gs = Gs;
exports.H = H;
exports.H2 = H2;
exports.Hi = Hi;
exports.Ie = Ie;
exports.Ja = Ja;
exports.Jd = Jd;
exports.Jx = Jx;
exports.Kt = Kt;
exports.L2 = L2;
exports.Lt = Lt;
exports.M2 = M2;
exports.M8 = M8;
exports.Mk = Mk;
exports.N = N;
exports.Nk = Nk;
exports.Nt = Nt;
exports.OD = OD;
exports.Qd = Qd;
exports.Qk = Qk;
exports.Qr = Qr;
exports.Rd = Rd;
exports.Rk = Rk;
exports.Rt = Rt;
exports.Sr = Sr;
exports.T2 = T2;
exports.U2 = U2;
exports.Ud = Ud;
exports.Ur = Ur;
exports.Vd = Vd;
exports.Vr = Vr;
exports.Vs = Vs;
exports.Xk = Xk;
exports.Zo = Zo;
exports.aW = aW;
exports.bW = bW;
exports.be = be;
exports.cD = cD;
exports.ck = ck;
exports.cz = cz;
exports.de = de;
exports.ek = ek;
exports.fk = fk;
exports.fo = fo;
exports.g2 = g2;
exports.gz = gz;
exports.h2 = h2;
exports.hz = hz;
exports.k2 = k2;
exports.lo = lo;
exports.mz = mz;
exports.n2 = n2;
exports.ok = ok;
exports.po = po;
exports.qb = qb;
exports.qk = qk;
exports.s2 = s2;
exports.sD = sD;
exports.sd = sd;
exports.t2 = t2;
exports.tD = tD;
exports.te = te;
exports.tt = tt;
exports.u2 = u2;
exports.u8 = u8;
exports.ud = ud;
exports.vo = vo;
exports.vz = vz;
exports.wv = wv;
exports.x2 = x2;
exports.xk = xk;
exports.ye = ye;
